/***************************************************************************
                          CdboCdrecordAction.cpp  -  description
                             -------------------
    begin                : Wed Aug 14 2002
    copyright            : (C) 2001 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.           

***************************************************************************/

#include <qregexp.h>

#include <kconfig.h>
#include <kprocess.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <krun.h>
#include <kdebug.h>

#include "CdboCdrecordAction.h"
#include "CdboCdrecordAction.moc"

#include "CdboMkisofsAction.h"

#include "CdboConstants.h"


CdboCreateDataOTFAction::CdboCreateDataOTFAction( QObject* d, const char* name) :
	CdboCdrecordAction(d, name){
}
//---------------------------------------------------------------------------------------------
CdboCreateDataOTFAction::~CdboCreateDataOTFAction(){
	delete mkisoAction;
}
//---------------------------------------------------------------------------------------------
void CdboCreateDataOTFAction::initProcess(KProcess *proc, CdboParamsDict* params){

	mkisoAction = new CdboMkisofsAction(parent(), "CdboMkisofsAction");
	readyToLaunch = mkisoAction->initProcessForPiping(proc, params, this);
	if(!readyToLaunch){
		processFailed();
		return;
	}
	CdboCdrecordAction::initProcess(proc, params);
}
//---------------------------------------------------------------------------------------------
bool CdboCreateDataOTFAction::filterOutput(int &type, QString &message){
	if(message.find("Last chance to quit") != -1)
		return CdboCdrecordAction::filterOutput(type, message);
	
	bool ok = mkisoAction->filterOutput(type, message);
	if(ok)
		return CdboCdrecordAction::filterOutput(type, message);
	else
		return false;
}
//---------------------------------------------------------------------------------------------
void CdboCreateDataOTFAction::processLaunched(){
	if(actionType == "dummy")
		emitOutput(Cdbo::StatusOkMessage, i18n("Starting create CD on-the-fly process in test mode."));
	else
		emitOutput(Cdbo::StatusOkMessage, i18n("Starting create CD on-the-fly process."));
	CdboAction::processLaunched();
}
//---------------------------------------------------------------------------------------------
void CdboCreateDataOTFAction::processFailedToLaunch(){
	if(actionType == "dummy")
		emitOutput(Cdbo::ErrorMessage, i18n("Create CD on-the-fly process in test mode failed to start. Make sure you have enough privileges"));
	else
		emitOutput(Cdbo::ErrorMessage,
			i18n("Create CD on-the-fly process failed to start. Make sure you have enough privileges."));
	CdboAction::processFailedToLaunch();
}
//---------------------------------------------------------------------------------------------
void CdboCreateDataOTFAction::processSucceded(){
	if(actionType == "dummy")
		emitOutput(Cdbo::StatusOkMessage, i18n("Test create CD on-the-fly completed successfully."));
	else
		emitOutput(Cdbo::StatusOkMessage, i18n("Create CD on-the-fly completed successfully."));
	CdboAction::processSucceded();
}
//---------------------------------------------------------------------------------------------
void CdboCreateDataOTFAction::processFailed(){
	if(actionType == "dummy")
		emitOutput(Cdbo::StatusNotOkMessage, i18n("Failed to create CD on-the-fly in test mode."));
	else
		emitOutput(Cdbo::StatusNotOkMessage, i18n("Failed to create CD on-the-fly."));
	CdboAction::processFailed();
}
//////////////////////////////////////////////////////
//////////////////////////////////////////////////////
//////////////////////////////////////////////////////

CdboCdrecordAction::CdboCdrecordAction( QObject* d, const char* name) :
	CdboAction(d, name){
	
	instanceCount = 0;
}
//---------------------------------------------------------------------------------------------
CdboCdrecordAction::~CdboCdrecordAction(){

}
//---------------------------------------------------------------------------------------------
void CdboCdrecordAction::initProcess(KProcess *proc, CdboParamsDict* /*params*/){
	shownPerInfo = false;
	instanceCount++;
	tracksDone = 0;
	secCount = 100;
	imageSize = 0;
/////////////////////////////////////////////////////////////////////////////////////////////

	QString device = paramVal("TARGET_DEVICE");
	if(device.isEmpty())
		return;
		
	actionType = paramVal("ACTION_TYPE");
	if(actionType.isEmpty())
		return;

	QString speed = paramVal("SPEED");
	if(speed.isEmpty())
		return;

	QString image = paramVal("IMAGE_NAME");
	if(image.isEmpty())
		return;

	bool audio = false;
	boolParamVal("IS_AUDIO", audio, false);
	bool creatingNewCd = false;
	boolParamVal("CREATING_NEW_CD", creatingNewCd, false);
	
	
	bool creatingCdOnTheFly = false;
	boolParamVal("IS_ON_THE_FLY_CREATE_DATA", creatingCdOnTheFly, false);
	
	QString tsize = "";
	if(creatingCdOnTheFly)
		tsize = paramVal("ON-THE-FLY_TSIZE", false);
	
	if(audio){
		int result = KMessageBox::questionYesNo(0,
				i18n("Should this CD be finalized (fixated)? If you choose 'Yes' it will not be possible"
					" to add any more tracks to it. CD can be fixated later with Fixate Dialog."
					" Some players do not support playing unfixated CDs."),
				i18n("Finalize CD?"));
		switch(result) {
			case KMessageBox::No:
				leaveUnfixated = true;
				break;
			case KMessageBox::Yes:
				leaveUnfixated = false;
				break;
		}
	}
	else{
		leaveUnfixated = false;
		imageSize = 0;
		intParamVal("IMAGE_SIZE", imageSize, false);
	}
	sessionType = Cdbo::InvalidSession;
	if(!intParamVal("SESSION_TYPE", sessionType, false) ||
				(sessionType < Cdbo::FirstSession || sessionType > Cdbo::FinalSession))
		sessionType = Cdbo::InvalidSession;

	bool sameDriveCopy = false;
	boolParamVal("SAME_DRIVE_COPY", sameDriveCopy, false);
	if(sameDriveCopy)
		requestCDR(false, paramVal("SOURCE_DEVICE_NAME", false));

/////////////////////////////////////////////////////////////////////////////////////////////

	emitStatus(i18n("Writing..."));

	config->setGroup("Wizard Detected");
	QString exe = config->readEntry("Path to cdrecord", "cdrecord");
	KRun::shellQuote(exe);
	*proc << exe;

	config->setGroup("Default Settings");
	if(config->readNumEntry("Level Of Output", Cdbo::StandartOutput) == Cdbo::CustomOutput){
		switch(config->readNumEntry("Custom Level Of Output", 1)){
			case 0: *proc  << "-s"; break;
			case 1: *proc  << "-v"; break;
			case 2: *proc  << "-V"; break;
			case 3: *proc  << "-VV"; break;
		}
		if(config->readBoolEntry("Show Debug Info", false))
			*proc  << "-d";
	}
	else
		*proc  << "-v";

	config->setGroup("Recording Options");
	///// eject or not
	if(config->readBoolEntry("Eject Media At End", true))
		*proc  << "-eject";

	//////// speed
	*proc  << "speed=" + speed;

	//////// dummy
	if(actionType == "dummy"){
		emitStatus(i18n("Writing (Test Mode)..."));
		*proc  << "-dummy";
	}

	/////////////////////////////////////////////////////////////////+++++++++++
	config->setGroup("ISO Settings");
	bool ms = config->readEntry("Session Type", "Single") ==  "Multi-Session";
	if(ms){
		if(creatingNewCd &&
				sessionType != Cdbo::FinalSession &&
				sessionType != Cdbo::InvalidSession){
			*proc  << "-multi";
		}
		if(creatingCdOnTheFly)
			*proc  << "-waiti";
	}
	/////////////////////////////////////////////////////////////////+++++++++++
	config->setGroup("Recording Options");
	if(actionType.find("append") != -1){
		ms = true;
		if(sessionType == Cdbo::FirstSession || sessionType == Cdbo::MiddleSession)
			*proc  << "-multi";
		else if(sessionType == Cdbo::InvalidSession){
			int result = KMessageBox::questionYesNo(0,
					i18n("Will this be the final session on this CD?"),
					i18n("Final session?"));
			switch(result) {
				case KMessageBox::No:{
					sessionType = Cdbo::MiddleSession;
					*proc  << "-multi";
					break;
				}
				case KMessageBox::Yes:{
					sessionType = Cdbo::FinalSession;
					break;
				}
			}
		}
		if((actionType) == "dummy append"){
			emitStatus(i18n("Appending (Test Mode)..."));
			*proc  << "-dummy";
		}
		else
			emitStatus(i18n("Appending..."));
	}
	
	if(config->readBoolEntry("Overburn CD", false))
		*proc  << "-overburn";
	QString buff = config->readEntry("Media Catalog Number", "");
	if(!buff.isEmpty()){
		KRun::shellQuote(buff);
		*proc  << "mcn=" + buff;
	}
	
	if(config->readBoolEntry("Use Custom Options", false)){
		if(config->readBoolEntry("Disk At Once", false) && !ms)
			*proc  << "-dao";
		addCustomOpt(proc);
	}

	*proc  << "dev=" + device;
	
	
	if(leaveUnfixated)
		*proc  << "-nofix";

	if(audio)
		*proc  << "-audio -pad" ;
	else
		*proc  << "-data" ;
	
	if(!tsize.isEmpty())
		*proc  << "tsize=" + tsize;

	if(image == "-")
		*proc  << " - ";
	else if(image.left(1) == "'")
		*proc  << " " + image + " ";
	else{
		KRun::shellQuote(image);
		*proc  << image;
	}
}
//---------------------------------------------------------------------------------------------
void CdboCdrecordAction::addCustomOpt(KProcess *proc){
	config->setGroup("Recording Options");

	if(config->readBoolEntry("Burn Proof", false))
		*proc  << "driveropts=burnproof";
	
	int iBuff = config->readNumEntry("FIFO Size", 4);
	if(iBuff > 4)
		*proc  << "fs=" + QString::number(iBuff) + "m";
		
	if(config->readBoolEntry("Use Inf Files", false))
		*proc  << "-useinfo";
	
	iBuff = config->readNumEntry("Default Pregap", 0);
	*proc  << "defpregap=" + QString::number(iBuff);

	iBuff = config->readNumEntry("SCSI Time Out", 0);
	*proc  << "timeout=" + QString::number(iBuff);

	QString strBuff = config->readEntry("Driver Name", "");
	if(!strBuff.isEmpty()){
		KRun::shellQuote(strBuff);
		*proc  << "driver=" + strBuff;
	}
	strBuff = config->readEntry("Driver Options", "");
	if(!strBuff.isEmpty()){
		KRun::shellQuote(strBuff);
		*proc  << "driveropts=" + strBuff;
	}
}
//---------------------------------------------------------------------------------------------
bool CdboCdrecordAction::filterOutput(int &type, QString &message){
	QString buff;

	//if(message.find("seconds.") !=-1 && (message.find("Last") !=-1 ||  message.find("Operation") !=-1))
	//	type = Cdbo::StatusOkMessage;

	if(message.find("Re-load disk and hit") != -1){
		requestReload();
		return false;
	}

	if(secCount < 9){
		//tere is always some garbich in front >{
		message = message.remove(0,message.find(QRegExp("[0-9]{1,1}")));
		if(actionType == "dummy")
			buff =  i18n("Last chance to quit, starting dummy write in %1").arg(message);
		else
			buff =  i18n("Last chance to quit, starting real write in %1").arg(message);
		
		type = Cdbo::OverwritePrevious;
		emitOutput(type, buff);
		secCount++;
		return false;
	}
	if(message.find("Last chance to quit") != -1 && secCount == 100){
		type = Cdbo::StatusOkMessage;
		secCount = 0;
	}

	
	if(message.find(i18n("should be same as in message. Used as a way to communicate",
							"completed successfully")) !=-1){
		emitPercent("CdboCdrecordAction" + QString::number(instanceCount), 100);
	}
	
	if(message.find("Starting new") !=-1){
		tracksDone++;
		if(tracksDone < 10)
			buff = "0" + QString::number(tracksDone);
		else
			buff = QString::number(tracksDone);

		emitPercentInfo("CdboCdrecordAction" + QString::number(instanceCount), i18n("Burning Track %1, percent done:").arg(buff) );
		return false;
	}

	
	if(message.find("MB written") !=-1){

		buff =  (message.right(message.length() - message.find(":") - 1)).stripWhiteSpace();
		//message =  (message.right(message.length() - message.find(":") - 1)).stripWhiteSpace();
		QStringList l = QStringList::split(" ", buff);
		float i = l[0].toFloat();
		float y = l[2].toFloat();
		if(y == 0)
			y = imageSize;

		i = i / (y / 100);
		if(i > 100 || y == 0)
			i = 100;

		if(!shownPerInfo){
			message = message.left(message.find(":")).stripWhiteSpace();//seems broken
			emitPercentInfo("CdboCdrecordAction" + QString::number(instanceCount), i18n("Burning Track %1, percent done:").arg(message) );
			tracksDone = message.section(" ",1,1).stripWhiteSpace().toInt();
			shownPerInfo = true;
		}

		emitPercent("CdboCdrecordAction" + QString::number(instanceCount), (int)i, ": " + buff.simplifyWhiteSpace());
		return false;
	}
	if(message.startsWith("WARNING:") || message.find("read/written:") != -1){
		emitOutput(Cdbo::ProcessMessage, "dummy");
		type = Cdbo::OverwritePrevious;
	}

	return true;
}
//---------------------------------------------------------------------------------------------
void CdboCdrecordAction::processLaunched(){
	if(actionType == "dummy")
		emitOutput(Cdbo::StatusOkMessage, i18n("Starting burn process in test mode."));
	else
		emitOutput(Cdbo::StatusOkMessage, i18n("Starting burn process."));
	CdboAction::processLaunched();
}
//---------------------------------------------------------------------------------------------
void CdboCdrecordAction::processFailedToLaunch(){
	leaveUnfixated = false;
	if(actionType == "dummy")
		emitOutput(Cdbo::ErrorMessage, i18n("Test failed to start. Make sure you have enough privileges."));
	else
		emitOutput(Cdbo::ErrorMessage, i18n("Burn process failed to start. Make sure you have enough privileges."));
	CdboAction::processFailedToLaunch();
}
//---------------------------------------------------------------------------------------------
void CdboCdrecordAction::processSucceded(){
	if(actionType == "dummy")
		emitOutput(Cdbo::StatusOkMessage, i18n("Test completed successfully."));
	else
		emitOutput(Cdbo::StatusOkMessage, i18n("Burn completed successfully."));
	if(leaveUnfixated){
		leaveUnfixated = false;
		emitOutput(Cdbo::InfoMessage, i18n("This CD was not fixated."));
	}
		
	CdboAction::processSucceded();
}
//---------------------------------------------------------------------------------------------
void CdboCdrecordAction::processFailed(){
	leaveUnfixated = false;
	
	if(actionType == "dummy")
		emitOutput(Cdbo::StatusNotOkMessage, i18n("Test burn failed."));
	else
		emitOutput(Cdbo::StatusNotOkMessage, i18n("Burn failed."));
	CdboAction::processFailed();
	
}
//---------------------------------------------------------------------------------------------
void CdboCdrecordAction::resetSettings(){
	CdboAction::resetSettings();
	sessionType = Cdbo::InvalidSession;
}

