/***************************************************************************
                          CdboCreateAudioCdAction.cpp  -  description
                             -------------------
    begin                : Sat Sep 14 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <qtimer.h>
#include <qfile.h>

#include <kconfig.h>
#include <kprocess.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kmimetype.h>
#include <kdebug.h>
#include <krun.h>

#include "CdboCreateAudioCdAction.h"
#include "CdboCreateAudioCdAction.moc"

#include "CdboConstants.h"

CdboCreateAudioCdAction::CdboCreateAudioCdAction(KLibFactory* factory, QObject* d, const char* name) :
	CdboActionHandler(factory, d, name){

}
//---------------------------------------------------------------------------------------------
CdboCreateAudioCdAction::~CdboCreateAudioCdAction(){

}
//---------------------------------------------------------------------------------------------
void CdboCreateAudioCdAction::initProcess(KProcess *, CdboParamsDict* params){
	readyToLaunch = false;
	this->params = params;
	newTocName = "";
/////////////////////////////////////////////////////////////////////////////////////////////
	bool multyStepAudio = false;
	boolParamVal("IS_MULTY_STEP_AUDIO", multyStepAudio, false);
	
	
	QString tocName = paramVal("TOC_NAME", !multyStepAudio);
	if(tocName.isEmpty() && !multyStepAudio)
		return;
	
	QStringList fileList;
	if(!listParamVal("FILE_LIST", fileList, "\n", multyStepAudio) && multyStepAudio)
		return;

	if(fileList.count() > 0){
		QString tmpDir = paramVal("TMP_DIRECTORY");
		if(tmpDir.isEmpty())
			return;
		
		QString newName, mType; 
		QString baseName = tmpDir + "cdbo_audio_" + timeStamp() + "_";
		
		for(uint i = 0; i < fileList.count(); i++){
			KMimeType::Ptr mimetype = KMimeType::findByPath(fileList[i]);
			mType = mimetype->name();
			if(mType.find("x-mp3") != -1){
				mp3ToConvert.append(fileList[i]);
				newName = baseName + QString::number(i) + ".wav";
				mp3Converted.append(newName);
				finalNames.append(newName);
			}
			else if(mType.find("x-ogg") != -1){
				oggToConvert.append(fileList[i]);
				newName = baseName + QString::number(i) + ".wav";
				oggConverted.append(newName);
				finalNames.append(newName);
			}
			else
				finalNames.append(fileList[i]);
		}
		
		if(requireDecompress() && !multyStepAudio){
			if(!fixToc(tocName, baseName))
				return;
		}
	}

	if(!multyStepAudio){
		recordFlag = "write";
		setParamVal("CDRDAO_MODE", &recordFlag);
	}
	else{
		recordFlag = "TRUE";
		setParamVal("IS_AUDIO", &recordFlag);
		QString buff;
		for(uint i = 0; i < finalNames.count(); i++){
			buff = finalNames[i];
			KRun::shellQuote(buff);
			fileListHolder += buff + " ";
		}
		setParamVal("IMAGE_NAME", &fileListHolder);
	}

	//kdDebug() << "CdboCreateAudioCdAction:: got this many oggToConvert: " << oggToConvert.count() << endl;
	//kdDebug() << "CdboCreateAudioCdAction:: got this many mp3ToConvert: " << mp3ToConvert.count() << endl;
/////////////////////////////////////////////////////////////////////////////////////////////

	config->setGroup("Default Settings");
	if(config->readBoolEntry("Check Tmp Size", true)){
		if(!createAction("CdboCheckTmpAction")){
			processFailed();
			return;
		}
	}
	
	config->setGroup("Recording Options");
	if(config->readBoolEntry("Check Drive First", false)){
		if(!createAction("CdboCheckDriveAction")){
			processFailed();
			return;
		}
	}
	if(oggToConvert.count() > 0){
		if(!createAction("CdboOgg123Action")){
			processFailed();
			return;
		}
	}
	if(mp3ToConvert.count() > 0){
		if(!createAction("CdboMpg123Action")){
			processFailed();
			return;
		}
	}

	if(multyStepAudio){
		if(!createAction("CdboCdrecordAction")){
			processFailed();
			return;
		}
	}
	else{
		if(!createAction("CdboCdrdaoAction")){
			processFailed();
			return;
		}
	}

	QTimer::singleShot(1, this, SLOT(begin()));
}
//---------------------------------------------------------------------------------------------
bool CdboCreateAudioCdAction::requireDecompress(){
	if(oggToConvert.count() == 0 && mp3ToConvert.count() == 0)
		return false;

	if(oggToConvert.count() > 0){
		oggHolder = oggToConvert.join("\n");
		oggConvertedHolder = oggConverted.join("\n");
		setParamVal("OGG_FILES_TO_CONVERT", &oggHolder);
		setParamVal("OGG_CONVERTED_NAMES", &oggConvertedHolder);
	}
	if(mp3ToConvert.count() > 0){
		mp3Holder = mp3ToConvert.join("\n");
		mp3ConvertedHolder = mp3Converted.join("\n");
		setParamVal("MP3_FILES_TO_CONVERT", &mp3Holder);
		setParamVal("MP3_CONVERTED_NAMES", &mp3ConvertedHolder);
	}
	oggCounter = "1";
	mp3Counter = QString::number(oggToConvert.count());
	totalToProcess = QString::number(mp3ToConvert.count() + oggToConvert.count());
	
	setParamVal("OGG_START_COUNTER", &oggCounter);
	setParamVal("MP3_START_COUNTER", &mp3Counter);
	setParamVal("TOTAL_TO_CONVERT", &totalToProcess);
	return true;
}
//---------------------------------------------------------------------------------------------
bool CdboCreateAudioCdAction::fixToc(const QString &tocName, const QString &baseName){
	newTocName = baseName + ".toc";
	QFile toc(tocName);
	QFile newToc(newTocName);

	if(!toc.open(IO_ReadOnly)) {
		emitOutput(Cdbo::ErrorMessage,  i18n("Failed to create tracks layout description file."));
		processFailedToLaunch();
		return false;
	}
	if(!newToc.open(IO_WriteOnly)) {
		emitOutput(Cdbo::ErrorMessage,  i18n("Failed to create tracks layout description file."));
		processFailedToLaunch();
		toc.close();
		return false;
	}
	QString line;
	bool checkMp3 = mp3ToConvert.count() > 0;
	bool checkOgg = oggToConvert.count() > 0;
	bool gotIt = false;

	QTextStream inStream( &toc ), outStream( &newToc );
	while ( !inStream.eof() ) {
		line = inStream.readLine();
		if(!line.startsWith("FILE"))
			gotIt = true;

		if(checkOgg && !gotIt){
			for(uint i = 0; i < oggToConvert.count(); i++){
				if(line.contains(oggToConvert[i], true)){
					line = line.right(line.length() - line.findRev("\"") - 1);
					line = "FILE \"" + oggConverted[i] + "\" " + line;
					gotIt = true;
					break;
				}
			}
		}
		if(checkMp3 && !gotIt){
			for(uint i = 0; i < mp3ToConvert.count(); i++){
				if(line.contains(mp3ToConvert[i], true)){
					line = line.right(line.length() - line.findRev("\"") - 1);
					line = "FILE \"" + mp3Converted[i] + "\" " + line;
					gotIt = true;
					break;
				}
			}
		}

		outStream << line + "\n";
		gotIt = false;
	}

	toc.close();
	newToc.close();
	setParamVal("TOC_NAME", &newTocName);
	return true;
}
//---------------------------------------------------------------------------------------------
void CdboCreateAudioCdAction::processFailed(){
	emitOutput(Cdbo::StatusNotOkMessage, i18n("Failed to create Audio CD."));
	CdboAction::processFailed();
}
//---------------------------------------------------------------------------------------------
void CdboCreateAudioCdAction::processSucceded(){
	emitOutput(Cdbo::StatusOkMessage, i18n("Audio CD created successfully."));
	CdboAction::processSucceded();
}
//---------------------------------------------------------------------------------------------
void CdboCreateAudioCdAction::resetSettings(){
	CdboActionHandler::resetSettings();
	if(oggConverted.count() > 0){
		for(uint i = 0; i < oggConverted.count(); i++)
			removeOneFile(oggConverted[i], false);
	}
	if(mp3Converted.count() > 0){
		for(uint i = 0; i < mp3Converted.count(); i++)
			removeOneFile(mp3Converted[i], false);
	}
	if(!newTocName.isEmpty())
		removeOneFile(newTocName, false);
}
