/***************************************************************************
                          CdboReadDataCdAction.cpp  -  description
                             -------------------
    begin                : Fri Aug 16 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <qregexp.h>

#include <kconfig.h>
#include <kprocess.h>
#include <klocale.h>
#include <krun.h>
#include <kdebug.h>

#include "CdboReadDataCdAction.h"
#include "CdboReadDataCdAction.moc"
#include "CdboConstants.h"

CdboReadDataCdAction::CdboReadDataCdAction( QObject* d, const char* name) :
	CdboAction(d, name){

}
//---------------------------------------------------------------------------------------------
CdboReadDataCdAction::~CdboReadDataCdAction(){

}
//---------------------------------------------------------------------------------------------
void CdboReadDataCdAction::initProcess(KProcess *proc, CdboParamsDict* /*params*/){
	plCount = -1;
	showedFixingErrorMess = false;
/////////////////////////////////////////////////////////////////////////////////////////////

	QString device = paramVal("SOURCE_DEVICE");
	if(device.isEmpty())
		return;

	QString isoImageName = paramVal("IMAGE_NAME");
	if(isoImageName.isEmpty())
		return;

	bool scsi = false;
	boolParamVal("IS_SCSI_SOURCE", scsi, false);
		

/////////////////////////////////////////////////////////////////////////////////////////////

	if(scsi){
		bool noError = false;
		boolParamVal("IS_NO_ERROR_ENABLED", noError, false);
		initReadcdProcess(proc, device, isoImageName, noError);
	}
	else
		initDdProcess(proc, device, isoImageName);
}
//---------------------------------------------------------------------------------------------
void CdboReadDataCdAction::initDdProcess(KProcess *proc, QString &device, QString &isoImageName){
	usingReadcd = false;
	*proc << "dd";
	*proc << "if=" + device;
	KRun::shellQuote(isoImageName);
	*proc << "of=" + isoImageName;
}
//---------------------------------------------------------------------------------------------
void CdboReadDataCdAction::initReadcdProcess(KProcess *proc, QString &device, QString &isoImageName, bool noError){
	usingReadcd = true;
	config->setGroup("Wizard Detected");
	QString exe = config->readEntry("Path to readcd", "readcd");
	KRun::shellQuote(exe);
	*proc << exe;

	config->setGroup("Default Settings");
	if(config->readNumEntry("Level Of Output", Cdbo::StandartOutput) == Cdbo::CustomOutput){
		switch(config->readNumEntry("Custom Level Of Output", 1)){
			case 0: *proc  << "-q"; break;
			case 1: break;
			case 2: *proc  << "-v"; break;
			case 3: *proc  << "-V"; break;
		}
		if(config->readBoolEntry("Show Debug Info", false))
			*proc  << "-d";
	}
	
	if(noError)
		*proc << "-noerror";

	*proc  << "dev=" + device;
	KRun::shellQuote(isoImageName);
	*proc << "f=" + isoImageName;

}
//---------------------------------------------------------------------------------------------
bool CdboReadDataCdAction::filterOutput(int &type, QString &message){

	if(message.startsWith("addr:")){
		if(totalToDo == 0){
			emitPercent("CdboReadDataCdAction", 100, i18n(", %1 of %2 done").arg(QString::number(totalToDo)).arg(QString::number(totalToDo)));
			return false;
		}
		message = message.right(message.length() - 5).stripWhiteSpace();
		message = message.left(message.find(" "));
		float done = message.toInt();

		done = done / (totalToDo / 100);
		if(done > 100)
			done = 100;

		emitPercent("CdboReadDataCdAction", (int)done,
			i18n(", %1 of %2 done").arg(message).arg(QString::number(totalToDo)));
		return false;
	}

	if(message.startsWith("end:")){
		message = message.right(message.length() - 4).stripWhiteSpace();
		totalToDo = message.toInt();

		emitPercent("CdboReadDataCdAction", 0);
		emitPercentInfo("CdboReadDataCdAction", i18n("Reading CD, percent done:"));
		return false;
	}

	QRegExp ex("^[\.]+$");
	if(ex.exactMatch(message)){
		if(!showedFixingErrorMess){
			showedFixingErrorMess = true;
			emitOutput(Cdbo::StatusOkMessage, i18n("Trying to correct this error. Please wait..."));
		}
		return false;
	}
	
	if(message == "~" || message == "+" || message == "-"){
		if(plCount == -1){
			plCount = 0;
			prevMessage = "";
			type = Cdbo::ProcessMessage;
		}
		else
			type = Cdbo::OverwritePrevious;
		
		if(plCount == 7){
			plCount = 0;
			prevMessage = "";
		}
		
		if(message == "+")
			plCount++;

		prevMessage += message;
		message = prevMessage;
	}
	else
		plCount = -1;
		
	return true;
}
//---------------------------------------------------------------------------------------------
void CdboReadDataCdAction::processLaunched(){
	emitOutput(Cdbo::StatusOkMessage, i18n("Starting read process."));
	emitStatus(i18n("Reading data CD..."));
	CdboAction::processLaunched();
}
//---------------------------------------------------------------------------------------------
void CdboReadDataCdAction::processFailedToLaunch(){
	emitOutput(Cdbo::ErrorMessage,
		i18n("Read process failed to start. Make sure you have enough privileges."));
	CdboAction::processFailedToLaunch();
}
//---------------------------------------------------------------------------------------------
void CdboReadDataCdAction::processSucceded(){
	if(usingReadcd)
			emitPercent("CdboReadDataCdAction", 100, i18n(", %1 of %2 done").arg(QString::number(totalToDo)).arg(QString::number(totalToDo)));
	emitOutput(Cdbo::StatusOkMessage, i18n("Read completed successfully."));
	CdboAction::processSucceded();
}
//---------------------------------------------------------------------------------------------
void CdboReadDataCdAction::processFailed(){
	emitOutput(Cdbo::StatusNotOkMessage, i18n("Read of data CD failed."));
	CdboAction::processFailed();
}
