/***************************************************************************
                          CdboAudioCDBase.cpp  -  description
                             -------------------
    begin                : Fri Apr 5 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include "CdboAudioCDBase.h"
#include "CdboAudioCDBase.moc"

#include <qvariant.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qframe.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qspinbox.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qimage.h>
#include <qpixmap.h>


#include "CdboDriveInfoPnl.h"
#include "CdboSpeedPnl.h"
#include "CdboDialogCntPnl.h"


//
CdboAudioCDBase::CdboAudioCDBase( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : CdboDialogBase( parent, name, modal, fl )
{
    if ( !name )
	setName( "CdboAudioCDBase" );

	setCaption( i18n( "New Audio CD Dialog" ) );
	cdboAudioCDBaseLayout = new QGridLayout( this ); 
	cdboAudioCDBaseLayout->setSpacing( 0 );
	cdboAudioCDBaseLayout->setMargin( 0 );
	cdboAudioCDBaseLayout->setResizeMode(QLayout::Minimum);
	

	baseFrm = new QFrame( this, "baseFrm" );

	baseFrm->setMargin( 0 );
	baseFrm->setMidLineWidth( 0 );
	baseFrmLayout = new QVBoxLayout( baseFrm ); 
	baseFrmLayout->setSpacing( 6 );
	baseFrmLayout->setMargin( 11 );

	targetDevicePnl = new CdboDriveInfoPnl( baseFrm, "targetDevicePnl" );
	baseFrmLayout->addWidget( targetDevicePnl );

	speedPnl = new CdboSpeedPnl( baseFrm, "speedPnl" );
	baseFrmLayout->addWidget( speedPnl );

	cdboAudioCDBaseLayout->addWidget( baseFrm, 0, 0 );

	dialogCntPnl = new CdboDialogCntPnl( this, "dialogCntPnl" );

	cdboAudioCDBaseLayout->addMultiCellWidget( dialogCntPnl, 1, 1, 0, 1 );
	
	
	dialogCntPnl = new CdboDialogCntPnl( this, "dialogCntPnl" );

	cdboAudioCDBaseLayout->addMultiCellWidget( dialogCntPnl, 1, 1, 0, 1 );

	detailsPnl = new QFrame( this, "detailsPnl" );
	detailsPnlLayout = new QVBoxLayout( detailsPnl );
	detailsPnlLayout->setSpacing( 6 );
	detailsPnlLayout->setMargin( 11 );

	recordDetailesPnl = new QGroupBox( detailsPnl, "recordDetailesPnl" );
	recordDetailesPnl->setTitle( i18n( "Recording Details" ) );
	QWhatsThis::add(  recordDetailesPnl, i18n( "Recording Details." ) );

	recordDetailesPnl->setColumnLayout(0, Qt::Vertical );
	recordDetailesPnl->layout()->setSpacing( 0 );
	recordDetailesPnl->layout()->setMargin( 0 );
	recordDetailesPnl->layout()->setResizeMode(QLayout::Minimum);

	recordDetailesPnlLayout = new QHBoxLayout( recordDetailesPnl->layout() );
	recordDetailesPnlLayout->setAlignment( Qt::AlignTop );
	recordDetailesPnlLayout->setResizeMode(QLayout::Minimum);
	
	recordDetailesPnlLayout->setMargin( KDialog::marginHint() );

	actionCmb = new QComboBox( FALSE, recordDetailesPnl, "actionCmb" );
	actionCmb->insertItem( i18n( "Test Add to Track-at-Once CD" ));
   
	QToolTip::add( actionCmb, i18n( "Select the action to take. <b></b>Use 'What's This' to get description about each available Mode.") );
	recordDetailesPnlLayout->addWidget( actionCmb );

	QSpacerItem* spacer_1 = new QSpacerItem( 20, 20, QSizePolicy::Fixed, QSizePolicy::Minimum );
	recordDetailesPnlLayout->addItem( spacer_1 );
	
	QSpacerItem* spacer_3 = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
	recordDetailesPnlLayout->addItem( spacer_3 );
	
	numOfCopiesLbl = new QLabel( recordDetailesPnl, "numOfCopiesLbl" );
	numOfCopiesLbl->setText( i18n( "&Number of copies: " ) );
	numOfCopiesLbl->setAlignment( int( QLabel::WordBreak | QLabel::AlignCenter ) );
	numOfCopiesLbl->setIndent( -1 );
	recordDetailesPnlLayout->addWidget( numOfCopiesLbl );


	numberSpn = new QSpinBox( recordDetailesPnl, "numberSpn" );
	numberSpn->setMinValue( 1 );
	numberSpn->setValue( 1 );
	QToolTip::add(  numberSpn, i18n( "Specify number of copies to make" ) );
	recordDetailesPnlLayout->addWidget( numberSpn );

	
	 detailsPnlLayout->addWidget(recordDetailesPnl, 4);
	 
	audioOptPnl = new QGroupBox( detailsPnl, "audioOptPnl" );
	audioOptPnl->setTitle( i18n( "Audio CD Details (CD-TEXT)" ) );
	audioOptPnl->setColumnLayout(0, Qt::Vertical );
	audioOptPnl->layout()->setSpacing( 6 );
	audioOptPnl->layout()->setMargin( 11 );
	audioOptPnlLayout = new QGridLayout( audioOptPnl->layout() );
	audioOptPnlLayout->setAlignment( Qt::AlignTop );

	titleTxt = new QLineEdit( audioOptPnl, "titleTxt" );
	QToolTip::add( titleTxt, i18n( "Specify the album title for the new Audio CD." ) );

	audioOptPnlLayout->addMultiCellWidget( titleTxt, 0, 0, 1, 3 );

	perfomLbl = new QLabel( audioOptPnl, "perfomLbl" );
	perfomLbl->setText( i18n( "&Artist:" ) );

	audioOptPnlLayout->addWidget( perfomLbl, 1, 0 );

	titleLbl = new QLabel( audioOptPnl, "titleLbl" );
	titleLbl->setText( i18n( "A&lbum:" ) );

	audioOptPnlLayout->addWidget( titleLbl, 0, 0 );

	perfomTxt = new QLineEdit( audioOptPnl, "perfomTxt" );
	QToolTip::add( perfomTxt, i18n( "Specify the artist's name for the new Audio CD." ) );

	audioOptPnlLayout->addMultiCellWidget( perfomTxt, 1, 1, 1, 3 );
	uploadBtn = new QPushButton( audioOptPnl, "uploadBtn" );
	uploadBtn->setEnabled( FALSE );
	uploadBtn->setText( i18n( "Upload" ) );
	QToolTip::add( uploadBtn, i18n( "Submit information about this CD to the CDDB server via SMTP. <b></b>This option will be available as soon as KDE's CDDB lib is functional." ) );

	audioOptPnlLayout->addWidget( uploadBtn, 2, 3 );

	launchTrackEditorBtn = new QPushButton( audioOptPnl, "uploadBtn" );
	launchTrackEditorBtn->setText( i18n( "&Tracks Editor" ) );
	QToolTip::add( launchTrackEditorBtn, i18n( "Use Tracks Editor to manipulate new CD layout (such as change Tracks length or break Tracks in part) <b></b>and add Tracks information." ) );

	audioOptPnlLayout->addMultiCellWidget( launchTrackEditorBtn, 2, 2, 0, 1 );
	QSpacerItem* spacer_2 = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
	audioOptPnlLayout->addItem( spacer_2, 2, 2 );
	
	
	detailsPnlLayout->addWidget(audioOptPnl, 6);

	cdboAudioCDBaseLayout->addWidget( detailsPnl, 0, 1 );
	cdboAudioCDBaseLayout->setColStretch(0, 4);
	cdboAudioCDBaseLayout->setColStretch(1, 6);

	cdboAudioCDBaseLayout->setRowStretch(1, 1);

    numOfCopiesLbl->setBuddy( numberSpn );
	titleLbl->setBuddy( titleTxt );
	perfomLbl->setBuddy( perfomTxt );
}

/*
 *  Destroys the object and frees any allocated resources
 */
CdboAudioCDBase::~CdboAudioCDBase()
{
    // no need to delete child widgets, Qt does it all for us
}


