/***************************************************************************
                          CdboAudioCDDialog.cpp  -  description
                             -------------------
    begin                : Fri Apr 5 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <qcombobox.h>
#include <qtooltip.h>
#include <qlabel.h>
#include <qspinbox.h>
#include <qdir.h>
#include <qdatetime.h>
#include <qlineedit.h>
#include <qtimer.h>
#include <qwhatsthis.h>

#include <kiconloader.h>
#include <kfiledialog.h>
#include <klocale.h>
#include <kconfig.h>
#include <kdebug.h>
#include <kmessagebox.h>
#include <kstdguiitem.h>

#include "CdboAudioCDDialog.h"
#include "CdboAudioCDDialog.moc"

#include "CdboEraseCDDialog.h"
#include "CdboTracksEditor.h"

//---------------------------------------------------------------------------------------------
CdboAudioCDDialog::CdboAudioCDDialog(QWidget *parent, const char *name) :
	CdboAudioCDBase(parent,name,true),
	DCOPObject( "CdboDialogBase" ){

	init();
}
//---------------------------------------------------------------------------------------------
CdboAudioCDDialog::CdboAudioCDDialog(QStringList *flist, QWidget *parent, const char *name) :
	CdboAudioCDBase(parent,name,true),
	DCOPObject( "CdboDialogBase" ){

	fileList = flist->join("\n");

	init();
	dialogCntPnl->output(Cdbo::InfoMessage, i18n("1 track on this list", "%n tracks on this list", flist->count()));
}
//---------------------------------------------------------------------------------------------
CdboAudioCDDialog::~CdboAudioCDDialog(){
	cleanupTmpToc();
}
//---------------------------------------------------------------------------------------------
void CdboAudioCDDialog::init(){
	CdboDialogBase::init();
	
	tocName = "";
	newToc = false;
	haveToc = false;
	lastTitle = "";
	lastArtist = "";
	updateTitles();
	
	
	titleTxt->setMaxLength(45);
	perfomTxt->setMaxLength(45);

	///////////////////////////////////////////////////
	targetDevicePnl->setType(CdboDriveInfoPnl::Target);
	targetDevicePnl->setUsrBtnTitle( i18n("&Erase"));
	/////////////////////////////////////////////////
	dialogCntPnl->setStartBtnTitle( i18n("C&reate"));
	QToolTip::add(dialogCntPnl->getStartBtn(), i18n( "Create" ) );
	/////////////////////////////////////////////////
	readOptions();
	targetDevicePnl->closeTray();

	
	connect( launchTrackEditorBtn, SIGNAL( clicked() ), this, SLOT( launchTrackEditor() ) );
	
	// connecting to the Drive Info Panel
	connect( targetDevicePnl->getDetectBtn(), SIGNAL( clicked() ), this, SLOT( detectSlot() ) );
	connect( targetDevicePnl->getUsrBtn(), SIGNAL( clicked() ), this, SLOT( eraseSlot() ) );

	// connecting to the Dialog Control Panel
	connect( dialogCntPnl->getStartBtn(), SIGNAL( clicked() ), this, SLOT( startBtnSlot() ) );
	connect( dialogCntPnl->getCancelBtn(), SIGNAL( clicked() ), this, SLOT( cancelSlot() ) );
	connect( dialogCntPnl->getSettingsBtn(), SIGNAL( clicked() ), this, SLOT( settingsSlot() ) );
	connect( dialogCntPnl->getQuitBtn(), SIGNAL( clicked() ), this, SLOT( quitSlot() ) );
	
	connect( actionCmb, SIGNAL( activated(int) ), this, SLOT( verifyMode(int) ) );
	
	settingTypes = "cdboburnopt cdbodefaultopt";
}
//---------------------------------------------------------------------------------------------
void CdboAudioCDDialog::saveOptions(){
	if(!config)
		return;
	//////////////////////////////////////////////////////////
	config->setGroup("Audio Create Options");
	config->writeEntry("Number of Copies", numberSpn->value());
	if(!fileList.isEmpty())
		config->writeEntry("Type of Audio Create0", actionCmb->currentItem());
	else
		config->writeEntry("Type of Audio Create1", actionCmb->currentItem());
	
	speedPnl->saveOptions();
 	targetDevicePnl->saveOptions();
	config->sync();
}
//---------------------------------------------------------------------------------------------
void CdboAudioCDDialog::readOptions(){
	saveOptions();
	CdboDialogBase::readOptions();
     ///////////////////////////////////////////////////////////////////
	
	config->setGroup("Default Settings");
	tempDir = config->readEntry("Tmp Dir", "/tmp/");
	minTmpSize = config->readEntry("Minimum Tmp Size", "650");
	
	config->setGroup("Audio Create Options");
	numberSpn->setValue(config->readNumEntry("Number of Copies", 0));

	speedPnl->readOptions();
	targetDevicePnl->readOptions();
	initBtns();
}
//---------------------------------------------------------------------------------------------
void CdboAudioCDDialog::initBtns(){
	if(targetDevicePnl->isReady()){
		blockBtns(false);
		dialogCntPnl->getStartBtn()->setEnabled(true);

		blockActionBtns(false);

		dialogCntPnl->status(i18n("Ready"));
	}
	else{
		blockBtns(true);
		dialogCntPnl->getCancelBtn()->setEnabled(false);
		dialogCntPnl->status(i18n("No recording device." ));
		return;
	}
	
	
	if(!haveToc && fileList.isEmpty()){
		dialogCntPnl->getStartBtn()->setEnabled(false);
		launchTrackEditorBtn->setEnabled(false);
		dialogCntPnl->output(Cdbo::StatusNotOkMessage,
				i18n("No files to burn. Can not proceed."));
	}
	else{
		launchTrackEditorBtn->setEnabled(true);
	}
	restructureCreateOptions();
}
//---------------------------------------------------------------------------------------------
void CdboAudioCDDialog::blockActionBtns(bool yes){
	numberSpn->setEnabled(!yes);
	numOfCopiesLbl->setEnabled(!yes);
}
//----------------------------------------------------------------------------------------------
void CdboAudioCDDialog::updateTitles(){
	titleTxt->setText(lastTitle);
	perfomTxt->setText(lastArtist);
}
//----------------------------------------------------------------------------------------------
void CdboAudioCDDialog::blockBtns(bool yes){
	dialogCntPnl->getStartBtn()->setEnabled(!yes);
	dialogCntPnl->getCancelBtn()->setEnabled(yes);
	speedPnl->setEnabled(!yes);
	detailsPnl->setEnabled(!yes);
	targetDevicePnl->blockBtns(yes);
	if(doingSomething()){
		targetDevicePnl->getDetectBtn()->setEnabled(false);
		dialogCntPnl->getSettingsBtn()->setEnabled(false);
	}
	else{
		targetDevicePnl->getDetectBtn()->setEnabled(true);
		dialogCntPnl->getSettingsBtn()->setEnabled(true);
	}
}
//----------------------------------------------------------------------------------------------
void CdboAudioCDDialog::restructureCreateOptions(){
	actionCmb->clear();
	QWhatsThis::remove(actionCmb);
	QString whatsThis = i18n("Choose how and what type of CD should be created.");
	actionCmb->insertItem( i18n( "Create CD" ) );
	actionCmb->insertItem( i18n( "Test Create CD" ) );
	whatsThis += i18n("<br><br><b>Create CD:</b> Will create Audio CD in Disk-At-Once Mode. This mode allows to add Tracks informaion as CD Text and "\
		"manipulate CD layout. Such CD is created in one shot, it will not be possible to add Tracks at a later time.");
	if(!fileList.isEmpty()){
		actionCmb->insertItem( i18n( "Add to Track-at-Once CD" ), 1);
		actionCmb->insertItem( i18n( "Test Add to Track-at-Once CD" ), 3);
		whatsThis +=  i18n("<br><br><b>Add to Track-at-Once CD:</b> Will append tracks to an empty CD-R(W) or an existing Audio CD in Track-at-Once Mode. "\
			"If not finalized, it will be possible to add Tracks to such CD at a later time (provided there is enough space left). "\
			"However, CD Text can not be added and CD layout can not be manipulated. "\
			"To append to an existing Audio CD it must itself be in Track-at-Once Mode and not finalized (fixated).");
		actionCmb->setCurrentItem( config->readNumEntry("Type of Audio Create0", 0));
	}
	else
		actionCmb->setCurrentItem( config->readNumEntry("Type of Audio Create1", 0));
	

	whatsThis += i18n("<br><br><b>Test ... :</b> Same as coresponding real Mode (one that does not start with test), "\
		"but laser will stay cool - nothing will be actually burned. Allows testing without possibility of corrupting recordable media.");
	QWhatsThis::add(actionCmb, whatsThis);
	verifyMode(actionCmb->currentItem());
}
//----------------------------------------------------------------------------------------------
void CdboAudioCDDialog::verifyMode(int actionType){
	if(!targetDevicePnl->isReady() || doingSomething())
		return;
	if(!haveToc && fileList.isEmpty())
		return;

	if((actionType == 1 || actionType == 3) && !fileList.isEmpty())
		launchTrackEditorBtn->setEnabled(false);
	else
		launchTrackEditorBtn->setEnabled(true);
}
//---------------------------------------------------------------------------------------------
QString* CdboAudioCDDialog::actionType(){
	int pos = actionCmb->currentItem();
	
	if((!fileList.isEmpty() && pos > 1) || (fileList.isEmpty() && pos == 1))
		mActionType = "dummy";
	else
		mActionType = "burn";
	return &mActionType;
}
//---------------------------------------------------------------------------------------------
void CdboAudioCDDialog::eraseSlot(){
	saveOptions();
	hide();
	CdboEraseCDDialog *ed = new CdboEraseCDDialog((QWidget *)parent(),"eraseD");
	ed->exec();
	delete ed;
	readOptions();
	exec();
}
//---------------------------------------------------------------------------------------------
void CdboAudioCDDialog::cleanupTmpToc(){
	if(newToc && !tocName.isEmpty()){
		QFile file(tocName);
		file.remove();
		newToc = false;
	}
}
//---------------------------------------------------------------------------------------------
void CdboAudioCDDialog::setTrackList(QString tracksList, QString delim){
	cleanupTmpToc();
	clearOutput();
	
	QStringList list = QStringList::split(delim , tracksList, false);

	dialogCntPnl->output(Cdbo::InfoMessage, i18n("1 track on this list", "%n tracks on this list", 
		list.count()));

	fileList = list.join("\n");
	haveToc = false;
	tocName = "";
	lastTitle = "";
	lastArtist = "";
	updateTitles();
	initBtns();
}
//=====================================
void CdboAudioCDDialog::setTocFile(const QString &tocFile){
	cleanupTmpToc();
	clearOutput();
	fileList = "";
	lastTitle = "";
	lastArtist = "";
	
	CdboTracksEditor *cte = new CdboTracksEditor(this, "cte");
	bool valid = cte->isValidToc(tocFile, lastTitle, lastArtist);
	delete cte;
	if(!valid){
		tocName = "";
		haveToc = false;
	}
	else{
		tocName = tocFile;
		haveToc = true;
	}
	updateTitles();
	initBtns();
	QTimer::singleShot(1, this, SLOT(prepareTocCopy()));
}
//=====================================
void CdboAudioCDDialog::prepareTocCopy(){
	launchTrackEditorBtn->setEnabled(false);
	QString newTocName = tempDir + "cdbo_audio_" + timeStamp() + ".toc";
	
	CdboTracksEditor *cte = new CdboTracksEditor(this, "cte");
	
	bool success = cte->copyToc(tocName, newTocName, true);
	delete cte;
	if(!success)
		return;

	tocName = newTocName;
	newToc = true;
	launchTrackEditorBtn->setEnabled(true);
}
//=====================================
void CdboAudioCDDialog::launchTrackEditor() {
	clearOutput();
	if(haveToc && !tocName.isEmpty()){
		//this should never happen
		if(!newToc){
			int result = KMessageBox::warningYesNo(this, 
				i18n("This will overwrite existing tracks layout description file. Continue?"), 
				i18n("Overwrite existing?"), KStdGuiItem::yes(), KStdGuiItem::no());
			if(result == KMessageBox::No)
				return;
		}
		
		lastTitle = titleTxt->text();
		lastArtist = perfomTxt->text();
		CdboTracksEditor *cte = new CdboTracksEditor(this, "cte");
		bool loaded = cte->loadToc(tocName, lastTitle, lastArtist);
		if(loaded)
			cte->exec();
		delete cte;
		return;
	}
	
	if(tocName.isEmpty()){
		tocName = tempDir + "cdbo_audio_" + timeStamp() + ".toc";
		newToc = true;
	}
	CdboTracksEditor *cte = new CdboTracksEditor(this, "cte");
	lastTitle = titleTxt->text();
	lastArtist = perfomTxt->text();
	cte->loadFileList(fileList, tocName, "\n", lastTitle, lastArtist);
	int result = cte->exec();
	delete cte;
	
	if(result == QDialog::Accepted)
		haveToc = true;
	else if(!haveToc)
		tocName = "";
}
//---------------------------------------------------------------------------------------------
void CdboAudioCDDialog::startBtnSlot(){
	clearOutput();
	
	if(launchTrackEditorBtn->isEnabled() && !haveToc){
		int result = KMessageBox::warningContinueCancel(this, 
			i18n("Tracks layout description file was not created for this CD. This file is required. "\
				"Generate default file or cancel and use 'Tracks Editor' to create a custom one."), 
			i18n("Tracks layout description missing."), i18n("&Generate Default"),
			"Generate default tracks layout");
		if(result == KMessageBox::Cancel){
			launchTrackEditor();
			return;
		}
		tocName = tempDir + "cdbo_audio_" + timeStamp() + ".toc";
		lastTitle = titleTxt->text();
		lastArtist = perfomTxt->text();
		CdboTracksEditor *cte = new CdboTracksEditor(this, "cte");
		bool created = cte->createDefaultToc(fileList, tocName, "\n", lastTitle, lastArtist);
		delete cte;
		if(!created){
			dialogCntPnl->output(Cdbo::StatusNotOkMessage,
				i18n("Failed to create generate default tracks layout. Can not proceed."));
			return;
		}
		newToc = true;
		haveToc = true;
	}
	//here check if title or artist has changed since last save
	if(launchTrackEditorBtn->isEnabled() && (lastTitle != titleTxt->text() || lastArtist != perfomTxt->text())){
		lastTitle = titleTxt->text();
		lastArtist = perfomTxt->text();
		QString newTocName = tempDir + "cdbo_audio_" + timeStamp() + ".toc";
		CdboTracksEditor *cte = new CdboTracksEditor(this, "cte");
		bool success = cte->copyToc(tocName, newTocName, false, lastTitle, lastArtist);
		delete cte;
		if(!success)
			return;
		cleanupTmpToc();
		tocName = newTocName;
		newToc = true;
		haveToc = true;
	}
	if(launchTrackEditorBtn->isEnabled())
		multiStep = "FALSE";
	else
		multiStep = "TRUE";

	numberOfCopies = QString::number(numberSpn->value());

	if(!createAction("CdboCreateAudioCdAction"))
		return;

	targetDevicePnl->closeTray();

	addActionParam("ACTION_TYPE", actionType());
	addActionParam("SPEED", speedPnl->speed());
	addActionParam("TARGET_DEVICE",  targetDevicePnl->scsiDevice());

	addActionParam("FILE_LIST", &fileList);
	addActionParam("TOC_NAME", &tocName);
	addActionParam("IS_MULTY_STEP_AUDIO", &multiStep);

	addActionParam("TMP_DIRECTORY", &tempDir);
	addActionParam("IMAGE_SIZE", &minTmpSize);
	
	addActionParam("NUMBER_OF_COPIES", &numberOfCopies);
	//debugCommands();
	launchCurrAction();

}
//---------------------------------------------------------------------------------------------
void CdboAudioCDDialog::taskDone(bool success){
	//kdDebug() << "CdboAudioCDDialog::taskDone...........\n";
	if(success){
		//
	}
	CdboDialogBase::taskDone(success);
}
