/***************************************************************************
                          CdboRipAudioBase.cpp  -  description
                             -------------------
    begin                : Mon Feb 5 2002
    copyright            : (C) 2001 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include "CdboRipAudioBase.h"
#include "CdboRipAudioBase.moc"

#include <qcombobox.h>
#include <qframe.h>
#include <qgroupbox.h>
#include <qheader.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qimage.h>
#include <qpixmap.h>
#include <kurlrequester.h>

#include "CdboCddaView.h"
#include "CdboDialogCntPnl.h"
#include "CdboDriveInfoPnl.h"


//=====================================================================
CdboRipAudioBase::CdboRipAudioBase( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : CdboDialogBase( parent, name, modal, fl )
{
    if ( !name )
	setName( "CdboRipAudioBase" );

    setCaption( i18n( "Rip (Read) Audio CD - CD Bake Oven" ) );
    cdboRipAudioBaseLayout = new QGridLayout( this );
    cdboRipAudioBaseLayout->setSpacing( 0 );
    cdboRipAudioBaseLayout->setMargin( 0 );

    baseFrm = new QWidget( this, "baseFrm" );
    baseFrmLayout = new QGridLayout( baseFrm );
    baseFrmLayout->setSpacing( 5 );
    baseFrmLayout->setMargin( 8 );

    srcDevicePnl = new CdboDriveInfoPnl( baseFrm, "srcDevicePnl" );

    baseFrmLayout->addWidget( srcDevicePnl, 0, 0 );

    detailsPnl = new QGroupBox( baseFrm, "detailsPnl" );
    detailsPnl->setTitle( i18n( "Destination" ) );
    QWhatsThis::add(  detailsPnl, i18n( "Output Details" ) );
    detailsPnl->setColumnLayout(0, Qt::Vertical );
    detailsPnl->layout()->setSpacing( 0 );
    detailsPnl->layout()->setMargin( 0 );
    detailsPnlLayout = new QGridLayout( detailsPnl->layout() );
    detailsPnlLayout->setAlignment( Qt::AlignTop );
    detailsPnlLayout->setSpacing( 6 );
    detailsPnlLayout->setMargin( 11 );

    outputDirUr = new KURLRequester( "", detailsPnl, "outputDirUr" );
    outputDirUr->setMode(KFile::Directory | KFile::LocalOnly);
    QToolTip::add(  outputDirUr, i18n( "Output directory" ) );
    detailsPnlLayout->addMultiCellWidget( outputDirUr, 0, 0, 0, 1);

    fileFormatCmb = new QComboBox( FALSE, detailsPnl, "fileFormatCmb" );
    fileFormatCmb->insertItem( i18n("Ogg-Vorbis (.ogg)") );
    fileFormatCmb->insertItem( i18n("MP3 (.mp3)") );
    fileFormatCmb->insertItem( i18n("PCM (.wav)") );
    fileFormatCmb->insertItem( i18n("Raw PCM (.cda)") );

    QToolTip::add(  fileFormatCmb, i18n( "Select the output format" ) );
    QWhatsThis::add(  fileFormatCmb, i18n( "Select the file output format" ) );

    detailsPnlLayout->addWidget( fileFormatCmb, 1, 1);

    fileFormatLbl = new QLabel( detailsPnl, "fileFormatLbl" );
    fileFormatLbl->setText( i18n( "&Format:" ) );

    detailsPnlLayout->addWidget( fileFormatLbl, 1, 0 );
    detailsPnlLayout->setColStretch(1, 1);

    baseFrmLayout->addWidget( detailsPnl, 1, 0 );

    cdboRipAudioBaseLayout->addWidget( baseFrm, 0, 0 );
//////////
    tracksPnl = new QWidget( this, "tracksPnl" );
    tracksPnlLayout = new QVBoxLayout( tracksPnl );
    tracksPnlLayout->setSpacing( 6 );
    tracksPnlLayout->setMargin( 7 );
//////////
    trackView = new CdboCddaView( tracksPnl, "trackViewInCopy" );
    tracksPnlLayout->addWidget( trackView );

    cdboRipAudioBaseLayout->addWidget( tracksPnl, 0, 1 );

    dialogCntPnl = new CdboDialogCntPnl( this, "dialogCntPnl" );

    cdboRipAudioBaseLayout->addMultiCellWidget( dialogCntPnl, 1, 1, 0, 1 );
    cdboRipAudioBaseLayout->setColStretch(0, 4);
    cdboRipAudioBaseLayout->setColStretch(1, 6);
	
	fileFormatLbl->setBuddy( fileFormatCmb );
}

