/***************************************************************************
                          cdbodialogloader.cpp  -  description
                             -------------------
    begin                : Fri Sep 6 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/
#include <kaboutdata.h>
#include <kcmdlineargs.h>
#include <klocale.h>
#include <kapplication.h>
#include <dcopclient.h>

#include "CdboDialogFactory.h"
#include "CdboVersion.h"

static const char *description =
	I18N_NOOP("cdbodialogloader - tool for loading cdbakeoven dialogs.");


static KCmdLineOptions options[] =
{
	{ "dialog-type <type>", I18N_NOOP("Type of dialog to load. Available types are:\n\t"\
		"CdboDataCDDialog \n\tCdboCopyCDDialog \n\tCdboAudioCDDialog \n\tCdboRipAudioDialog \n\tCdboEraseCDDialog\n\t"\
		"CdboFixateCDDialog \n\t"),
		0 },
	{ "default-device <device>", I18N_NOOP("Device (ie. `/dev/scd0') to be chosen as default 'target device' by dialog."), 0 },
	{ "source-dirs <list>", I18N_NOOP("Comma separated list of top level directories. (CdboDataCDDialog only)"), 0 },
	{ "iso-image <file>", I18N_NOOP("Full path to ISO image. (CdboDataCDDialog only)"), 0 },
	{ "from-url <url>", I18N_NOOP("Either newcd protocol (ie. `newcd:/New Data CD/') URL or a desktop file. (CdboDataCDDialog only)"), 0 },
	{ "iso-image-only", I18N_NOOP("Do not burn. (CdboDataCDDialog only)"), 0 },
	{ "pathspec-list <file>", I18N_NOOP("mkisofs style filelist. (CdboDataCDDialog only)"), 0 },
	{ "hide-joliet-list <file>", I18N_NOOP("mkisofs style filelist. (CdboDataCDDialog only)"), 0 },
	{ "hide-rr-list <file>", I18N_NOOP("mkisofs style filelist. (CdboDataCDDialog only)"), 0 },
	{ "hide-hfs-list <file>", I18N_NOOP("mkisofs style filelist. (CdboDataCDDialog only)"), 0 },
	{ "audio-list <list>", I18N_NOOP("Comma separated list of audio files. (CdboAudioCDDialog only)"), 0 },
	{ "toc-file <file>", I18N_NOOP("cdrdao style toc file. (CdboAudioCDDialog only)"), 0 },
	{ "debug-commands", I18N_NOOP("Show constructed commands."), 0 },
	{ "debug-output", I18N_NOOP("Dump command output."), 0 },
	{ 0, 0, 0 }
};

int main(int argc, char *argv[]){

	KAboutData *aboutData = new KAboutData( "cdbakeoven", I18N_NOOP("CD Bake Oven"),
		CDBO_VERSION, description, KAboutData::License_GPL,
		"(c) 2001-2002, Alexy Podolsky",
		I18N_NOOP("\nThis is free software with ABSOLUTELY NO WARRANTY."
		          "\nUse at your own risk! See --license for more details."),
		"http://cdbakeoven.sourceforge.net");
	aboutData->addAuthor("Alexy Podolsky",
		I18N_NOOP("Author and maintainer"),
		"alexpod@users.sourceforge.net");
	
	KCmdLineArgs::init( argc, argv, aboutData );
	KCmdLineArgs::addCmdLineOptions( options );

	KApplication app;
	KLocale::setMainCatalogue("cdbakeoven");
	
	app.dcopClient()->registerAs("cdbakeoven", false);

	CdboDialogFactory *dialogFactory = new CdboDialogFactory();
	dialogFactory->loadArgs(KCmdLineArgs::parsedArgs());
	
	return app.exec();
}

