/***************************************************************************
                          CdboTmpDirConfig.cpp  -  description
                             -------------------
    begin                : Fri Jun 1 2001
    copyright            : (C) 2001 by Alexy Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <qcheckbox.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qspinbox.h>
#include <qtooltip.h>

#include <klocale.h>
#include <kiconloader.h>
#include <kfiledialog.h>
#include <kconfig.h>
#include <kurl.h>
#include <kmessagebox.h>
#include <kprocess.h>
#include <kdebug.h>
#include <kstdguiitem.h>
#include <krun.h>

#include "CdboTmpDirConfig.h"
#include "CdboTmpDirConfig.moc"

CdboTmpDirConfig::CdboTmpDirConfig( QWidget* parent, const char* name, WFlags fl )
    : CdboTmpDirConfigBase( parent, name, fl ){
	tmpBrowseBtn->setPixmap( BarIcon("fileopen", KIcon::SizeSmall ) );

	showMessage = false;
	showBrowseError = false;
	config = new KConfig("cdbakeovenrc");
	connect( minTmpSizeSpn, SIGNAL( valueChanged(int) ), this, SLOT( slotChanged() ) );
	connect( minTmpSizeSpn, SIGNAL( valueChanged(int) ), this, SLOT( updateSizeInfoLbl() ) );
	connect( checkSizeChk, SIGNAL( clicked() ), this, SLOT( slotChanged() ) );
}
//---------------------------------------------------------------------------------------------
CdboTmpDirConfig::~CdboTmpDirConfig(){
	delete config;
}
//---------------------------------------------------------------------------------------------
void CdboTmpDirConfig::readOptions(){
	config->setGroup("Default Settings");
	tempDir = config->readEntry("Tmp Dir");
	if(tempDir.isEmpty())
		tempDir = "/tmp/";
	tmpLbl->setText(tempDir);
	tmpSize = config->readNumEntry("tmp Size", 0);
	
	checkSizeChk->setChecked(config->readBoolEntry("Check Tmp Size", true));
	minTmpSizeSpn->setValue(config->readNumEntry("Minimum Tmp Size", 700));
	checkSizeSlot();
	updateSizeInfoLbl();
}
//-----------------------------------------------------------------------------------------------
void CdboTmpDirConfig::saveOptions(){
	config->setGroup("Default Settings");
	config->writeEntry("Tmp Dir", tmpLbl->text());
	
	config->writeEntry("Check Tmp Size", checkSizeChk->isChecked());
	config->writeEntry("Minimum Tmp Size", minTmpSizeSpn->value());
	config->sync();
}
//-----------------------------------------------------------------------------------------------
void CdboTmpDirConfig::defaults(){
	config->setGroup("Default Settings");
	config->writeEntry("Tmp Dir", "/tmp/");
	config->writeEntry("tmp Size", 0);
	config->writeEntry("Check Tmp Size", true);
	config->writeEntry("Minimum Tmp Size", 700);
	config->sync();
}
//------------------------------------------------------------------------------------------
void CdboTmpDirConfig::checkSizeSlot(){
	bool yes = checkSizeChk->isChecked();
	minLbl->setEnabled(yes);
	minTmpSizeSpn->setEnabled(yes);
}
//----------------------------------------------------------------------------------------------
void CdboTmpDirConfig::tmpBrowseSlot(){
	tempDir = tmpLbl->text();
	if(!tempDir || tempDir == ""){
		tempDir = KFileDialog::getExistingDirectory(QDir::homeDirPath(),
			this, i18n("Choose Temporary Directory"));
	}
	else{
		tempDir = KFileDialog::getExistingDirectory(tempDir, this,
			i18n("Choose Temporary Directory"));
	}
	tempDir = tempDir.stripWhiteSpace();
	if(tempDir.isEmpty()){
		tempDir = tmpLbl->text();
		return;
	}
	QDir dir(tempDir);
	if(!dir.exists()){
		tempDir = tmpLbl->text();
		return;
	}
	slotChanged();
	showBrowseError = showMessage;
	checkTmpSize();
}
//---------------------------------------------------------------------------------------------
void CdboTmpDirConfig::checkTmpSize(){
	KShellProcess *cdboProc = new KShellProcess();
	QString command = tempDir;
	
	KRun::shellQuote(command);
	command = "df -m " + command;
	*cdboProc << command;

	connect(cdboProc, SIGNAL(processExited(KProcess *)),
		this, SLOT(kcmshellDone(KProcess *)));
 	connect(cdboProc, SIGNAL(receivedStdout (KProcess *, char *, int)),
		this, SLOT(receivedSizeMessageSlot(KProcess *, char *, int)));

	cdboProc->start(KProcess::NotifyOnExit, KProcess::AllOutput);
}
//---------------------------------------------------------------------------------------------
void CdboTmpDirConfig::receivedSizeMessageSlot(KProcess *, char *buffer, int buflen){
	if(buflen <= 0)
		return;

	QString buff(buffer);
	buff = buff.left(buflen);
	QStringList comOutput(QStringList::split("\n",buff));
	comOutput = QStringList::split(" ", comOutput[1]);
	buff = comOutput[3].simplifyWhiteSpace();
	int newSize = buff.toInt();
	
	if(showBrowseError && checkSizeChk->isChecked() && newSize < minTmpSizeSpn->value()){
		int result = KMessageBox::warningYesNo(this, i18n(
			"This directory has less space than the minimum required. Use it anyway?"),
			i18n("Please confirm"), KStdGuiItem::yes(), KStdGuiItem::no(), "Check Tmp Dir Size");
		if(result != KMessageBox::No){
			tempDir = tmpLbl->text();
			return;
		}
	}
	
	showBrowseError = false;
	tmpSize = newSize;
	updateTmp();
	updateSizeInfoLbl();
}
//---------------------------------------------------------------------------------------------
void CdboTmpDirConfig::updateTmp(){
	tmpLbl->setText(tempDir);
	QToolTip::add(tmpLbl, i18n("Using '%1' as the temporary directory to store the disk image.").arg(tempDir));
	KConfig* config = new KConfig("cdbakeovenrc");
	config->setGroup("Default Settings");
	config->writeEntry("tmp Size", tmpSize);
	config->sync();
	delete config;
}
//---------------------------------------------------------------------------------------------
void CdboTmpDirConfig::updateSizeInfoLbl(){
	QString flag = i18n("more");
	if(tmpSize < 650)
		flag = i18n("less");
		
	tmpSizeInfoLbl->setText(
		i18n("This directory has <b>%1</b> than %1MB available.").arg(flag).arg(QString::number(minTmpSizeSpn->value())));
}
//---------------------------------------------------------------------------------------------
bool CdboTmpDirConfig::bigEnouphTmp(){
	if(!checkSizeChk->isChecked())
		return true;
	return  tmpSize > minTmpSizeSpn->value();
}
//---------------------------------------------------------------------------------------------
void CdboTmpDirConfig::kcmshellDone(KProcess *proc){
	delete proc;
}
//---------------------------------------------------------------------------------------------
void CdboTmpDirConfig::slotChanged(){
	emit modified();
}
//---------------------------------------------------------------------------------------------
void CdboTmpDirConfig::enableErrorMessage(){
	showMessage = true;
}
//---------------------------------------------------------------------------------------------
void CdboTmpDirConfig::disableErrorMessage(){
	showMessage = false;
}

