/***************************************************************************
                          CdboCDRom.cpp  -  description
                             -------------------
    begin                : Tue Sep 24 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sf.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

//#include <sys/types.h>
//#include <sys/stat.h>		// mkdir
#include <sys/ioctl.h>		// ioctls
#include <unistd.h>		// lseek, read. etc
#include <fcntl.h>		// O_RDONLY etc.
//#include <linux/iso_fs.h>	// isofs constants
#include <linux/cdrom.h>	// old ioctls for cdrom
#include <limits.h> // PATH_MAX

//#include <scsi/scsi.h>
//#include <fstream.h>
//#include <limits.h>
//#include <stdio.h>
//#include <stdlib.h>

//#include <iostream.h>


#include <kdebug.h>

#include "CdboCDRom.h"
#include <qfile.h>
#include "CdboCDRom.moc"

#ifndef MNTTYPE_ISO9660
#define MNTTYPE_ISO9660 "iso9660"
#endif


//===========================================
CdboCDRom::CdboCDRom(QString path, QObject* parent, const char* name)
		: QObject( parent, name ){

	this->path = path;

	allowEject = true;
	cdromfd = -1;
	eTimeId = -1;
	testInterval = 600;
	currMediaStatus = (isMounted()) ? Cdbo::MediaMounted : Cdbo::MediaUnknown;
}
//===========================================
CdboCDRom::~CdboCDRom(){
   CdboCDRom::close();
}
//===========================================
bool CdboCDRom::isValid(){
	bool opened = false;
	if (cdromfd < 0) {
		if ( open() < 0 )
			return false;
		opened = true;
	}

	int ret = ioctl(cdromfd, CDROM_DRIVE_STATUS, CDSL_CURRENT);

	if (opened)
		close();

	return (ret >= 0);
}
//===========================================
bool CdboCDRom::isDiskOK(){
   return (::ioctl(cdromfd, CDROM_DRIVE_STATUS, CDSL_CURRENT) == CDS_DISC_OK);
}
//===========================================
bool CdboCDRom::isOpen(){
   return (cdromfd > 0);
}
//===========================================
int CdboCDRom::open(){
	if(cdromfd == -1) {
		cdromfd = ::open(QFile::encodeName(path), O_RDONLY | O_NONBLOCK );
		if (cdromfd < 0)
			return cdromfd;

		::ioctl(cdromfd, CDROM_CLEAR_OPTIONS, CDO_AUTO_CLOSE);
		/*if(allowEject) {
	//	 ::ioctl(cdromfd, CDROM_CLEAR_OPTIONS, CDO_LOCK);
#ifdef CDROM_LOCKDOOR
			::ioctl(cdromfd, CDROM_LOCKDOOR, 0);
#else
# warning "Linux kernel >= 2.2.4 required to be able to (un)lock CD-ROMs."
#endif
		}*/
	}
	return cdromfd;
}
//===========================================
bool CdboCDRom::hasMediaChanged(){
	return (ioctl(cdromfd, CDROM_MEDIA_CHANGED, CDSL_CURRENT) > 0);
}
//===========================================
int CdboCDRom::close(){
	int ret = ::close(cdromfd);
	cdromfd = -1;
	return ret;
}
//===============================
void CdboCDRom::checkState(){

//kdDebug() << "CdboCDRom::Checking " << path.latin1() << endl;

	if(!isOpen())
		open();


	//if(!hasMediaChanged())
		//return;

//kdDebug() << "CdboCDRom::Media Changed" << endl;




	Cdbo::MediaStatus prevMediaStatus = currMediaStatus;

	testMedia();


	/*if(prevMediaStatus == currMediaStatus()){
		kdDebug() << "CdboCDRom::disc status UN_changed" << endl;
		return;
	}

	kdDebug() << "CdboCDRom::disc status changed" << endl;
		*/
	switch(currMediaStatus) {
		case Cdbo::MediaTrayIsOpen:
			//kdDebug() << "CdboCDRom::tray_is_open" << endl;
			break;

		case Cdbo::MediaUnknown:
			//kdDebug() << "CdboCDRom::unknown" << endl;
			if(prevMediaStatus != Cdbo::MediaTrayIsOpen)
				//kdDebug() << "CdboCDRom::disc ejected " << endl;
			break;

		case Cdbo::MediaUnmounted:
			//kdDebug() << "CdboCDRom::unmounted" << endl;
			break;

		case Cdbo::MediaMounted:
			//kdDebug() << "CdboCDRom::mounted" << endl;
			//kdDebug() << "CdboCDRom::disc inserted" << endl;

			break;

		case Cdbo::MediaAudio:
			//kdDebug() << "CdboCDRom::audio" << endl;
			break;
	}




	//close();//////////////////////



	if(prevMediaStatus != currMediaStatus)
		emit stateChanged(*this);

}
//=========================
Cdbo::MediaStatus CdboCDRom::testMedia(){
	currMediaType = Cdbo::MediaTypeUnknown;

	if (cdromfd >= 0) {
		int ret = ioctl(cdromfd, CDROM_DRIVE_STATUS, CDSL_CURRENT);
		//kdDebug() << "CdboCDRom::CDROM_DRIVE_STATUS: " << ret << endl;
		switch(ret) {
			case CDS_DISC_OK:
				break;
			case CDS_TRAY_OPEN:
			case CDS_NO_DISC:
				currMediaStatus = Cdbo::MediaTrayIsOpen;
				return currMediaStatus;
			case CDS_NO_INFO:
			case CDS_DRIVE_NOT_READY:
			default:
				currMediaStatus = Cdbo::MediaUnknown;
				return currMediaStatus;
		}

		//ret = ioctl(cdromfd, CDROM_GET_CAPABILITY, CDC_PLAY_AUDIO);

		 //<<<<<<<<<<<<<<<<<< /proc/sys/dev/cdrom >>>>>>>>>>>>>>

		/*ret =ioctl(cdromfd, TYPE_DISK, 0);
		cerr << "\n\n TYPE_DISK: " << ret << endl;
		ret = ioctl(cdromfd, INQUIRY, 0);
		cerr << "\n\n INQUIRY: " << ret << endl;
		ret = ioctl(cdromfd, MEDIUM_SCAN, 0);
		cerr << "\n\n MEDIUM_SCAN: " << ret << endl;
		*/

		if (hasMediaChanged()) {
			currMediaStatus = Cdbo::MediaTrayIsOpen;
		}
		else {
			if((currMediaStatus == Cdbo::MediaTrayIsOpen) || (currMediaStatus == Cdbo::MediaUnknown)) {
				int type = ::ioctl(cdromfd, CDROM_DISC_STATUS, CDSL_CURRENT);
				switch(type) {
					case CDS_DATA_1:
					case CDS_DATA_2:
						currMediaStatus = (isMounted()) ? Cdbo::MediaMounted : Cdbo::MediaUnmounted;
						currMediaType = Cdbo::MediaTypeData;
						break;
					case CDS_AUDIO:
						currMediaStatus = Cdbo::MediaAudio;
						currMediaType = Cdbo::MediaTypeAudio;
						break;
					case CDS_MIXED:
						currMediaStatus = (isMounted()) ? Cdbo::MediaMounted : Cdbo::MediaUnmounted;
						currMediaType = Cdbo::MediaTypeMixed;
						break;
					case CDS_NO_INFO:
					case CDS_NO_DISC:
						currMediaStatus = Cdbo::MediaUnknown;
						break;
					default:
						//kdDebug() << "Unknown data type: " << type << endl;
						currMediaStatus = Cdbo::MediaUnknown;
				}
			}
			else if (currMediaStatus == Cdbo::MediaMounted || currMediaStatus == Cdbo::MediaUnmounted) {
				currMediaStatus = (isMounted()) ? Cdbo::MediaMounted : Cdbo::MediaUnmounted;
			}

			/*if(allowEject) {
				//::ioctl(cdromfd, CDROM_CLEAR_OPTIONS, CDO_LOCK);
#ifdef CDROM_LOCKDOOR
				::ioctl(cdromfd, CDROM_LOCKDOOR, 0);
#else
# warning "Linux kernel >= 2.2.4 required to be able to (un)lock CD-ROMs."
#endif
			}*/
		}
	}
	else
		currMediaStatus = Cdbo::MediaUnknown;

	return currMediaStatus;
}
//===========================================
void CdboCDRom::startMonitoringState(){

	if(eTimeId != -1)
		stopMonitoringState();

	eTimeId = startTimer(testInterval);
}
//===========================================
void CdboCDRom::stopMonitoringState(){
	killTimer(eTimeId);
	eTimeId = -1;
}
//===========================================
void CdboCDRom::timerEvent(QTimerEvent *event){
	if(event->timerId() == eTimeId){
		checkState();
	}
}



//==============HMMMMMMMMMMMM=================
//==============HMMMMMMMMMMMM=================
//==============HMMMMMMMMMMMM=================
//==============HMMMMMMMMMMMM=================
//==============HMMMMMMMMMMMM=================
//==============HMMMMMMMMMMMM=================
//==============HMMMMMMMMMMMM=================










//===========================================
bool CdboCDRom::isMounted(){
  /* ifstream mnts("/proc/mounts");
   char line[PATH_MAX*3];

   char lpath[PATH_MAX];

   while(mnts.getline(line, PATH_MAX*3-1)) {
      char mpoint[PATH_MAX], dname[PATH_MAX];
      sscanf(line, "%s %s", dname, mpoint);
      int len = readlink(dname, lpath, PATH_MAX);
      if( len > 0 && len < PATH_MAX) {
	lpath[len] = 0;
      }
      if(path == dname || path == lpath)
      {
         // cout << "path=" << path << ",dname=" << dname << "\n";
         // cout << "mountpath=" << mountpath << ",mpoint=" << mpoint << "\n";
	 mountPath = mpoint;
	 mnts.close();
	 return true;
      }
   }
   mnts.close();*/
   return false;
}
//===========================================
bool CdboCDRom::mount() {
  /* bool ret = false;

   if(!allowEject)
      close();

   KProcess p;
   p << "/bin/mount" << path;
   if (p.start(KProcess::Block) && p.normalExit() &&
       (p.exitStatus() == 0))
   {
      currMediaStatus = Cdbo::MediaMounted;
      ret = true;
   }
   else
   {
      currMediaStatus = Cdbo::MediaUnknown;
   }

   if(!allowEject)
      open();

   return ret;
*/


/*
bool readonly, const QString& format, const QString& device, const QString&
mountpoint,
              const QString & desktopFile, bool show_filemanager_window

KAutoMount *m=new KAutoMount( readonly, format, device, mountpoint,
              desktopFile, show_filemanager_window);



   */
   return false;
}
//==========================================
bool CdboCDRom::umount() {
	return true;
}
//===========================================
bool CdboCDRom::eject(){
   return (::ioctl(cdromfd, CDROMEJECT) == 0);
}
//===========================================
void CdboCDRom::closeTray(){
}

