/***************************************************************************
                          CdboDialogBase.cpp  -  description
                             -------------------
    begin                : Tue Jul 17 2001
    copyright            : (C) 2001 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <qtimer.h>

#include <klocale.h>
#include <kprocess.h>
#include <kdebug.h>
#include <kconfig.h>
#include <kmessagebox.h>
#include <klibloader.h>
#include <kiconloader.h>
//#include <kapplication.h>

#include <dcopclient.h>


#include "CdboDialogBase.h"
#include "CdboDialogBase.moc"



CdboDialogBase::CdboDialogBase(QWidget* parent,  const char* name, bool modal, WFlags fl )
    : KDialog( parent, name, modal, fl ),
	DCOPObject( "CdboDialogBase" ){
	settingTypes = "cdbodefaultopt";
	DEBUG_COMMANDS = /*"TRUE"; */"FALSE";
	DEBUG_OUTPUT =/* "TRUE";*/"FALSE";
	if(!icon())
		setIcon(SmallIcon("cdbakeoven"));

}
//=======================================
CdboDialogBase::~CdboDialogBase(){

}
//=======================================
void CdboDialogBase::init(){
	currAction = 0;
	config = 0;
	mDoingSomething = false;
}
//---------------------------------------------------------------------------------------------
void CdboDialogBase::resetConfig(){
	if(config)
		delete config;
	config = new KConfig("cdbakeovenrc");
}
//---------------------------------------------------------------------------------------------
void CdboDialogBase::readOptions(){
	resetConfig();
}
//---------------------------------------------------------------------------------------------
bool CdboDialogBase::doingSomething(){
	return mDoingSomething;
}
//---------------------------------------------------------------------------------------------
void CdboDialogBase::cancelSlot(){
	if(doingSomething() && currAction)
		currAction->tryToCancel();
}
//---------------------------------------------------------------------------------------------
void CdboDialogBase::quitSlot(){
	if(doingSomething() && currAction){
		if(currAction->tryToCancel()){
			saveOptions();
			accept();
		}
		return;
	}
	saveOptions();
	accept();
}
//---------------------------------------------------------------------------------------------
void CdboDialogBase::closeEvent(QCloseEvent *e){
	if(doingSomething() && currAction){
		if(!currAction->tryToCancel()){
			e->ignore();
			return;
		}
	}
	e->accept();
}
//---------------------------------------------------------------------------------------------
void CdboDialogBase::keyPressEvent(QKeyEvent *e){
	if(e->key() == Key_Escape){
		if(doingSomething() && currAction){
			if(!currAction->tryToCancel())
				return;
		}
		saveOptions();
		accept();
	}
	if(e->key() == Key_F2){
		if(!doingSomething())
			settingsSlot();
	}
}
//---------------------------------------------------------------------------------------------
void CdboDialogBase::taskDone(bool){
	reset();
}
//---------------------------------------------------------------------------------------------
void CdboDialogBase::taskFailedToStart(){
	reset();
}
//---------------------------------------------------------------------------------------------
void CdboDialogBase::taskCancelled(){
	reset();
}
//---------------------------------------------------------------------------------------------
void CdboDialogBase::reset(){
	stopForwardTimer();
	mDoingSomething = false;
	currAction = 0;
	readOptions();
}
//---------------------------------------------------------------------------------------------
void CdboDialogBase::accept(){
	KDialog::accept();
	QTimer::singleShot(10, this, SLOT(emitDialogDone()));
}
//---------------------------------------------------------------------------------------------
void CdboDialogBase::reject(){
	KDialog::reject();
	QTimer::singleShot(10, this, SLOT(emitDialogDone()));
}
//---------------------------------------------------------------------------------------------
bool CdboDialogBase::createAction(const QString type, const QString libName){
	saveOptions();

	KLibLoader* libLoader =  KLibLoader::self();
	KLibFactory *actionFactory = libLoader->factory(libName.latin1()) ;
	if(!actionFactory){
		kdError() << "Failed to load library named '" + libName + "'" << endl;
		kdError() << libLoader->lastErrorMessage() << endl;
	}

	QObject *obj = actionFactory->create(this, type.latin1(), type.latin1());
	if(!obj){
		KMessageBox::error(this,i18n("Failed to create object of type '%1'.").arg(type));
		taskFailedToStart();
		return false;
	}
	
	if ( !obj->inherits ( "CdboAction" ) ){
		KMessageBox::error(this,i18n("Failed to create action of type '%1'.").arg(type));
		taskFailedToStart();
		return false;
	}
	currAction = static_cast<CdboAction* >(obj);
	
	connect( currAction, SIGNAL( processDone(bool) ), this, SLOT( taskDone(bool)));
	connect( currAction, SIGNAL( failedToLaunchProcess() ), this, SLOT( taskFailedToStart()));
	connect( currAction, SIGNAL( processCancelled() ), this, SLOT( taskCancelled()));
	connect( currAction, SIGNAL( blockBtns(bool) ), this, SLOT( blockBtns(bool)));

	if(statusWidget()){
		connect( currAction, SIGNAL( status(QString) ),
				statusWidget(), SLOT( status(QString)));
	}
	if(outputWidget()){
		connect( currAction, SIGNAL( output(int, QString) ),
				outputWidget(), SLOT( output(int, QString)));
		connect( currAction, SIGNAL( percent(QString,int,QString) ),
				outputWidget(), SLOT( percent(QString,int,QString) ));
		connect( currAction, SIGNAL( percentInfo(QString,QString) ),
				outputWidget(), SLOT( percentInfo(QString,QString) ));
	}

	startForwardTimer();
	clearActionParams();
	mDoingSomething = true;
	addActionParam("DEBUG_COMMANDS", &DEBUG_COMMANDS);
	addActionParam("DEBUG_OUTPUT", &DEBUG_OUTPUT);
	return true;
}
//---------------------------------------------------------------------------------------------
void CdboDialogBase::addActionParam(QString name, QString *value){
	params.insert(name, value);
}
//---------------------------------------------------------------------------------------------
bool CdboDialogBase::removeActionParam(QString name){
	return params.remove(name);
}
//---------------------------------------------------------------------------------------------
void CdboDialogBase::clearActionParams(){
	params.clear();
}
//---------------------------------------------------------------------------------------------
void CdboDialogBase::settingsSlot(){
	popupSettings(settingTypes);
}
//---------------------------------------------------------------------------------------------
bool CdboDialogBase::launchCurrAction(){
	if(!currAction)
		return false;
	currAction->performTask(&params);
	return true;
}
//---------------------------------------------------------------------------------------------
bool CdboDialogBase::isCurrAction(QString wanted){
	return (currAction && wanted.startsWith(currAction->name()));
}
//---------------------------------------------------------------------------------------------
void CdboDialogBase::detectSlot(){
	popupSettings("cdbocdromdevices kcmdf");
}
//---------------------------------------------------------------------------------------------
void CdboDialogBase::popupSettings(QString type){
	KShellProcess *cdboProc = new KShellProcess();
	*cdboProc << "kcmshell" << type;
	connect(cdboProc, SIGNAL(processExited(KProcess*)), this, SLOT(kcmshellDone(KProcess*)));
	if(!cdboProc->start(KProcess::NotifyOnExit))
		kdWarning() << "Failed to start\n\n";
}
//---------------------------------------------------------------------------------------------
void CdboDialogBase::kcmshellDone(KProcess *proc){
	delete proc;
	readOptions();
	if(!doingSomething())
		initBtns();
}
//---------------------------------------------------------------------------------------------
QString CdboDialogBase::timeStamp(){
	return CdboAction::timeStamp();
}
//================================================================
bool CdboDialogBase::process(const QCString &fun, const QByteArray &data, QCString &replyType, QByteArray &replyData){
	if(fun == "readOptions()"){
		readOptions();
	}
	else
		return DCOPObject::process( fun, data, replyType, replyData );
	return true;
}
