/***************************************************************************
                          CdboViewPart.cpp  -  description
                             -------------------
    begin                : Wed Sep 4 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <qdir.h>
#include <qtimer.h>

#include <kaction.h>
#include <klocale.h>
#include <kglobal.h>
#include <kdebug.h>
#include <kconfig.h>
#include <kprocess.h>
#include <kfiledialog.h>
#include <kmessagebox.h>
#include <kio/netaccess.h>

#include "CdboViewPart.h"
#include "CdboViewPart.moc"

#include "CdboViewWidgetBase.h"


//---------------------------------------------------------------------------------------------
CdboViewPart::CdboViewPart(QWidget *parentWidget, const char *widgetName,
                            QObject *parent, const char *name, const QStringList& )
	: KParts::ReadWritePart( parent, name ){
	
	settingTypes = QString::null;
	m_type = i18n("Unknown");
	m_icon = "";
	m_lastStatus = "";
	m_shortCaption = "";
	m_caption = "";
	KGlobal::locale()->insertCatalogue("cdbakeoven");
	
	m_viewBase = new CdboViewWidgetBase(parentWidget, widgetName);
	m_viewBase->readOptions();
	m_viewBase->setFocusPolicy(QWidget::WheelFocus);
	setWidget(m_viewBase);

	setupActions();

}
//---------------------------------------------------------------------------------------------
CdboViewPart::~CdboViewPart(){
	m_viewBase->saveOptions();
}
//---------------------------------------------------------------------------------------------
void CdboViewPart::setupActions(){
	KStdAction::save(this, SLOT(saveFile()), actionCollection());
	KStdAction::saveAs(this, SLOT(saveAs()), actionCollection());
	new KActionSeparator( actionCollection(), "separator" );
	
	
	new KAction( i18n( "Create CD" ), "cdwriter_unmount", KShortcut(CTRL+Key_B),
		this, SLOT( createCDSlot() ), actionCollection(), "burnThisCD" );
	new KAction( i18n( "New CD Settings" ), KShortcut(CTRL+Key_Return), this,
		SLOT( settingsSlot() ), actionCollection(), "newCDProperties" );
		
	actionCollection()->addDocCollection(m_viewBase->getActionCollection());
}
//---------------------------------------------------------------------------------------------
void CdboViewPart::settingsSlot(){
	if(settingTypes.isNull())
		return;
	popupSettings(settingTypes);
}
//---------------------------------------------------------------------------------------------
void CdboViewPart::popupSettings(QString type){
	KShellProcess *cdboProc = new KShellProcess();
	*cdboProc << "kcmshell" << type;
	connect(cdboProc, SIGNAL(processExited(KProcess*)), this, SLOT(kcmshellDone(KProcess*)));
	if(!cdboProc->start(KProcess::NotifyOnExit))
		kdWarning() << "Failed to start\n\n";
}
//---------------------------------------------------------------------------------------------
void CdboViewPart::kcmshellDone(KProcess *proc){
	doingSomething = false;
	delete proc;
}
//---------------------------------------------------------------------------------------------
bool CdboViewPart::mountDevice(QString &drive, bool &needToUmount){
	needToUmount = false;
	
	if(drive.isNull())
		return false;
		
	//check if already mounted
	QString mountPoint = KIO::findDeviceMountPoint(drive);
	if(!mountPoint.isNull()){
		return true;
	}
	KConfig* config = new KConfig("cdbakeovenrc");
	config->setGroup("Wizard Detected");
	mountPoint = config->readEntry("Mount point for " + drive, QString::null);
	delete config;
	
	if(mountPoint.isNull()){
		KMessageBox::sorry(viewBase(),
					i18n("Mount point for this device is not configured yet. "
					"Please use CD Bake Oven Configuration Wizard to detect mount point for this device."));
		return false;
	}
	if(mountPoint.endsWith("*supermount")){
		drive = mountPoint.section("*", 0, 0);
		return true;
	}
	
	emit haveStatus(i18n("Mounting previous session. Please wait..."));
	emit processStarted();
	doingSomething = true;
	
	KIO::SimpleJob * job = KIO::mount(true, 0, drive, mountPoint, false);
	job->setName(mountPoint.utf8());

	connect(job, SIGNAL(result ( KIO::Job *)),
		this, SLOT(jobDone(KIO::Job *) ));
		
	
	while(doingSomething)
		qApp->processEvents(1);

	emit processDone();
	emit haveStatus("");
	
	if(job->error()){
		KMessageBox::error(viewBase(),
				i18n("Failed to mount. Make sure that the disc is valid "
				"and you have enough privileges to mount it."));
		return false;
	}
	drive = mountPoint;
	needToUmount = true;
	return true;
}
//---------------------------------------------------------------------------------------------
void CdboViewPart::umountDevice(QString &drive){
	if(drive.isNull())
		return;
	doingSomething = true;

	KIO::SimpleJob * job = KIO::unmount(drive, false);
	connect(job, SIGNAL(result ( KIO::Job *)),
			this, SLOT(jobDone(KIO::Job *) ));
			
	while(doingSomething)
		qApp->processEvents(1);

	emit processDone();
	emit haveStatus("");
}
//---------------------------------------------------------------------------------------------
void CdboViewPart::jobDone(KIO::Job *){
	doingSomething = false;
}
//---------------------------------------------------------------------------------------------
void CdboViewPart::createCDSlot(){
	if(isEmpty()){
		showEmptyError();
		return ;
	}
	createCD();
}
//---------------------------------------------------------------------------------------------
void CdboViewPart::proposeDeviceDetection(){
	int doIt = KMessageBox::warningYesNo(m_viewBase,
				i18n("No usable devices were found! Would you like to launch the "
				"'Device Detection' dialog now?"),
				i18n("No Usable Devices"));
				
	if(doIt == KMessageBox::Yes)
		popupSettings("cdbocdromdevices kcmdf");
}
//---------------------------------------------------------------------------------------------
void CdboViewPart::showEmptyError(){
	KMessageBox::detailedSorry(m_viewBase,
			i18n("Compilation list is empty!"),
			i18n("You must populate the compilation list with the files\n"
				"you wish to have on your new CD. To accomplish this,\n"
				"just drag the source files or directories from one of the\n"
				"upper (source) views down into the compilation view.\n"
				"This option can then be used to burn your files on to CD."));
		return;
}
//---------------------------------------------------------------------------------------------
bool CdboViewPart::checkOverwrite(const QString &path){
	QFileInfo d(path);
	if(d.exists()){
		int result = KMessageBox::warningContinueCancel(m_viewBase,
			i18n("A file with the same name already exists. "
			"Click 'Cancel' to display the 'Settings Dialog' and "
			"change the ISO name, or overwrite the existing one."),
			i18n("Overwrite Existing?"),
			i18n("&Overwrite"));
		switch(result) {
			case KMessageBox::Continue:
				return true;
			default:{
				popupSettings("cdboisoopt");
				return false;
			}
		}
	}
	return true;
}
//---------------------------------------------------------------------------------------------
bool CdboViewPart::checkDirSize(){
	KConfig *config = new KConfig("cdbakeovenrc");
	config->setGroup("Default Settings");

	if(config->readBoolEntry("Check Tmp Size", true)){
		int size = config->readNumEntry("tmp Size", -1);

		if(size == -1){
			int result = KMessageBox::warningContinueCancel(m_viewBase,
					i18n("Unable to determine temporary directory size. Would you "
					"like to display the 'Settings Dialog' to choose the temporary directory now?"),
					i18n("Display Settings Dialog Now?"),
					i18n("&Yes"));
    			switch(result) {
    				case KMessageBox::Continue:{
						popupSettings("cdbodefaultopt");
					delete config;
					return true;
				}
			}///////
		}
		else{
			if(size < getSize()){
				KMessageBox::error(m_viewBase,
						i18n("The temporary directory does not have enough space available " 
						"to store the disk image. Please remove some files from the list "
						"or choose another location from the 'Settings Dialog'."));
				delete config;
				return false;
			}
			config->writeEntry("Image Size", getSize());
			config->sync();
		}
	}
	delete config;
	return true;
}
//---------------------------------------------------------------------------------------------
void CdboViewPart::invalidPathError(const QString &path){
	KMessageBox::error(m_viewBase,
		i18n("Cannot accept %1: "
		"Not a valid path or you do not "
		"have enough privileges to add this item.").arg(path));
}
//---------------------------------------------------------------------------------------------
void CdboViewPart::localUrlsOnlyError(const QString &path){
	KMessageBox::sorry(viewBase(),
		i18n("Cannot accept %1. Only local files are supported at this time.").arg(path));
}
//---------------------------------------------------------------------------------------------
int CdboViewPart::shouldOverwrite(QString name){
	return KMessageBox::warningYesNoCancel(m_viewBase,
		i18n("An item with the same name already exists. "
		"Click 'Yes' if you would like to replace the old item with this new item, "
		"or 'Cancel' to stop the whole operation."),
		i18n("%1 is Already in the List").arg(name),
		KStdGuiItem::yes(),
		i18n("&Skip"));
}
//---------------------------------------------------------------------------------------------
void CdboViewPart::cannotFitError(const QString &fname){
	lastFailedName = fname;
	QTimer::singleShot(1, this, SLOT(showCannotFitError()));
}
//---------------------------------------------------------------------------------------------
void CdboViewPart::showCannotFitError(){
	KMessageBox::sorry(m_viewBase,
		i18n("Cannot accept %1. It does not fit on this CD.").arg(lastFailedName));
}
//---------------------------------------------------------------------------------------------
void CdboViewPart::emitLimitReached(){
	QTimer::singleShot(1, this, SLOT(limitReachedError()));
}
//---------------------------------------------------------------------------------------------
void CdboViewPart::limitReachedError(){
	KMessageBox::sorry(m_viewBase,
		i18n("Unable to accept any more files, CD-R(W) size limit has been reached."));
}
//====================================
bool CdboViewPart::closeURL(){
	if(isEmpty() || !isModified())
		return true;
	
	KConfig *config = new KConfig("cdbakeovenrc");
	config->setGroup("Notification Messages");	
	if(!config->readBoolEntry("Do Show Save List Warning", true)){
		delete config;
		return true;
	}
	delete config;
	int result = KMessageBox::warningYesNoCancel(m_viewBase,
					i18n("File list of %1 has been modified, would you like to save it?").arg(caption()),
					i18n("Save File List?"), KStdGuiItem::save(), KStdGuiItem::discard());
	switch(result){
		case KMessageBox::No:
			return true;
		case KMessageBox::Cancel:
			return false;
		case KMessageBox::Yes:
			return saveFile();
	}
	return true;
}
//=================================
bool CdboViewPart::saveAs(){
	QString prevName = m_file;
	m_file = "";
	if(saveFile())
		return true;
	m_file = prevName;
	return false;
}
//=================================
bool CdboViewPart::saveFile(){
	if(m_file.isEmpty()){
		do{
			if(!getNewFileName())
				return false;

			if(!QFile::exists(m_file))
				break;
			int doIt = KMessageBox::warningContinueCancel(m_viewBase,
				i18n("A file named %1 already exists. Are you sure you want to overwrite it?").arg(m_file),
				i18n("Overwrite File?"), i18n("&Overwrite"), "Overwrite Existing File");

			if(doIt == KMessageBox::Continue){
				KIO::NetAccess::del(KURL(m_file));
				break;
			}
		}while(true);
	}
	KConfig *file = new KConfig(m_file);
	file->setGroup("CD Bake Oven - CD File List");
	file->writeEntry("Type of CD", type());

	saveToFile(file);
	
	file->sync();
	delete file;
	setModified(false);
	m_url.setPath(m_file);
	QFileInfo info(m_file);
	m_shortCaption = info.baseName(true);
	setCaption(m_file);
	emit shortCaptionChanged(this);
	emit captionChanged(this);
	return true;
}
//=====================================================
bool CdboViewPart::getNewFileName(){
	if(m_file.isEmpty())
		m_file = m_file.left(m_file.findRev("/", -2));
	else
		m_file = QDir::homeDirPath(); //QString::null;

	m_file = KFileDialog::getSaveFileName(m_file,
				i18n("*.cdbo|CD List Files (*.cdbo)\n*|All files"),
				m_viewBase, i18n("Save CD List File As..."));
				
	if(m_file.isEmpty())
		return false;
	if(m_file.find(".cdbo", -5, false) == -1)
		m_file = m_file.stripWhiteSpace() + ".cdbo";
	
	return true;
}
//=====================================================
bool CdboViewPart::openURL(const KURL&url){
	bool opened = KParts::ReadOnlyPart::openURL(url);
	if(opened){
		QFileInfo info(m_file);
		m_shortCaption = info.baseName(true);
		setCaption(url.path());
		emit shortCaptionChanged(this);
		emit captionChanged(this);
	}
	return opened;
}
//=====================================================
void CdboViewPart::setModified ( bool modified ){
	KParts::ReadWritePart::setModified(modified);
	emit captionChanged(this);
}
//---------------------------------------------------------------------------------------------
void CdboViewPart::readOptions(){
}

