/***************************************************************************
                          CdboPluginManager.cpp  -  description
                             -------------------
    begin                : Sat Nov 27 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <qdir.h>
#include <qtimer.h>
//#include <qlayout.h>

//#include <kapplication.h>
#include <kstandarddirs.h>
#include <kconfig.h>
#include <kiconloader.h>

#include <kdebug.h>
#include <klocale.h>
#include <klibloader.h>
#include <kmessagebox.h>
#include <kparts/partmanager.h>

#include "CdboPluginManager.h"
#include "CdboPluginManager.moc"
#include "CdboShell.h"
#include "CdboMainView.h"

#include "CdboViewPart.h"


//=====================================
CdboPluginManager::CdboPluginManager(QObject* parent, const char* name) :
		QObject(parent, name),
	DCOPObject( "CdboPluginManager" ){
	m_supportedExt = "";
	m_prettySupportedExt = "";
	helpers.setAutoDelete(true);
	lastHelper = 0;
}
//==============================
CdboPluginManager* CdboPluginManager::s_self = 0;
CdboPluginManager* CdboPluginManager::self(){
	if ( !s_self )
		s_self = new CdboPluginManager;
	return s_self;
}
//=====================================
CdboPluginManager::~CdboPluginManager(){
	helpers.clear();
}
//=====================================
void CdboPluginManager::createTop(){
	KLibFactory *factory = KLibLoader::self()->factory("libcdbosrcview");
	if (!factory){
		kdError() << "no lib libcdbodataview" << endl; 
		KMessageBox::error(0, i18n("Failed to load required lib: %1!").arg("libcdbosrcview"));
		return;
	}
	KParts::Part* newTop = static_cast<KParts::Part *>(factory->create(mainView->topHolder(), "CdboSrcVeiwFactory", "KParts::ReadOnlyPart"));
	if(!newTop){
		kdError() << "failed to load top part" << endl;
		KMessageBox::error(0, i18n("Failed to load %1!").arg("CdboSrcVeiwFactory"));
		return;
	}

	mainView->setTop(newTop);
}
//=====================================
void CdboPluginManager::loadViewPlugins(){
	parent->slotSetStatusBarText(i18n("Loading file view plugins..."));
	parent->updateProgress(90);
	parent->partManager()->addPart( mainView->topPart(), true );

	parent->slotSetStatusBarText(i18n("Loading mode plugins..."));

	QString pluginsHome = locate("data", "cdbakeoven/viewplugins/") ;
	QDir thisDir(pluginsHome);

	thisDir.setFilter ( QDir::Files);
	thisDir.setNameFilter("*.desktop");
	QStringList list = thisDir.entryList();

	if(list.count() == 0){
		kdDebug() << "CdboPluginManager:: no views plugins found\n";
		return;
	}
	else{
		kdDebug() << "CdboPluginManager::found " << list.count() << " plugins" << endl;
		KConfig * pointer;
		QStringList types;
		QStringList infos;
		for(unsigned int i = 0; i < list.count(); i++){
			pointer = new KConfig(pluginsHome + list[i]);
			loadPlugin(pointer);
			delete pointer;
			types.append(lastHelper->name());
			infos.append(lastHelper->comment);
		}
		mainView->reloadButtomMessage(types, infos);
	}
	
	parent->updateProgress(95);
	parent->replugNewViewAction();
	parent->slotSetStatusBarText("");
	parent->updateProgress(99);
	registerExt(this, QStringList("cdbo"), QStringList("cdbo"));

	parent->updateProgress(100);
}
//=====================================
void CdboPluginManager::loadPlugin(KConfig * pointer){
	pointer->setGroup("Desktop Entry");
	QString name = pointer->readEntry("Name", QString::null);
	if(name.isEmpty())
		return;
	QString partLibName = pointer->readEntry("X-KDE-Library", QString::null);
	if(partLibName.isEmpty())
		return;
	QString partName = pointer->readEntry("X-KDE-FactoryName", QString::null);
	if(partName.isEmpty())
		return;

	createHelper(name, partLibName, partName, pointer->readEntry("Icon", ""),
			pointer->readEntry("Shortcut", ""), pointer->readEntry("Comment", i18n("No Info")));
}
//=====================================
void CdboPluginManager::createHelper(const QString& name, const QString& partLibName,
							const QString& partName, const QString& icon, const QString& shortcut, const QString& comment){

	CdboPluginManagerHelper* helper = new CdboPluginManagerHelper(this, name.latin1());

	helper->partLibName = partLibName;
	helper->partName = partName;
	helper->comment = comment;
	helper->icon = icon;

	KAction *newView = new KAction(name, icon,
		shortcut.isEmpty()? KShortcut(): KShortcut(shortcut), helper,
		SLOT(loadView()), parent->actionCollection(), partName.latin1());


	helpers.append(helper);
	setLastHelper(helper);

	parent->plugIntoNewViewAction(newView);
}
//================================
void CdboPluginManager::registerExt(QObject *receiver, const QStringList &extList, const QStringList &prettyExtList){
	if(!receiver || extList.isEmpty())
		return;

	if(extList.count() == 1)
		m_supportedExt += " *." + extList[0];
	else
		m_supportedExt += " *." + extList.join(" *.");
	if(!prettyExtList.isEmpty()){
		if(prettyExtList.count() == 1)
			m_prettySupportedExt += (m_prettySupportedExt == "" ? "*.": ", *.") + prettyExtList[0];
		else
			m_prettySupportedExt += (m_prettySupportedExt == "" ? "*.": ", *.") + prettyExtList.join(", *.");
	}

	QString buff;
	for(uint i = 0; i < extList.count(); i++)
		extPluginMap.insert(extList[i], receiver);
}
//================================
void CdboPluginManager::handleURL(const KURL& url){
	QFileInfo info(url.path());
	QObject* receiver = extPluginMap[info.extension(false)];
	if(!receiver){
		KMessageBox::error(0, i18n("Failed to load handler for %1!").arg(url.path()));
		return;
	}
	connect(this, SIGNAL(loadURL(const KURL&)), receiver, SLOT(openURL(const KURL&)));
	emit loadURL(url);
	disconnect(receiver);
}
//================================
void CdboPluginManager::openURL(const KURL& url){

	QString fileName = url.path();
	if(!QFile::exists(fileName)){
		kdDebug() << fileName << " does not exist!" << endl;
		return;
	}

	KConfig *file = new KConfig(fileName);
	file->setGroup("CD Bake Oven - CD File List");
	QString type = file->readEntry("Type of CD", QString::null);
	if(type.isEmpty()){
		kdDebug() << "'" << type << "' type not set!" << endl;
		return;
	}
	activateHelper(type, url);
}
//================================
void CdboPluginManager::activateHelper(const QString &type){
	activateHelper(type, KURL());
}
//================================
void CdboPluginManager::activateHelper(const QString &type, const KURL& url){
	CdboPluginManagerHelper* helper;
	for(uint i = 0; i < helpers.count(); i++){
		helper = helpers.at(i);
		if(helper->name() == type){
			helper->loadView(url);
			return;
		}
	}
}
//=====================================
void CdboPluginManager::addView(KParts::Part* newPart){
	mainView->addPart(newPart);
	parent->partManager()->addPart( newPart, true );
	parent->actionCollection()->action( "file_close" )->setEnabled(true);
}
//=====================================
QWidget* CdboPluginManager::partsHolder(){
	return mainView->partsHolder();
}
//=======================================
bool CdboPluginManager::queryClose(){
	QStringList lastUrls;
	if(!mainView->queryClose(lastUrls))
		return false;
	KConfig* config = new KConfig("cdbakeovenrc");
	config->setGroup("Default Settings");
	if(config->readBoolEntry("Reopen File Lists", true)){
		config->setGroup("General Options");
		config->writeEntry("Last Opened Views", lastUrls);
	}
	config->sync();
	delete config;

	return true;
}
//=======================================
void CdboPluginManager::loadLastViews(){
	KConfig* config = new KConfig("cdbakeovenrc");
	config->setGroup("Default Settings");
	if(!config->readBoolEntry("Reopen File Lists", true))
		return;

	config->setGroup("General Options");
	QStringList lastUrls = config->readListEntry("Last Opened Views");
	delete config;
	if(lastUrls.count() == 0)
		return;
	KURL url;
	int chunk = 100 / lastUrls.count();
	
	for(uint i = 0, done = 0; i < lastUrls.count(); i++, done += chunk){
		parent->slotSetStatusBarText(i18n("Reopening files.... Please wait."));
		parent->updateProgress(done);
		qApp->processEvents();
		url.setPath(lastUrls[i]);//FIXME: handle cdbo and other parts differently
		openURL(url);
	}
	parent->updateProgress(100);
	parent->slotSetStatusBarText("");
}
//=======================================
void CdboPluginManager::loadPart(const QString& partLibName, const QString& partName, const KURL& url, bool readWrite, const QString& icon, const QString& type){
	CdboPluginManagerHelper* helper = new CdboPluginManagerHelper(this, type.latin1());
	helper->partLibName = partLibName;
	helper->partName = partName;
	helper->readWrite = readWrite;
	helper->icon = icon;

	helpers.append(helper);
	helper->loadView(url);
}
//================================================================
void CdboPluginManager::updateSettings(){
	mainView->updateSettings();
}
//================================================================
bool CdboPluginManager::process(const QCString &fun, const QByteArray &data, QCString &replyType, QByteArray &replyData){
	if(fun == "updateSettings()"){
		updateSettings();
	}
	else
		return DCOPObject::process( fun, data, replyType, replyData );
	return true;
}


//=====================================
CdboPluginManagerHelper::CdboPluginManagerHelper(QObject *manager, const char *name)
	 : QObject(manager, name){
	 readWrite = true;
}
//=====================================
CdboPluginManagerHelper::~CdboPluginManagerHelper(){
}

//=====================================
void CdboPluginManagerHelper::loadView(){
	loadView(KURL());
}
//=====================================
void CdboPluginManagerHelper::loadView(const KURL& url){
	CdboPluginManager::self()->setLastHelper(this);

	KParts::Part *viewPart = createPart();
	if(!viewPart)
		return;

	CdboViewPart *cdboViewPart = static_cast<CdboViewPart *>(viewPart);
	if(cdboViewPart){
		cdboViewPart->setIcon(icon);
		cdboViewPart->setType(name());
	}
	KParts::ReadWritePart *partRW = static_cast<KParts::ReadWritePart *>(viewPart);
	if(partRW){
		partRW->setReadWrite(readWrite);
	}

	KParts::ReadOnlyPart *partRO = static_cast<KParts::ReadOnlyPart *>(viewPart);
	if(!url.isEmpty() && partRO)
			partRO->openURL(url);

	CdboPluginManager::self()->addView(viewPart);
}
//=====================================
KParts::Part * CdboPluginManagerHelper::createPart(){
	KLibFactory *factory = KLibLoader::self()->factory(partLibName.latin1());
	if (factory){
		KParts::Part *viewPart = static_cast<KParts::Part *>(factory->create(CdboPluginManager::self()->partsHolder(), partName.latin1(), "KParts::ReadWritePart"));
		if(viewPart)
			return viewPart;
	}
	KMessageBox::error(0, i18n("Failed to load %1!").arg(name()));
	return 0;
}

