/***************************************************************************
                          cdbo_konqy_plugins.cpp  -  description
                             -------------------
    begin                : Wed Sep 4 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include "cdbo_konqy_plugins.h"
#include "cdbo_konqy_plugins.moc"

#include <qpopupmenu.h>

#include <kmessagebox.h>
#include <konq_dirpart.h>
#include <kgenericfactory.h>
#include <kurl.h>
#include <kshortcut.h>
#include <kprocess.h>
#include <kdebug.h>
#include <krun.h>

#include <qdir.h>

K_EXPORT_COMPONENT_FACTORY( cdbokonqyplugins, KGenericFactory<CdboKonqyPlugins> );

CdboKonqyPlugins::CdboKonqyPlugins(QObject* parent, const char* name, const QStringList& )
	:KParts::Plugin(parent, name){

	KListAction* newcdAction = new KListAction(i18n("New CD"), "cdwriter_unmount", KShortcut::null(), actionCollection(), "newcdAction");

	QPopupMenu* menu = newcdAction->popupMenu();
	
	newCDProperties=new KAction( i18n( "New CD Settings" ), KShortcut::null(), this,
		SLOT( newCDPropertiesSlot() ), actionCollection(), "newCDProperties" );
	createISO=new KAction( i18n( "Save to File (Make ISO)" ), KShortcut::null(), this,
		SLOT( createISOSlot() ), actionCollection(), "createISO" );
	burnThisCD=new KAction( i18n( "Create CD" ), KShortcut::null(), this,
		SLOT( burnThisCDSlot() ), actionCollection(), "burnThisCD" );
	KAction* launchWizard=new KAction( i18n( "CD-Recording Setup Wizard" ), "wizard", KShortcut::null(), this,
		SLOT( launchWizardSlot() ), actionCollection(), "burnThisCD" );
		
		
	burnThisCD->plug(menu);
	newCDProperties->plug(menu);
	createISO->plug(menu);
	menu->insertSeparator();
	launchWizard->plug(menu);

	KonqDirPart * part = parentPart();
	if ( part )
		connect(part,SIGNAL(aboutToOpenURL()), this,SLOT(updateActions()));

}
//===========================================
KonqDirPart* CdboKonqyPlugins::parentPart() const{
   // Don't use dynamic_cast in a plugin
	if ( !parent() || !parent()->inherits ( "KonqDirPart" ) ){
		kdError() << "CdboKonqyPlugins: Program error, please report a bug." << endl;
		return 0L;
	}
	return static_cast<KonqDirPart*>( parent() );
}
//===========================================
void CdboKonqyPlugins::updateActions(){
	KonqDirPart * part = parentPart();
	if ( !part )
		return;
	KURL url = part->url();
	enableActions((url.protocol() == "newcd") && (url.prettyURL() != "newcd:/"));
};
//===========================================
void CdboKonqyPlugins::enableActions(bool doIt){
	newCDProperties->setEnabled(doIt);
	createISO->setEnabled(doIt);
	burnThisCD->setEnabled(doIt);
}
//===========================================
void CdboKonqyPlugins::newCDPropertiesSlot(){
	launchCommand("kcmshell cdboisoopt cdboburnopt");
}
//===========================================
void CdboKonqyPlugins::createISOSlot(){
	KonqDirPart * part = parentPart();
	if ( !part )
		return;
	KURL url = part->url();
	QString urlStr = url.prettyURL();
	KRun::shellQuote(urlStr);
	launchCommand("cdbodialogloader --dialog-type CdboDataCDDialog --from-url " + urlStr + " --iso-image-only");
}
//===========================================
void CdboKonqyPlugins::burnThisCDSlot(){
	KonqDirPart * part = parentPart();
	if ( !part )
		return;
	KURL url = part->url();
	QString urlStr = url.prettyURL();
	KRun::shellQuote(urlStr);
	launchCommand("cdbodialogloader --dialog-type CdboDataCDDialog --from-url " + urlStr);
}
//===========================================
void CdboKonqyPlugins::launchWizardSlot(){
	launchCommand("cdbosetuptool --launchWizard");
}
//===========================================
void CdboKonqyPlugins::launchCommand(const QString& command){
	KShellProcess *cdboProc = new KShellProcess();
	*cdboProc << command;
	kdDebug() << "Ready to launch " << command << endl;
	connect(cdboProc, SIGNAL(processExited(KProcess*)), this, SLOT(kcmshellDone(KProcess*)));
	if(!cdboProc->start(KProcess::NotifyOnExit)){
		kdWarning() << "Failed to launch " << command << endl;
		delete cdboProc;
	}
}
//===========================================
void CdboKonqyPlugins::kcmshellDone(KProcess *proc){
	delete proc;
}

