/***************************************************************************
                          CdboDataViewPart.cpp  -  description
                             -------------------
    begin                : Wed Sep 4 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/
#include <qsplitter.h>
#include <qevent.h>
#include <qcursor.h> 

#include <kaboutdata.h>
#include <kaction.h>
#include <kinstance.h>
#include <klocale.h>
#include <kxmlguifactory.h>
#include <kglobal.h>
#include <kdebug.h>
#include <kiconloader.h>
#include <kmessagebox.h>
#include <kurldrag.h>
#include <kconfig.h>
#include <kio/netaccess.h>
#include <kstdguiitem.h>

#include "CdboDataViewPart.h"
#include "CdboDataViewPart.moc"

#include "CdboDataEstimate.h"
#include "CdboViewWidgetBase.h"
#include "CdboDataFoldersView.h"
#include "CdboDataFilesView.h"
#include "CdboDataFolder.h"
#include "CdboFolderDataPropsDialog.h"

#include "CdboDataCDDialog.h"
#include "CdboSelectDriveDialog.h"

#include "CdboVersion.h"

K_EXPORT_COMPONENT_FACTORY( libcdbodataview, CdboDataViewFactory );

//===================================
CdboDataViewPart::CdboDataViewPart(QWidget *parentWidget, const char *widgetName,
                            QObject *parent, const char *name, const QStringList& list)
	: CdboViewPart(parentWidget, widgetName, parent, name, list){
	
	settingTypes = "cdboisoopt cdboburnopt cdbodefaultopt";
	setInstance( CdboDataViewFactory::instance() );
	
	new CdboDataViewBrowserExtension( this );
	
	estimate = new CdboDataEstimate(viewBase()->getEstimateHolder(), "estimate");
	viewBase()->plugEstimate(estimate);
	
	
	
	viewsSpliter = new QSplitter(viewBase()->getListViewHolder(), "compilViewSplitter");

	foldersView = new CdboDataFoldersView(viewsSpliter,"foldersView");	
	filesView = new CdboDataFilesView(viewsSpliter,"filesView");
	filesView->setFocusPolicy(QWidget::WheelFocus);
	
	foldersView->setEstimate(estimate);
	
	viewBase()->plugView(viewsSpliter);
	viewBase()->setFocusProxy(filesView);
	
	KConfig* config = new KConfig("cdbakeovenrc");
	config->setGroup("Settings of CdboDataViewPart");
	viewsSpliter->setSizes(config->readIntListEntry("Sizes of Parts"));
	delete config;
	
	readOptions();
	setupActions();
	
	setXMLFile( "cdbodataviewpartui.rc" );
	
	createImageOnly = false;
	estimate->showStats(true);
	foldersView->removeAll();
	
	connect(  filesView, SIGNAL(dropped (QDropEvent *, QListViewItem *)),
		this, SLOT(dropOcurred(QDropEvent *, QListViewItem *)));
	connect(  foldersView, SIGNAL(dropped (QDropEvent *, QListViewItem *)),
		this, SLOT(dropOcurred(QDropEvent *, QListViewItem *)));
	connect(  filesView, SIGNAL(menuRequested()),
		this, SLOT(filesViewMenuRequested()));
	connect(  foldersView, SIGNAL(menuRequested()),
		this, SLOT(foldersViewMenuRequested()));
						
	connect( foldersView, SIGNAL(folderClicked(CdboDataFolder *)),
		filesView, SLOT(loadFiles( CdboDataFolder *)) );
	connect(  filesView, SIGNAL(selectFolder(QListViewItem *)),
		foldersView, SLOT(slotSelectFile(QListViewItem *)));
	connect(  filesView, SIGNAL(modified()),
		foldersView, SLOT(resetSize()));
	connect(  filesView, SIGNAL(onFolder (CdboDataFolder *)),
		foldersView, SLOT(setParentForDrop (CdboDataFolder *)));

	connect(foldersView, SIGNAL(processStarted()), 
		viewBase(), SLOT(setWaitCursor()));
	connect(foldersView, SIGNAL(processDone()), 
		viewBase(), SLOT(resetCursor()));
	connect(foldersView, SIGNAL(processDone()), 
		this, SLOT(resetStatus()));
	connect(foldersView, SIGNAL(processDone()), 
		this, SLOT(checkUmount()));
		
	connect(foldersView, SIGNAL(rejectedSize(const QString &)), 
		this, SLOT(cannotFitError(const QString &)));
	
	connect(foldersView, SIGNAL(haveStatus(const QString &)), 
		this, SLOT(emitStatus(const QString &)));
		
	connect(  foldersView, SIGNAL(modified()),
						this, SLOT(setModified()));
	connect(  filesView, SIGNAL(modified()),
						this, SLOT(setModified()));
}
//===================================
CdboDataViewPart::~CdboDataViewPart(){
	saveOptions();
}
//===================================
void CdboDataViewPart::setupActions(){

	//BEGIN //////////////////////// copy menu ////////////////////////////////////////////////
	copyActionMenu = new KActionMenu( i18n("DND Menu"), actionCollection(), "dnd menu");
	
	KAction* a = new KAction( i18n( "&Copy Here" ), "editcopy", KShortcut::null(), this,
				SLOT( copyItemsSlot() ), actionCollection(), "copy_action" );
	copyActionMenu->insert(a);
	a = new KAction( i18n( "&Move Here" ), KShortcut::null(), this, 
				SLOT( moveItemsSlot() ), actionCollection(), "move_action" );
	copyActionMenu->insert(a);
	
	copyActionMenu->insert( actionCollection()->action("separator") );

	a = new KAction( i18n( "C&ancel" ), "cancel", KShortcut::null(), this,
		SLOT( dummySlot() ), actionCollection(), "cancel_action" );
	copyActionMenu->insert(a);
	//END //////////////////////// copy menu ////////////////////////////////////////////////
	
	foldersView->setupActions(actionCollection());
	filesView->setupActions(actionCollection());
	
	a = actionCollection()->action("burnThisCD" );
	foldersView->plugMenuAction(a);
	filesView->plugMenuAction(a);
	
	a = new KAction( i18n( "Save to File (Make ISO)" ), "cdimage", KShortcut(CTRL+Key_I),
		this, SLOT( createISO() ), actionCollection(), "createISO" );
	foldersView->plugMenuAction(a);
	filesView->plugMenuAction(a);
	
	a = actionCollection()->action("newCDProperties" );
	foldersView->plugMenuAction(a);
	filesView->plugMenuAction(a);
	
	a = actionCollection()->action("separator" );
	foldersView->plugMenuAction(a);
	filesView->plugMenuAction(a);
	
	a = new KAction( i18n("Properties..."), KShortcut(ALT+Key_Return), this,
				SLOT(slotProperties()), actionCollection(), "properties" );
	foldersView->plugMenuAction(a);
	filesView->plugMenuAction(a);
	
	a = new KAction( i18n("Load Previous Session..."), KShortcut::null(), this,
				SLOT(loadSession()), actionCollection(), "loadSession" );
	
	filesView->prependMenuAction(actionCollection()->action("mkdir" ));
	
}
//========================
void CdboDataViewPart::readOptions(){
	filesView->updateSettings();
	foldersView->updateSettings();
}
//========================
void CdboDataViewPart::saveOptions(){
	KConfig* config = new KConfig("cdbakeovenrc");
	config->setGroup("Settings of CdboDataViewPart");
	config->writeEntry("Sizes of Parts", viewsSpliter->sizes());
	config->sync();
	delete config;
}
//========================
void CdboDataViewPart::filesViewMenuRequested(){
	if(filesView->selectedFolders()->count() == 0 && filesView->selectedFiles()->count() == 0){
		actionCollection()->action("properties")->setEnabled(false);
	}
	else{
		actionCollection()->action("properties")->setEnabled(true);
	}
	foldersViewActive = false;
}
//========================
void CdboDataViewPart::foldersViewMenuRequested(){
	if(!foldersView->selectedItem() || foldersView->selectedItem() == foldersView->firstChild()){
		actionCollection()->action("properties")->setEnabled(false);
	}
	else{
		actionCollection()->action("properties")->setEnabled(true);
	}
	foldersViewActive = true;
}
	
//========================
void CdboDataViewPart::createCD(){
	startProcess();
}
//========================
void CdboDataViewPart::createISO(){
	createImageOnly = true;
	createCDSlot();
}
//========================
void CdboDataViewPart::startProcess(){
	if(!foldersView->generateMappingsString())
		return;

	if(!checkDirSize())
		return;

	KConfig *config = new KConfig("cdbakeovenrc");
	config->setGroup("Default Settings");
	QString fn = config->readEntry("Tmp Dir", "/tmp/");

	config->setGroup("ISO Settings");
	fn = fn + config->readEntry("Custom ISO Name", "cdboImage_<DATE>.iso");
	delete config;
	
	if(!checkOverwrite(fn))
		return;

	CdboDataCDDialog *ncd = new CdboDataCDDialog(viewBase(), "ncd");
	ncd->setImageOnly(createImageOnly);
	
	ncd->setPathspecName(foldersView->getPathspecName());
	ncd->setHidehfsName(foldersView->getHidehfsName());
	ncd->setHiderrName(foldersView->getHiderrName());
	ncd->setHidejoiName(foldersView->getHidejoiName());
	
	ncd->exec();
	delete ncd;
	foldersView->postCreateCleanUp();
	foldersView->updateSettings();
	createImageOnly = false;
}
//=================================
bool CdboDataViewPart::openFile(){
	KConfig *file = new KConfig(m_file);
	foldersView->loadFile(file);
	delete file;
	setModified(false);
	return true;
}
//=================================
void CdboDataViewPart::saveToFile(KConfig *file){
	KConfig *config = new KConfig("cdbakeovenrc");
	config->setGroup("ISO Settings");
	file->writeEntry("Name of CD", config->readEntry("Custom ISO Name", "cdboImage_<DATE>.iso"));
	delete config;
	
	file->writeEntry("Total Size", foldersView->getTotalSize());
	foldersView->persist(file);
}
//=================================
bool CdboDataViewPart::isEmpty(){
	return (filesView->childCount() == 0 && foldersView->firstChild()->childCount() == 0);
}
//=================================
int CdboDataViewPart::getSize(){
	 return (int)(foldersView->getTotalSize() / 1024 / 1024);
}
//=========================================================
void CdboDataViewPart::loadSession(){
	int doIt = KMessageBox::warningContinueCancel(viewBase(),
		i18n("All items currently on the list will be removed before session load. "
			"Continue to load session?"),
		i18n("Continue to load?"), KStdGuiItem::cont(), "Remove All Before Session Load");
				
	if(doIt == KMessageBox::Cancel)
		return;
			
	foldersView->removeAll();
		
	CdboSelectDriveDialog *driveInfoDlg = new CdboSelectDriveDialog(viewBase(), "propDlg" );
	if(!driveInfoDlg->isReady())
		proposeDeviceDetection();
	else{
		connect( driveInfoDlg, SIGNAL( load(QString) ), this , SLOT( loadSession(QString) ) );
		driveInfoDlg->exec();
	}
	delete driveInfoDlg;

}
//=========================================================
void CdboDataViewPart::loadSession(QString drive){
	lastDrive = drive;
	if(!mountDevice(lastDrive, needToUmount))
		return;
	
	
	foldersView->importSession(lastDrive);
}
//=========================================================
void CdboDataViewPart::checkUmount(){
	if(needToUmount)
		umountDevice(lastDrive);
}
//=====================================
void CdboDataViewPart::slotProperties(){
	if(foldersViewActive){
		CdboDataFolder *ftf = dynamic_cast<CdboDataFolder *>(foldersView->currentItem());
		if (!ftf)
			return;
		new CdboFolderDataPropsDialog(ftf, viewBase(), "propDlg" );
	}
	else{
		filesView->createSelectedLists();
		CdboDataFoldersList*  folders = filesView->selectedFolders();
		CdboFileEntriesList* files = filesView->selectedFiles();
		CdboFolderDataPropsDialog *propDlg;

		if(folders->count() == 0 && files->count() == 0){
			return;
		}
		else if(folders->count() + files->count() == 1){
			if(files->count() > 0){
				propDlg = new CdboFolderDataPropsDialog(filesView->getParentFolder(), files->at(0), viewBase(), "propDlg" );
					//fakeFolder->getEntry(fItem->text(0).stripWhiteSpace()), this, "propDlg" );
			}
			else{
				propDlg = new CdboFolderDataPropsDialog(folders->at(0), viewBase(), "propDlg" );
			}
		}
		else{
			propDlg = new CdboFolderDataPropsDialog(filesView->getParentFolder(), folders, files, viewBase(), "propDlg" );
		}
	}
	filesView->reloadListSlot();
}
//====================================
void CdboDataViewPart::dropOcurred(QDropEvent * e, QListViewItem *){
	viewBase()->setFocus();
	if(e->source() == filesView){
		foldersView->setDragedItem(0);
		copyActionMenu->popup(QCursor::pos());
		e->accept();
		return;
	}
	else if(e->source() == foldersView){
		if(foldersView->getDragedItem() == foldersView->getParentForDrop()){
			e->ignore();
			return;
		}
		copyActionMenu->popup( QCursor::pos());
		e->accept();
		return;
	}
	e->accept();
	foldersView->setDragedItem(0);
	KURL::List urls;
	KURLDrag::decode( e, urls );
	if(urls.count() > 0)
		addItems( urls );
}
//====================================
void CdboDataViewPart::addItems(KURL::List &urls){
	viewBase()->setFocus();
	for(int i = 0; i < (int)urls.count(); i++){
		if(!urls[i].isLocalFile()){//FIXME: handle this better
			localUrlsOnlyError(urls[i].prettyURL());
			urls.remove(urls[i]);
			i--;
		}
		else{
			if(!canAccept(urls[i])){
				break;
			}
		}
	}
}
//=====================================
bool CdboDataViewPart::canAccept(const KURL &url) {
	foldersView->setDragedItem(0);

	bool stopHere;
	if(!readyForOverwrite(url.fileName(), &stopHere))
		return !stopHere;

	if(!foldersView->canAccept(url.path(), stopHere)){
		invalidPathError(url.path());
		return false;
	}
	else
		return !stopHere;
}
//=====================================
bool CdboDataViewPart::moveItemsSlot(){
	
	if(!foldersView->getDragedItem()){//if no folder
		moveSelectedTo(foldersView->getParentForDrop());
		return true;
	}
	
	if(foldersView->getDragedItem() == foldersView->getParentForDrop()){
		KMessageBox::sorry(viewBase(),
				i18n("You cannot move the directory into itself."));
		return true;
	}
	if(foldersView->isParentOf(foldersView->getDragedItem(), foldersView->getParentForDrop())){
		KMessageBox::sorry(viewBase(),
					i18n("You cannot move a parent directory into its child."));
		return false;
	}
	bool cancel;
	if(!readyForOverwrite(foldersView->getDragedItem()->getName(), &cancel))
		return !cancel;

	((CdboDataFolder *)foldersView->getDragedItem()->parent())->subtractFromSize(foldersView->getDragedItem()->getSize());
	foldersView->getDragedItem()->parent()->takeItem(foldersView->getDragedItem());

	foldersView->getParentForDrop()->insertItem(foldersView->getDragedItem());
	foldersView->getParentForDrop()->addToSize(foldersView->getDragedItem()->getSize());

	setModified(true);
	return true;
}
//=====================================
void CdboDataViewPart::moveSelectedTo(CdboDataFolder *targetFolder){
	if(filesView->getParentFolder() == targetFolder){
		KMessageBox::sorry(viewBase(),
			i18n("Cannot move to the same directory."));
		return;
	}
	bool cancel;
	CdboDataFoldersList*  folders = filesView->selectedFolders();
	if(folders->count() > 0){
		for (CdboDataFolder *entry=folders->first(); entry; entry=folders->next()){
			foldersView->setDragedItem(entry);
			if(!moveItemsSlot()){
				return;
			}
		}
	}
	
	CdboFileEntriesList* files = filesView->selectedFiles();
	if(files->count() > 0){
		for (CdboFileEntry *entry=files->first(); entry; entry=files->next()){
			if(!readyForOverwrite(entry->name, &cancel)){
				if(cancel)
					return;
				else
					continue;
			}
			targetFolder->addEntry(entry);
			filesView->getParentFolder()->removeEntry(entry);
		}
	}

	filesView->reloadListSlot();
	setModified(true);
}
//=====================================
bool CdboDataViewPart::copyItemsSlot(){
	if(!foldersView->getDragedItem()){//if no folder
		copySelectedTo(foldersView->getParentForDrop());
		return true;
	}
	
	if(!foldersView->canFit(foldersView->getDragedItem()->getSize() / 1024 / 1024, foldersView->getDragedItem()->getName())){
		foldersView->resetSize();
		return false;
	}

	bool cancel;
	if(!readyForOverwrite(foldersView->getDragedItem()->getName(), &cancel))
		return !cancel;

	foldersView->showProgress(i18n("Copying %1. Please wait...").arg(foldersView->getDragedItem()->getName()), 
		i18n("&Stop"), foldersView->getDragedItem()->getSize() / 1000);
	
	
	CdboDataFolder *newFolder = new CdboDataFolder(foldersView->getDragedItem(), foldersView, foldersView->getProgressDlg());
	CdboDataFolder *newItem;
	for(CdboDataFolder *i=(CdboDataFolder *)foldersView->getDragedItem()->firstChild(); ; i=(CdboDataFolder *)i->nextSibling()){
		if(!i)
			break;
		newItem = new CdboDataFolder(i, newFolder, foldersView->getProgressDlg());
		if(i->childCount() > 0){
			if(!foldersView->copyRow(i, newItem)){
				delete newFolder;
				foldersView->hideProgress();
				return false;
			}
		}
	}
	foldersView->takeItem(newFolder);
	foldersView->getParentForDrop()->insertItem(newFolder);
	foldersView->getParentForDrop()->addToSize(newFolder->getSize());
	
	foldersView->updateProgress(foldersView->getDragedItem()->getSize() / 1000);
	foldersView->hideProgress();
	
	filesView->loadFiles(foldersView->getParentForDrop());
	estimate->updateStats(foldersView->totalFiles - 1, foldersView->totalFiles);

	setModified(true);
	foldersView->resetSize();
	return true;
}
//=============================================
void CdboDataViewPart::copySelectedTo(CdboDataFolder *targetFolder){
	if(filesView->getParentFolder() == targetFolder){
		/*KMessageBox::sorry(this, "You can't drop a directory onto itself");*/
		return;
	}
	
	filesView->setIgnoreReload(true);
	foldersView->setDragedItem(0);
	
	bool cancel;
	CdboDataFoldersList*  folders = filesView->selectedFolders();
	if(folders->count() > 0){
		for (CdboDataFolder *entry=folders->first(); entry; entry=folders->next()){
			foldersView->setDragedItem(entry);
			if(!copyItemsSlot()){
				filesView->setIgnoreReload(false);
				return;
			}
		}
	}

	CdboFileEntriesList* files = filesView->selectedFiles();
	if(files->count() > 0){
		for (CdboFileEntry *entry=files->first(); entry; entry=files->next()){
			if(!readyForOverwrite(entry->name, &cancel)){
				if(cancel)
					return;
				else
					continue;
			}
			if(!foldersView->canFit(entry->size / 1024 / 1024, entry->name)){
				foldersView->resetSize();
				return;
			}
			targetFolder->addEntry(entry);
		}
	}
	
	filesView->setIgnoreReload(false);

	setModified(true);
}
//===================================== 
bool CdboDataViewPart::readyForOverwrite(QString fileName, bool *cancel){
	if(foldersView->getParentForDrop()->containsEntry(fileName)){
		switch( shouldOverwrite(fileName) ) {
			case KMessageBox::No:{
				*cancel = false;
				return false;
			}
			case KMessageBox::Cancel:{
				*cancel = true;
				return false;
			}
			case KMessageBox::Yes:{
				CdboDataFolder *newItem = foldersView->getParentForDrop()->getChild(fileName);
				if(newItem != 0){
					if(newItem == foldersView->getDragedItem()){
						*cancel = false;
						return false;
					}
					if(newItem->isImmutable()){
						int result = KMessageBox::warningYesNo(viewBase(),
							i18n("You cannot replace '%1'. "
								"It was imported from the previous session.").arg(fileName),
							i18n("Cannot Remove %1.").arg(fileName),
							i18n("&Skip"),
							KStdGuiItem::cancel());

						switch(result){
							case KMessageBox::Yes:
								*cancel = false;
								return false;
							case KMessageBox::No:
								*cancel = true;
								return false;
						}
					}
					if(foldersView->getDragedItem() != 0 && 
							foldersView->isParentOf(newItem, foldersView->getDragedItem())){
						KMessageBox::sorry(viewBase(),
									i18n("You cannot replace a parent directory with its child."));
						*cancel = false;
						return false;
					}
					if(newItem ==  filesView->getParentFolder() || 
							foldersView->isParentOf(newItem, filesView->getParentFolder())){
						KMessageBox::sorry(viewBase(),
									i18n("You cannot replace a parent directory with its child."));
						*cancel = false;
						return false;
					}
					
					//removing here
					foldersView->getParentForDrop()->subtractFromSize(newItem->getSize());
					setModified(true);
					delete newItem;
				}//end  of if folder
				
				else{
					if(!foldersView->getParentForDrop()->removeEntry(fileName)){
						int result = KMessageBox::warningYesNo(viewBase(),
							i18n("You cannot replace this item. It was imported from the previous session."),
							i18n("Cannot Remove %1.").arg(fileName),
							i18n("&Skip"),
							KStdGuiItem::cancel());

						switch(result){
							case KMessageBox::Yes:{
								*cancel = false;
								return false;
							}
							case KMessageBox::No:{
								*cancel = true;
								return false;
							}
						}
					}
					setModified(true);
				}
			}//end of YES
		}
	}
	*cancel = false;
	return true;
}
//===================================
KAboutData* CdboDataViewPart::createAboutData(){
	return new KAboutData( "cdbodataviewpart", I18N_NOOP("CD Bake Oven"),
		CDBO_VERSION, I18N_NOOP("Thank you for using CD Bake Oven - CD Burning facility for KDE"),
		KAboutData::License_GPL,
		I18N_NOOP("(c) 2001-2002, Alexy Podolsky"));
}





//===================================
CdboDataViewBrowserExtension::CdboDataViewBrowserExtension( CdboDataViewPart *p )
		: KParts::BrowserExtension( p, "CdboDataViewBrowserExtension" ){
	
	KGlobal::locale()->insertCatalogue("cdbakeoven");
	
}
//===================================
CdboDataViewBrowserExtension::~CdboDataViewBrowserExtension(){
}

