/***************************************************************************
                          CdboDataViewPart.h  -  description
                             -------------------
    begin                : Wed Sep 4 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#ifndef CDBODATAVIEWPART_H
#define CDBODATAVIEWPART_H

#include <kparts/part.h>
#include <kparts/browserextension.h>
#include <kparts/genericfactory.h>

#include "CdboViewPart.h"

class QListViewItem;
class QSplitter;
class KAboutData;

class CdboDataEstimate;
class CdboDataFoldersView;
class CdboDataFilesView;
class CdboDataFolder;

class CdboDataViewPart : public CdboViewPart{
Q_OBJECT
public:
	CdboDataViewPart( QWidget *parentWidget, const char *widgetName,
		QObject *parent, const char *name=0, const QStringList& args = QStringList());
	virtual ~CdboDataViewPart();
    
	static KAboutData* createAboutData();

public slots:
	virtual bool openFile();
	virtual void addItems(KURL::List &urls);
	virtual void readOptions();

protected slots:
	void createISO();
	
	void loadSession();
	void loadSession(QString drive);
	void checkUmount();
	
	bool moveItemsSlot();
	bool copyItemsSlot();
	
	void filesViewMenuRequested();
	void foldersViewMenuRequested();
	
	void slotProperties();
	
	void dropOcurred(QDropEvent * e, QListViewItem *);
	
	void saveOptions();
	
protected:
	void setupActions();
	virtual void createCD();
	void startProcess();
	
	void moveSelectedTo(CdboDataFolder *targetFolder);
	void copySelectedTo(CdboDataFolder *targetFolder);
	bool readyForOverwrite(QString fileName, bool *cancel);
	
	bool canAccept(const KURL &url);
	virtual bool isEmpty();
	virtual int getSize();
	virtual void saveToFile(KConfig *);
	
	bool foldersViewActive;
	
private:
	QSplitter *viewsSpliter;
	CdboDataEstimate *estimate;
	CdboDataFoldersView* foldersView;
	CdboDataFilesView* filesView;
	
	KActionMenu* copyActionMenu;
	
	bool needToUmount;
	QString lastDrive;
	bool createImageOnly;
};

typedef KParts::GenericFactory<CdboDataViewPart> CdboDataViewFactory;

/**
 * A mysterious class, needed to make Konqueror intrgration work.
 */
class CdboDataViewBrowserExtension : public KParts::BrowserExtension{

	Q_OBJECT

public:
	CdboDataViewBrowserExtension( CdboDataViewPart * );
	~CdboDataViewBrowserExtension();
};

#endif

