/***************************************************************************
                          CdboNewcdViewPart.cpp  -  description
                             -------------------
    begin                : Wed Sep 4 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <kaboutdata.h>
#include <kaction.h>
#include <kinstance.h>
#include <klocale.h>
#include <kxmlguifactory.h>
#include <kglobal.h>
#include <kdebug.h>
#include <kiconloader.h>

#include "CdboNewcdViewPart.h"
#include "CdboNewcdViewPart.moc"

#include "CdboFileListView.h"
#include "CdboViewWidgetBase.h"
#include "CdboDataCDDialog.h"

#include "CdboVersion.h"

K_EXPORT_COMPONENT_FACTORY( libcdbonewcdview, CdboNewcdViewFactory );

//===================================
CdboNewcdViewPart::CdboNewcdViewPart(QWidget *parentWidget, const char *widgetName,
                            QObject *parent, const char *name, const QStringList& list)
	: CdboViewPart(parentWidget, widgetName, parent, name, list){
	
	settingTypes = "cdboisoopt cdboburnopt cdbodefaultopt";
	setType(i18n("Data - Konqueror Session"));
	setInstance( CdboNewcdViewFactory::instance() );
	
	new CdboNewcdViewBrowserExtension( this );
	
	listView = new CdboFileListView(viewBase()->getListViewHolder(), "listView");
	
	url = "newcd:/New Data CD/";
	listView->addBranch(url, i18n("New Data CD"), SmallIcon("cdtrack"), true);
	listView->setTopLimitUrl(url);
	
	listView->clearHistory();
	listView->disableLocationPnl();
	viewBase()->plugView(listView);
	
	listView->adoptCopyMenu(i18n( "&Copy to CD" ), i18n("&Move to CD"));
	listView->readOptions();
	setupActions();
	
	setXMLFile( "cdbonewcdviewpartui.rc" );

}
//===================================
CdboNewcdViewPart::~CdboNewcdViewPart(){
	listView->saveOptions();
}
//===================================
void CdboNewcdViewPart::setupActions(){

	listView->plugMenuAction(actionCollection()->action("newCDProperties" ));
	
	listView->plugMenuAction(new KAction( i18n( "Save to File (Make ISO)" ), "cdimage", KShortcut::null(),
		this, SLOT( createISO() ), actionCollection(), "createISO" ));
	
	listView->plugMenuAction(actionCollection()->action("burnThisCD" ));

	actionCollection()->addDocCollection(listView->getActionCollection());
}
//========================
void CdboNewcdViewPart::createCD(){
	CdboDataCDDialog *ncdDialog = new CdboDataCDDialog(viewBase(), "ncd");
	ncdDialog->setURL(url);
	ncdDialog->setImageOnly(false);
	ncdDialog->exec();
	delete ncdDialog;
}
//========================
void CdboNewcdViewPart::createISO(){
	CdboDataCDDialog *ncdDialog = new CdboDataCDDialog(viewBase(), "ncd");
	ncdDialog->setURL(url);
	ncdDialog->setImageOnly(true);
	ncdDialog->exec();
	delete ncdDialog;
}
//===================================
KAboutData* CdboNewcdViewPart::createAboutData(){
	return new KAboutData( "cdbonewcdviewpart", I18N_NOOP("CD Bake Oven"),
		CDBO_VERSION, I18N_NOOP("Thank you for using CD Bake Oven - CD Burning facility for KDE"),
		KAboutData::License_GPL,
		I18N_NOOP("(c) 2001-2002, Alexy Podolsky"));
}

//===================================
//===================================
//===================================
//===================================
//===================================
CdboNewcdViewBrowserExtension::CdboNewcdViewBrowserExtension( CdboNewcdViewPart *p )
		: KParts::BrowserExtension( p, "CdboNewcdViewBrowserExtension" ){
	
	KGlobal::locale()->insertCatalogue("cdbakeoven");
	
}
//===================================
CdboNewcdViewBrowserExtension::~CdboNewcdViewBrowserExtension(){
}

