/***************************************************************************
                          newcd_engine.cpp  -  description
                             -------------------
    begin                : Tue Sep 3 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <sys/stat.h>

#include <qdir.h>
#include <qfile.h>
#include <qregexp.h>

#include <kconfig.h>
#include <kstandarddirs.h>
#include <kio/netaccess.h>
#include <klocale.h>
#include <kmimetype.h>

#include <dcopclient.h>

#include "newcd_engine.h"
#include "newcd_engine.moc"

#define DEFAULT_LIMIT 650 * 1024 * 1024

using namespace KIO;

//===================================================
NewCDProtocolEngine::NewCDProtocolEngine( QObject* parent, const char* name )
		: QObject( parent, name ){

	dcopClient = new DCOPClient();
	if (!dcopClient->attach()){
		debug("Failed to connect to dcop");
	}
	newCDsHome = locateLocal("data", "cdbakeoven/newCDsHome/");
	reloadListFiles();
}
//==============================
NewCDProtocolEngine::~NewCDProtocolEngine(){
	delete dcopClient;
}
//==============================
NewCDProtocolEngine* NewCDProtocolEngine::s_self = 0;
NewCDProtocolEngine* NewCDProtocolEngine::self(){
	
    if ( !s_self )
        s_self = new NewCDProtocolEngine;
    return s_self;
}
//=====================================
void NewCDProtocolEngine::reloadListFiles(){

	QDir thisDir( newCDsHome );

  	if(!thisDir.exists())
		createDefaultLists();
	
	thisDir.setFilter ( QDir::Files);
	thisDir.setNameFilter("*.desktop");
	QStringList list = thisDir.entryList();

	if(list.count() == 0)
		createDefaultLists();

}
//=====================================
void NewCDProtocolEngine::createDefaultLists(){

	createListPointer(i18n("New Data CD"), "", "data");
	//createListPointer(i18n("New Audio CD"), "", "audio");//////////////////////////////////
}
//=====================================
void NewCDProtocolEngine::createListPointer(const QString& name, const QString& sourcePath, const QString& type){

	QByteArray data;
	QDataStream arg(data, IO_WriteOnly);
	arg << name;
	arg << sourcePath;
	arg << newCDsHome;
	arg << type;
	if (!dcopClient->send( "kded", "cdrwatcher",
			"createListPointer(QString,QString,QString,QString)", data) ){
		debug("NewCDProtocolEngine::createListPointer>> Failed to communicate with cdrwatcher");
	}
	else
		debug("NewCDProtocolEngine::createListPointer>> Communicate with cdrwatcher SUCCESSFULLY");

}
//=====================================
QString NewCDProtocolEngine::getSource(const QString& pointerName){
	KConfig *listFile = new KConfig(newCDsHome + pointerName + ".desktop");
	QString source = listFile->readEntry("Source", QString::null);
	delete listFile;
	
	return source;
}
//=====================================
unsigned long NewCDProtocolEngine::getSize(const QString& pointerName){
	KConfig *listFile = new KConfig(newCDsHome + pointerName + ".desktop");
	unsigned long size = listFile->readUnsignedLongNumEntry("Size", 0);
	delete listFile;
		
	return size;
}
//=====================================
void NewCDProtocolEngine::setSize(const QString& pointerName, unsigned long newSize){
	KConfig *listFile = new KConfig(newCDsHome + pointerName + ".desktop");
	listFile->writeEntry("Size", newSize);
	listFile->sync();
	delete listFile;
}
//=====================================
void NewCDProtocolEngine::subtructFromSize(const QString& pointerName, uint newSize){	
	KConfig *listFile = new KConfig(newCDsHome + pointerName + ".desktop");
	unsigned long size = listFile->readUnsignedLongNumEntry("Size", 0);
	if(size < newSize)
		listFile->writeEntry("Size", 0);
	else
		listFile->writeEntry("Size", size - (unsigned long)newSize);
	listFile->sync();
	delete listFile;
}
//=====================================
bool NewCDProtocolEngine::statEntry(KIO::UDSEntry& entry, const QString& pointerName, const QString& path){

//debug("NewCDProtocolEngine::statEntry>> pointerName: <" + pointerName + "> path: <" + path + ">");

	QString source = getSource(pointerName);
	if(source.isEmpty())
		return false;
	
	
	QString slavePath = "newcd:/" + pointerName;
	//QString name = pointerName;
	if(!path.isEmpty()){
		slavePath = slavePath + path;
		if(source.endsWith("/"))
			source = source + path;
		else
			source = source + "/" + path;
		//QStringList list = QStringList::split("/", path, false);
		//name = list[list.count() - 1];
		
	}
		
	QFileInfo d(source);
	if(!d.exists())
		return false;
	//debug("NewCDProtocolEngine::statEntry>> constructing dirEntry, name = <" + name + "> slavePath = <" + slavePath + ">");

	if(d.isDir()){
		createDirEntry(entry, d.fileName(), QFile::encodeName(slavePath), "inode/v-cd-folder");
	}
	else{
		createFileEntry(entry, &d, slavePath);
	}
	
	//debug("NewCDProtocolEngine::statEntry>> returning true");
	return true;
}
//=====================================
bool NewCDProtocolEngine::load(KIO::UDSEntryList& entryList, const QString& pointerName, const QString& path){

//debug("NewCDProtocolEngine::load>> pointerName: <" + pointerName + "> path: <" + path + ">");
	if(!exists(pointerName, path))
		return false;
		
	QString sourceDir = getSource(pointerName);

	QString slavePath = "newcd:/" + pointerName;
	if(!path.isEmpty()){
		if(sourceDir.endsWith("/") && path.startsWith("'/"))
			sourceDir.truncate(sourceDir.length()-1);
		sourceDir = sourceDir + path;
		slavePath = slavePath + path;
	}
	//debug("NewCDProtocolEngine::load>>calling populateRegList, slavePath: <" + slavePath + "> sourceDir: <" + sourceDir + ">");
	
	populateRegList(&entryList, sourceDir, slavePath);
	
	//debug("NewCDProtocolEngine::load>>returning true");
	return true;
}
//=====================================
bool NewCDProtocolEngine::populateRootList(KIO::UDSEntryList& rootList){
//debug("NewCDProtocolEngine::populateRootList>> entered");

	QDir thisDir( newCDsHome );

	if(!thisDir.exists())
		createDefaultLists();
	
	thisDir.setFilter ( QDir::Files);
	thisDir.setNameFilter("*.desktop");
	QStringList list = thisDir.entryList();

	if(list.count() == 0){
		createDefaultLists();
		thisDir.setFilter ( QDir::Files);
		thisDir.setNameFilter("*.desktop");
		list = thisDir.entryList();
	}

	KConfig *pointer;
	QString source;
	
	for(unsigned int i = 0; i < list.count(); i++){
		pointer = new KConfig(newCDsHome + list[i]);
		source = pointer->readEntry("Source", QString::null);
		if(source.isEmpty())
			continue;
		rootList.append(createDirEntry(pointer->readEntry("Name"), QFile::encodeName("newcd:/" + pointer->readEntry("Name")),
				 "inode/new-cd-list"));
	}
	return true;
}

//===========================================================
void NewCDProtocolEngine::populateRegList(KIO::UDSEntryList* entryList, QString url, QString slavePath){

//debug("NewCDProtocolEngine::populateRegList>> slavePath: <" + slavePath + ">, url: <" + url + ">");
	QDir thisDir( url ); 
  	QStringList filesList;
		
	thisDir.setFilter (QDir::All|QDir::Hidden|QDir::System);
	
	filesList = thisDir.entryList();
	filesList.remove(".");
	filesList.remove("..");
	
	QFileInfo d;

	for ( int i = 0; i < (int)filesList.count(); ++i ){
		d.setFile(url + filesList[i]);
		if(d.isDir()){
			//debug("NewCDProtocolEngine::populateRegList>> gotDir: <" + d.fileName() + ">");
			if(!slavePath.endsWith("/")){
				entryList->append(createDirEntry(d.fileName(), QFile::encodeName(slavePath + "/" + d.fileName()),
					"inode/v-cd-folder"));
			}
			else{
				entryList->append(createDirEntry(d.fileName(), QFile::encodeName(slavePath + d.fileName()),
					"inode/v-cd-folder"));
			}
		}
		else{
			//debug("NewCDProtocolEngine::populateRegList>> got File");
			entryList->append(createFileEntry(&d, slavePath));
		}
	}

}

//=====================================
bool NewCDProtocolEngine::exists(const QString& pointerName, const QString& path){
//debug("NewCDProtocolEngine::exists>> pointerName: <" + pointerName + "> path: <" + path + ">");
	QString source = getSource(pointerName);
	if(source.isEmpty())
		return false;

	QFileInfo d(source + path);
	return d.exists ();
}
//=====================================
bool NewCDProtocolEngine::mkdir(const QString& pointerName, const QString& path){
//debug("NewCDProtocolEngine::mkdir>> pointerName: <" + pointerName + "> path: <" + path + ">");
	QString source = getSource(pointerName);
	if(source.isEmpty())
		return false;
		
	QDir dir;
	
	bool success = dir.mkdir(source + path);
		
	return success;
}
//=====================================
bool NewCDProtocolEngine::remove(const QString& pointerName, const QString& path, bool isfile){
//debug("NewCDProtocolEngine::remove>> pointerName: <" + pointerName + "> path: <" + path + ">");
	QString source = getSource(pointerName);
	if(source.isEmpty())
		return false;
		
	QDir dir;
	if(isfile){
		QFileInfo d(source + path);
		uint newSize = d.size();
		if(dir.remove(source + path)){
			subtructFromSize(pointerName, newSize);
			return true;
		}
		else
			return false;
	}
	else
		return dir.rmdir(source + path);
}
//=====================================
bool NewCDProtocolEngine::getRedirect(const QString& pointerName, const QString& path, QString& redirectUrl){
//debug("NewCDProtocolEngine::getRedirect>> pointerName: <" + pointerName + "> path: <" + path + ">");
	QString source = getSource(pointerName);
	if(source.isEmpty())
		return false;
	
	if(path.startsWith("/")){
		QString n = path;
		n = n.remove(0,1);
		redirectUrl = source + n;
	}
	else
		redirectUrl = source + path;
	return true;
}
//=====================================
bool NewCDProtocolEngine::canFit(const QString& pointerName, uint fileSize){
	unsigned long currSize = getSize(pointerName);
	unsigned long newSize = currSize + fileSize;
	
	/*debug("NewCDProtocolEngine::canAdd>>currSize: " + QString::number(currSize) + " fileSize: " + QString::number(fileSize));
	debug("NewCDProtocolEngine::canAdd>>newSize: " + QString::number(newSize) + 
					" DEFAULT_LIMIT: " + QString::number(DEFAULT_LIMIT));
	*/
	if(newSize < DEFAULT_LIMIT){//FIXME: get from cd
		setSize(pointerName, newSize);
		return true;
	}
	else
		return false;
}
//=======================================
KIO::UDSEntry NewCDProtocolEngine::createFileEntry(QFileInfo *d, QString slavePath){
	KIO::UDSEntry entry;
	createFileEntry(entry, d, slavePath);
	return entry;
}
//=======================================
void NewCDProtocolEngine::createFileEntry(KIO::UDSEntry &entry, QFileInfo *d, QString slavePath){
//debug("NewCDProtocolEngine::createFileEntry>> slavePath: <" + slavePath + ">");
	QString name = d->fileName();
	long size = (long)d->size();
	
	KMimeType::Ptr mimetype = KMimeType::findByPath(d->filePath());
	QString mime = mimetype->name();
	
	QString pointsTo = "";
	if(d->isSymLink())
		pointsTo = d->readLink();
		
		
		
	static QDateTime beginningOfTimes(QDate(1970,1,1),QTime(1,0));
	QDateTime lastModified = d->lastModified();
	time_t time=beginningOfTimes.secsTo(lastModified);
		
		
	if(!slavePath.endsWith("/"))
		return createFileEntry(entry, name, QFile::encodeName(slavePath + "/" + name), pointsTo, mime, size, time);
	else
		return createFileEntry(entry, name, QFile::encodeName(slavePath + name), pointsTo, mime, size, time);
}
//=====================================
KIO::UDSEntry NewCDProtocolEngine::createDirEntry(const QString& name, 
		const QString& url, const QString& mime, const long size, time_t time){
	KIO::UDSEntry entry;
	createDirEntry(entry, name, url, mime, size, time);
	return entry;
}
//=====================================
void NewCDProtocolEngine::createDirEntry(KIO::UDSEntry& entry, const QString& name, 
		const QString& url, const QString& mime, const long size, time_t time){
	/*debug("NewCDProtocolEngine::createDirEntry>> constructing dirEntry, name = <" +
	name + ">, slavePath = <" + url + ">, mime = <" +
	mime + ">, size = <" + QString::number(size) + ">");*/
	
	entry.clear();
	addAtom(entry, KIO::UDS_NAME, 0, name);
	addAtom(entry, KIO::UDS_FILE_TYPE, S_IFDIR);
	addAtom(entry, KIO::UDS_ACCESS, 0500);
	addAtom(entry, KIO::UDS_MODIFICATION_TIME,time);
	addAtom(entry, KIO::UDS_MIME_TYPE, 0, mime);
	addAtom(entry, KIO::UDS_URL, 0, url);
	addAtom(entry, KIO::UDS_SIZE, size);
	addAtom(entry, KIO::UDS_GUESSED_MIME_TYPE, 0, "application/octet-stream");
}
//=====================================
KIO::UDSEntry NewCDProtocolEngine::createFileEntry(const QString& name,
		const QString& url, const QString& pointsTo, const QString& mime, const long size, time_t time){

	KIO::UDSEntry entry;
	createFileEntry(entry, name, url, pointsTo, mime, size, time);
	return entry;
}
//=====================================
void NewCDProtocolEngine::createFileEntry(KIO::UDSEntry& entry, const QString& name,
		const QString& url, const QString& pointsTo, const QString& mime, const long size, time_t time){
		
/*debug("NewCDProtocolEngine::createFileEntry>> constructing fileEntry, name = <" +
	name + ">, slavePath = <" + url + ">, mime = <" +
	mime + ">, pointsTo = <" + pointsTo + ">, size = <" + QString::number(size) + ">" );*/
	
	entry.clear();
	addAtom(entry, KIO::UDS_NAME, 0, name);
	addAtom(entry, KIO::UDS_FILE_TYPE, S_IFREG);
	addAtom(entry, KIO::UDS_URL, 0, url);
	addAtom(entry, KIO::UDS_ACCESS, 0500);
	addAtom(entry, KIO::UDS_MODIFICATION_TIME,time);
	addAtom(entry, KIO::UDS_MIME_TYPE, 0, mime);
	addAtom(entry, KIO::UDS_SIZE, size);
	addAtom(entry, KIO::UDS_LINK_DEST, 0, pointsTo);
	addAtom(entry, KIO::UDS_GUESSED_MIME_TYPE, 0, "application/octet-stream");
}
//=====================================
void NewCDProtocolEngine::addAtom(KIO::UDSEntry& entry, unsigned int ID, long l, const QString& s){
	KIO::UDSAtom atom;
	atom.m_uds = ID;
	atom.m_long = l;
	atom.m_str = s;
	entry.append(atom);
}


//=========================================
void NewCDProtocolEngine::debug(QString mess, QString fname){
	return;
	if(fname.isEmpty())
		fname = "/home/alexpod/NewCDProtocol";
	QFile file(fname);

	
	if(!file.open(IO_ReadWrite))
		return;

	file.at( file.size() ); 
	QTextStream stream(&file);
	
	QDate today = QDate::currentDate();
	QTime now = QTime::currentTime();
	stream <<  "[" + QString::number(today.month()) +
		":" + QString::number(today.day()) +
		":" + QString::number(now.hour()) +
		":" + QString::number(now.minute()) +
		":" + QString::number(now.second()) + "] ";
		
		
	stream << mess;
	stream << "\n";
	
	
		
	file.close();


}





