/***************************************************************************
                          newcd_engine.h  -  description
                             -------------------
    begin                : Tue Sep 3 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#ifndef NEWCDPROTOCOLENGINE_H
#define NEWCDPROTOCOLENGINE_H

class QFileInfo;
class KConfig;
class DCOPClient;

class NewCDProtocolEngine : public QObject  {
	Q_OBJECT
public: 
	~NewCDProtocolEngine();

	static NewCDProtocolEngine* self();
	
	bool populateRootList(KIO::UDSEntryList& rootList);

	bool statEntry(KIO::UDSEntry& entry, const QString& pointerName, const QString& path);	
	bool load(KIO::UDSEntryList& entryList, const QString& pointerName, const QString& path);
	
	bool mkdir(const QString& pointerName, const QString& path);
	bool remove(const QString& pointerName, const QString& path, bool isfile);
	bool exists(const QString& pointerName, const QString& path);

	bool getRedirect(const QString& pointerName, const QString& path, QString& redirectUrl);
	bool canFit(const QString& pointerName, uint fileSize);
	
	void createFileEntry(KIO::UDSEntry& entry, const QString& name, const QString& url,
			const QString& pointsTo, const QString& mime, const long size = 0, time_t time = 1);
	void createDirEntry(KIO::UDSEntry& entry, const QString& name, const QString& url, 
			const QString& mime, const long size = 0, time_t time = 1);

	
	void debug(QString mess, QString fname = "");
	
	
public slots:
	void reloadListFiles();//////////////////////

protected:
	NewCDProtocolEngine( QObject* parent = 0, const char* name = 0 );
	void createDefaultLists();
	void createListPointer(const QString& name, const QString& sourcePath, const QString& type);
	
	QString getSource(const QString& pointerName);
	unsigned long getSize(const QString& pointerName);
	
	void subtructFromSize(const QString& pointerName, uint newSize);
	void setSize(const QString& pointerName, unsigned long newSize);
	
	void populateRegList(KIO::UDSEntryList* entryList, QString url, QString slavePath);

	void addAtom(KIO::UDSEntry& entry, unsigned int ID, long l, const QString& s = QString::null);
	KIO::UDSEntry createDirEntry(const QString& name, const QString& url, const QString& mime, const long size = 0, time_t time = 1);
	KIO::UDSEntry createFileEntry(const QString& name,	const QString& url, 
				const QString& pointsTo, const QString& mime, const long size, time_t time = 1);
	KIO::UDSEntry createFileEntry(QFileInfo *d, QString slavePath);
	void createFileEntry(KIO::UDSEntry& entry, QFileInfo *d, QString slavePath);


private:
	static NewCDProtocolEngine* s_self;
	QString newCDsHome;
	DCOPClient *dcopClient;
	
};

#endif
