/***************************************************************************
                          CdboAudioListView.cpp  -  description
                             -------------------
    begin                : Sun Nov 18 2001
    copyright            : (C) 2001 by Alex Podolsky
    email                : alexpod@users.sf.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/ 

// QT includes
#include <qdir.h>
#include <qpushbutton.h>
#include <qdragobject.h>
#include <qregexp.h>
#include <qheader.h>
#include <qptrlist.h>

// KDE includes
#include <kurl.h>
#include <kiconloader.h>
#include <klocale.h>
#include <kconfig.h>
#include <kapplication.h>
#include <kio/job.h>
#include <kpropsdlg.h>
#include <kopenwith.h>
#include <kio/jobclasses.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <klistview.h>
#include <kmimemagic.h>
#include <kdirsize.h>
#include <kfiledialog.h>
#include <kmimetype.h>
#include <kaction.h>
#include <kstdaction.h>
#include <kstdaccel.h>
#include <kfilemetainfo.h>
#include <kdebug.h>

// CDBO includes
#include "CdboAudioListView.h"
#include "CdboAudioListView.moc"

#include "CdboAudioEstimate.h"

////////////////////////

class CdboAudioItem : public QListViewItem{

public:
	CdboAudioItem(CdboAudioListView * parent, QString path);
	~CdboAudioItem();
	
	void setFileName(QString n);
	QString getFileName();
	void setFilePath(QString p);
	QString getFilePath();
	void setLength(QString l);
	QString getLength();
	void setType(QString t);
	QString getType();
	void setTrackPos(int trackPos);
	int getTrackPos() { return trackPos; }
	QString getArtist();
	void setArtist(QString l);
	
	virtual void paintCell(QPainter *p, const QColorGroup &cg, int col, int width, int align);
			   
	virtual QString key ( int column, bool ascending ) const;
	
private:
	void init(const QString &path);
	
	int trackPos;
	long size;
	CdboAudioListView * m_parent;
};


//===================================
CdboAudioListView::CdboAudioListView( QWidget *parent, const char *name) :
		KListView(parent, name){

	setAcceptDrops(true);
	setDropVisualizer(true);
	//setItemsMovable(true);
	setSelectionMode(QListView::Extended);

	addColumn(  i18n("Track")  );
	header()->setClickEnabled( false, header()->count() - 1 );
	addColumn(   i18n("Title")  );
	header()->setClickEnabled( false, header()->count() - 1 );
	addColumn(   i18n("Artist") );
	header()->setClickEnabled( false, header()->count() - 1 );
	addColumn(   i18n("Length") );
	header()->setClickEnabled( false, header()->count() - 1 );
	addColumn(   i18n("File Type") );
	header()->setClickEnabled( false, header()->count() - 1 );
	addColumn(   i18n("Location") );
	header()->setClickEnabled( false, header()->count() - 1 );


	//setColumnAlignment(1, AlignRight);
	setColumnAlignment(3, AlignHCenter);

	setColumnWidth(0,45);
	setColumnWidth(1,190);
	setColumnWidth(2,100);
	setColumnWidth(3,60);
	setColumnWidth(4,100);
	setColumnWidth(5,320);
	firstMenuInsert = true;
	setAllColumnsShowFocus(true);
	setFullWidth(true);
	setSorting(0);

	setItemsRenameable(true);
	config = 0;
	updateSettings();
	
	setupActions();
	
	connect( this, SIGNAL(rightButtonPressed(QListViewItem*, const QPoint&, int)),
					 this, SLOT(popupMenuSlot(QListViewItem*, const QPoint&, int)));
	connect(  this, SIGNAL(doubleClicked ( QListViewItem *)),
						this, SLOT(trackDoubleClicked ( QListViewItem *)));
	connect(  this, SIGNAL(selectionChanged ()),
						this, SLOT(checkActions ()));
	
}
//===================================
CdboAudioListView::~CdboAudioListView(){
	delete actionCollection;
}
//===================================
void CdboAudioListView::setupActions(){
	actionCollection = new KActionCollection( this, "CdboAudioListView::actionCollection" );

	actionMenu =new KActionMenu( i18n("CdboAudioListView Menu"), actionCollection, "CdboAudioListView menu");
	
	KAction* a = new KAction( i18n("Preview With..."), KShortcut::null(), this, SLOT( openWithSlot() ), 
			actionCollection, "preview_with" );
	actionMenu->insert(a);
	
	a = new KAction( i18n("Preview"), KShortcut::null(), this, SLOT( previewSlot() ), 
			actionCollection, "preview_embeded" );
	
	
	
	a = new KAction( i18n( "Remove" ), "editdelete", KShortcut(Key_Delete), this,
				SLOT( remove() ), actionCollection, "delete_track" );
	actionMenu->insert(a);
	
	actionMenu->insert(new KActionSeparator( actionCollection, "separator" ));
	
	
	a = new KAction( i18n("Track Properties..."), KShortcut(ALT+Key_Return), this,
				SLOT(slotProperties()), actionCollection, "track_properties" );
	actionMenu->insert(a);
	
	
	new KAction( i18n("Remove All"), KShortcut::null(), this, SLOT( removeAll() ), 
		actionCollection, "delete_all" );
		
	new KAction( i18n("Move Up"), "up", KShortcut(ALT+Key_Up), this, SLOT( moveUp() ), actionCollection, "move_up" );
	new KAction( i18n("Move Down"), "down", KShortcut(ALT+Key_Down), this, SLOT( moveDown() ), actionCollection, "move_down" );
	
	KAction* reloadAction = KStdAction::redisplay( this, SLOT(reload()), actionCollection, "reload" );
	reloadAction->setText( i18n("Reload") );
	reloadAction->setShortcut( KStdAccel::shortcut( KStdAccel::Reload ));
	
	new KAction(i18n("Stop"), "stop", KShortcut(Key_Escape),
		this, SLOT(cancelOperation()), actionCollection, "stop_load_action");
	actionCollection->action( "stop_load_action" )->setEnabled(false);
	
	checkActions();

}
//===========================
void CdboAudioListView::popupMenuSlot(QListViewItem */*item*/, const QPoint &point, int){
	checkActions();
	actionMenu->popup( point);
}
//===========================
void CdboAudioListView::checkActions(){
	if( selectedItems().count() == 0){
		actionCollection->action("delete_track")->setEnabled(false);
		actionCollection->action("preview_with")->setEnabled(false);
		actionCollection->action("track_properties")->setEnabled(false);
		actionCollection->action("preview_embeded")->setEnabled(false);
	}
	else{
		actionCollection->action("delete_track")->setEnabled(true);
		actionCollection->action("preview_with")->setEnabled(true);
		actionCollection->action("track_properties")->setEnabled(true);
		actionCollection->action("preview_embeded")->setEnabled(previewEnabled);
	}
}
//===================================
void CdboAudioListView::plugMenuAction(KAction* action){
	if(firstMenuInsert){
		KActionSeparator *actionSeparator = new KActionSeparator( actionCollection, "separator" );
		actionMenu->insert( actionSeparator,  2);
		firstMenuInsert = false;
	}

	actionMenu->insert(action, 3);
}
//===================================
void CdboAudioListView::disablePreviewAction(bool doIt){
	previewEnabled = !doIt;
	checkActions();
}
//============================================================
void CdboAudioListView::removeAll(){
	clear();
	fList.clear();
	estimate->reset();
	updateSettings();
	emit modified();
	checkActions();
}
//===========================================================
void CdboAudioListView::remove(){
	QPtrList<QListViewItem> list = selectedItems();
	QString length, mType;
	int secs;
	QListViewItem *lastOne = list.getLast();
	
	for(QListViewItem *item = list.first(); item; item = list.next()){

		secs = 0;
		length = ((CdboAudioItem *)item)->getLength();
		if(length != i18n("Unknown")){
			secs = length.left(length.find(":")).toInt() * 60;
			secs += length.right(length.length() - length.find(":") -1).toInt();
		}

		mType = ((CdboAudioItem *)item)->getType();
		if(mType.find("MPEG") != -1)
			estimate->subtract(secs,CdboAudioEstimate::mp3Track);
		else if(mType.find("Ogg") != -1)
			estimate->subtract(secs,CdboAudioEstimate::oggTrack);
		else
			estimate->subtract(secs);


		fList.remove(((CdboAudioItem *)item)->getFilePath());
		if(item == lastOne){
			lastOne = item->nextSibling();
			if(!lastOne)
				lastOne = item->itemAbove();
			if(lastOne)
				lastOne->setSelected(true);
		}
		delete item;
	}
	correctPositions();
	emit modified();
}
//===========================================================
void CdboAudioListView::correctPositions(){
	CdboAudioItem *item = (CdboAudioItem *)firstChild();
	if(!item)
		return;
	for(int pos = 1; item; pos++, item = (CdboAudioItem *)item->nextSibling())
		item->setTrackPos(pos);
}
//===========================================================
void CdboAudioListView::moveDown(){
	CdboAudioItem *item = (CdboAudioItem *)currentItem();
	if(!item)
		return;
	CdboAudioItem *nextItem = (CdboAudioItem *)item->itemBelow();
	if(!nextItem)
		return;
	int pos = item->getTrackPos();
	item->setTrackPos(nextItem->getTrackPos());
	nextItem->setTrackPos(pos);
	sort();
	clearSelection();
	item->setSelected(true);
	ensureItemVisible(item);
}
//===========================================================
void CdboAudioListView::moveUp(){
	CdboAudioItem *item = (CdboAudioItem *)currentItem();
	if(!item)
		return;
	CdboAudioItem *nextItem = (CdboAudioItem *)item->itemAbove();
	if(!nextItem)
		return;
	int pos = item->getTrackPos();
	item->setTrackPos(nextItem->getTrackPos());
	nextItem->setTrackPos(pos);
	sort();
	clearSelection();
	item->setSelected(true);
	ensureItemVisible(item);
}
//============================================
void CdboAudioListView::slotProperties(){

	if (childCount() == 0 || !currentItem()) 
		return;

	clearSelection();
	currentItem()->setSelected(true);
	ensureItemVisible(currentItem());
	KPropertiesDialog *propDlg = new KPropertiesDialog( KURL( ((CdboAudioItem *)currentItem())->getFilePath() ) );
	connect( propDlg, SIGNAL( applied() ), this , SLOT( reload() ) );

}
//---------------------------------------------------------------------------------
void CdboAudioListView::openWithSlot(){

	QList<QListViewItem> selectedList = selectedItems();
	if(selectedList.count() == 0)
		return;
	QStringList list;
	for(CdboAudioItem *item = (CdboAudioItem *)selectedList.first(); item; item = (CdboAudioItem *)selectedList.next())
		list.append(item->getFilePath());

	KFileOpenWithHandler *kfowh = new KFileOpenWithHandler();
	kfowh -> displayOpenWithDialog( KURL::List( list ) );
}
//---------------------------------------------------------------------------------
void CdboAudioListView::previewSlot(){

	QList<QListViewItem> selectedList = selectedItems();
	if(selectedList.count() == 0)
		return;
	QStringList list;
	for(CdboAudioItem *item = (CdboAudioItem *)selectedList.first(); item; item = (CdboAudioItem *)selectedList.next())
		list.append(item->getFilePath());
	KURL::List urls( list );
	emit selectedURLs( urls );
}
//---------------------------------------------------------------------------------------------
void CdboAudioListView::trackDoubleClicked ( QListViewItem *item){
	if(!item)
		return;
	emit selectedURL(KURL( ((CdboAudioItem *)item)->getFilePath() ));
	
	//KApplication::kApplication()->startServiceByDesktopName ( "noatun", ((CdboAudioItem *)item)->getFilePath());
}
//==============================================================
void CdboAudioListView::updateSettings(){
	viewport()->update();
	if(config)
		delete config;
	config = new KConfig("cdbakeovenrc");
}
//================================================
void CdboAudioListView::setEstimate(CdboAudioEstimate *e) {
	if(!e) 
		return;
	estimate = e;
	connect(estimate->getRecalculateBtn(), SIGNAL(clicked ()),
		this, SLOT(reload()));
}
//==============================================================
void CdboAudioListView::reload(){
	KURL::List urls;
	for(int i = 0; i < (int)fList.count(); i++)
		urls.append(KURL(fList[i]));
	
	removeAll();
	emit addItems(urls);
}
//====================================
//====================================
//===== ADDING ITEMS ====================
//===================================

//====================================
bool CdboAudioListView::acceptDrag(QDropEvent* e ) const {
	return QUriDrag::canDecode( e ) &&
				( e->action() == QDropEvent::Copy
				|| e->action() == QDropEvent::Move
				|| e->action() == QDropEvent::Link );
}
//================================
bool CdboAudioListView::canAccept(QString path, bool &stopHere) {
	stopHere = false;
	QFileInfo d( path );
	if(!d.exists() || !d.isReadable()){
		return false;
	}
	if(d.isDir()){
		return pickAudioFromDir(path);
	}
	
	if(isValidType(path))
		stopHere = !tryToAdd(path);
	else
		KMessageBox::sorry(this, i18n("Unable to accept %1, it is not in audio format.").arg(path));
	
	return true;
}
//=========================================================
bool CdboAudioListView::pickAudioFromDir(QString path){
	if(path[path.length()-1] != '/')
		path = path + "/";

	KIO::ListJob * listJob = KIO::listRecursive( KURL(path), false);
	listJob->setName(path.utf8());
	jobs.append(listJob);
	
	emit processStarted();
	estimate->enableRecalcEstBtn(false);
	actionCollection->action( "stop_load_action" )->setEnabled(true);
	
	emit haveStatus(i18n("Retrieving audio files from this directory. Please wait..."));

	connect( listJob, SIGNAL(entries( KIO::Job *, const KIO::UDSEntryList& )),
						SLOT( slotEntries( KIO::Job*, const KIO::UDSEntryList& )));
	connect(listJob, SIGNAL(result ( KIO::Job *)),
			this, SLOT(audioPickerDone(KIO::Job *) ));
				
	return true;
}
//=============================================
bool CdboAudioListView::isValidType(QString path){
	KMimeType::Ptr mimetype = KMimeType::findByPath(path);
	return (mimetype->name().find("x-ogg", 0, false) != -1 ||
				mimetype->name().find("x-mp3", 0, false) != -1 ||
				mimetype->name().find("x-aiff", 0, false) != -1 ||
				mimetype->name().find("x-cda", 0, false) != -1 ||
				mimetype->name().find("audio/basic", 0, false) != -1 ||
				mimetype->name().find("x-wav", 0, false) != -1);
}
//================================
bool CdboAudioListView::tryToAdd(QString path){
	if(fList.contains(path) > 0){
		return true;
	}
	QString trackName, artist, length;
	getMetaInfo(path, trackName, artist, length);

	return addFileItem(path, artist, trackName, length);
}
//===============================================================
bool CdboAudioListView::addFileItem(QString path, QString artist, QString trackName, QString length){

	CdboAudioItem *lastOne = (CdboAudioItem *)lastChild();
	CdboAudioItem *fItem = new CdboAudioItem( this, path);

	if(!lastOne)
		fItem->setTrackPos(1);
	else
		fItem->setTrackPos(lastOne->getTrackPos()+ 1);

	fItem->setFileName( trackName);
	fItem->setLength( length);
	int secs = 0;
	if(length != i18n("Unknown")){
		secs = length.left(length.find(":")).toInt() * 60;
		secs += length.right(length.length() - length.find(":") -1).toInt();
	}
	fItem->setArtist(artist);
	KMimeType::Ptr mimetype = KMimeType::findByPath(path);
	QString mType = mimetype->name();
	fItem->setType( mType);

	bool bb = true;
	if(mType.find("x-mp3", 0, false) != -1)
		bb = estimate->add(secs,CdboAudioEstimate::mp3Track);
	else if(mType.find("x-ogg", 0, false) != -1)
		bb = estimate->add(secs,CdboAudioEstimate::oggTrack);
	else
		bb = estimate->add(secs);
	if(!bb){
		delete fItem;
		emit limitReached();
		return false;
	}

	fItem->setFilePath(path);
	fItem->setPixmap(0,BarIcon(KMimeType::iconForURL(path), KIcon::SizeSmall ));

	fList.append(path);
	ensureItemVisible(fItem);
	if(jobs.isEmpty()){
		clearSelection();
		fItem->setSelected(true);
	}
	emit modified();
	return true;
}
//================================
void CdboAudioListView::getMetaInfo(const QString &path, QString &title, QString &artist, QString &length){

////////////////////////////////////////
	KFileMetaInfo info(path);
	bool validInfo = info.isValid();

	KFileMetaInfoItem infoItem;
	title = i18n("Unknown");
	if(validInfo){
		infoItem = info.item("Title");
		if(infoItem.isValid()){
			title =  infoItem.string();
  			if(title.isNull() || title == "---")
				title = i18n("Unknown");
		}
	}
/////
	artist = i18n("Unknown");
	if(validInfo){
		infoItem = info.item("Artist");
		if(infoItem.isValid()){
			artist =  infoItem.string();
  			if(artist.isNull() || artist == "---")
				artist = i18n("Unknown");
		}
	}
//////
	length = i18n("Unknown");
	if(validInfo){
		infoItem = info.item("Length");
		if(infoItem.isValid()){
			length =  infoItem.string();
  			if(length.isNull() || length == "---")
				length = i18n("Unknown");
		}
	}
}
//=========================================================
void CdboAudioListView::slotEntries( KIO::Job* job, const KIO::UDSEntryList & list ){

	KIO::UDSEntryListConstIterator it = list.begin();
	KIO::UDSEntryListConstIterator end = list.end();
	bool checked = false;
	QString name;

	for (; it != end; ++it) {
		KIO::UDSEntry::ConstIterator it2 = (*it).begin();
		for( ; it2 != (*it).end(); it2++ ) {
			switch( (*it2).m_uds ) {
				case KIO::UDS_NAME:{///FIXME:: handle links
					name = QString::fromUtf8(job->name()) + (*it2).m_str;
					if(isValidType(name)){
						if(!tryToAdd(name)){
							cancelOperation();
							return;
						}
					}
					checked = true;
					break;
				}
				default:
					break;
			}
			if(checked)
				break;
		}
	}
}
//=====================================
void CdboAudioListView::cancelOperation(){
	KIO::Job *jobToKill;
	while(!jobs.isEmpty()){
		jobToKill = jobs.at(0);
		if(jobToKill)
			jobToKill->kill();
		jobs.remove(jobToKill);
	}
	if(lastChild()){
		clearSelection();
		lastChild()->setSelected(true);
	}
	emit processDone();
	estimate->enableRecalcEstBtn(true);
	actionCollection->action( "stop_load_action" )->setEnabled(false);
}
//========================================================
void CdboAudioListView::audioPickerDone(KIO::Job *j){
	jobs.remove(j);
	if(!jobs.isEmpty())
		return;
	
	if(lastChild()){
		clearSelection();
		lastChild()->setSelected(true);
	}
	actionCollection->action( "stop_load_action" )->setEnabled(false);
	emit processDone();
	estimate->enableRecalcEstBtn(true);
}
//=======================================
//=======================================
//=======================================
//=======================================

#define POS_FIELD 0
#define AUDIO_NAME_FIELD 1
#define AUDIO_ARTIST_FIELD 2
#define AUDIO_LENGTH_FIELD 3
#define TYPE_FIELD 4
#define LOCATION_FIELD 5


CdboAudioItem::CdboAudioItem(CdboAudioListView * parent, QString path) : QListViewItem(parent){
	m_parent = parent;
	QFileInfo d( path );
	setText(LOCATION_FIELD, d.dirPath());
	setText(AUDIO_NAME_FIELD, d.fileName());
}
//=======================================
CdboAudioItem::~CdboAudioItem(){
}
//=======================================
void CdboAudioItem::setFileName(QString n) {
	setText(AUDIO_NAME_FIELD,n);
}
//=======================================
void CdboAudioItem::setFilePath(QString p) {
	setText(LOCATION_FIELD,p);
}

//=======================================
QString CdboAudioItem::getArtist() { 
	return text(AUDIO_ARTIST_FIELD);
}
//=======================================
void CdboAudioItem::setArtist(QString l) {
	setText(AUDIO_ARTIST_FIELD,l);
}

//=======================================
QString CdboAudioItem::getFileName() { 
	return text(AUDIO_NAME_FIELD);
}
//=======================================
QString CdboAudioItem::getFilePath() { 
	return text(LOCATION_FIELD);
}
//=======================================
void CdboAudioItem::setLength(QString l) {
	setText(AUDIO_LENGTH_FIELD,l);
}
//=======================================
QString CdboAudioItem::getLength() {
	return text(AUDIO_LENGTH_FIELD);
}
//=======================================
void CdboAudioItem::setType(QString t) {
	setText(TYPE_FIELD,t);
}
//=======================================
QString CdboAudioItem::getType() {
	return text(TYPE_FIELD);
}
//=======================================
void CdboAudioItem::setTrackPos(int trackPos){
	this->trackPos = trackPos;
	if(trackPos > 9)
		setText(POS_FIELD,QString::number(trackPos));
	else
		setText(POS_FIELD,"0" + QString::number(trackPos));
}
//===================================================
void CdboAudioItem::paintCell(QPainter *p, const QColorGroup &cg, int col, int width, int align){
	KConfig* config = m_parent->getConfig();
	config->setGroup("Default Settings");
	if(config->readBoolEntry("Disable Audio Colors", false)){
		QListViewItem::paintCell(p, cg, col, width, align);
		return;
	}

	QColor color;
	QString n = text(TYPE_FIELD).latin1();
	if(n.find("x-mp3") != -1)
		color = config->readColorEntry("MP3 Audio Color", &(cg.base()));
	else if(n.find("x-ogg") != -1)
		color = config->readColorEntry("Ogg Audio Color", &(cg.base()));
	else if(n.find("audio/basic", 0, false) != -1 ||
			n.find("x-aiff", 0, false) != -1 ||
			n.find("x-wav", 0, false) != -1 ||
			n.find("x-cda", 0, false) != -1)
		color = config->readColorEntry("Ready Audio Color", &(cg.base()));
	else
		color = config->readColorEntry("Unknown Audio Color", &(cg.base()));

	QColorGroup mycg(cg);
	mycg.setBrush(QColorGroup::Base, color);

	QListViewItem::paintCell(p, mycg, col, width, align);
}
//=======================================
QString CdboAudioItem::key ( int column, bool) const{
	
	if(column == AUDIO_LENGTH_FIELD){
		QString tmpString;
		tmpString.sprintf( "%08d", (int)size );
		return tmpString;
	}
	return text(column);
}

