/***************************************************************************
                          CdboCddaView.cpp  -  description
                             -------------------
    begin                : Sun Feb 3 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <qheader.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qregexp.h>

#include <klocale.h>
#include <kiconloader.h>
#include <kdebug.h>
#include <kurl.h>
#include <kaction.h>
#include <klistview.h>

#include "CdboCddaView.h"
#include "CdboCddaView.moc"
#include "CdboPlayerPnl.h"

CdboCddaView::CdboCddaView( QWidget* parent,  const char* name, WFlags fl )
    : QFrame( parent, name, fl ){
	layout = new QVBoxLayout( this );
	layout->setSpacing( 0 );
	layout->setMargin( 0 );

	trackView = new KListView( this, "trackView" );
	trackView->addColumn( i18n( "Track" ) );
	trackView->header()->setClickEnabled( FALSE, trackView->header()->count() - 1 );
	trackView->addColumn( i18n( "Length" ) );
	trackView->header()->setClickEnabled( FALSE, trackView->header()->count() - 1 );
	trackView->addColumn( i18n( "Title" ) );
	trackView->header()->setClickEnabled( FALSE, trackView->header()->count() - 1 );

	trackView->setColumnWidth(0,55);
	trackView->setColumnWidth(2,340);
	trackView->setColumnWidth(1,55);
	trackView->setColumnAlignment(1, AlignHCenter);
	trackView->setFullWidth(true);

	layout->addWidget( trackView );

	player = new CdboPlayerPnl( this, "CdboCddaView Player" );
	player->setMinimumSize( QSize( 0, 27 ) );

	layout->addWidget( player );
	renameable = false;
	showingSize = false;
	setupActions();
	player->readOptions();
	connect( trackView, SIGNAL(rightButtonPressed(QListViewItem*, const QPoint&, int)),
		this, SLOT(popupMenuSlot(QListViewItem*, const QPoint&, int)));
	connect(  trackView, SIGNAL(doubleClicked(QListViewItem *)),
		this, SLOT(slotSelectFile(QListViewItem *)));
	connect(  player, SIGNAL(emptyPlayRequested()),
		this, SLOT(previewSlot()));

}
//============================================================
CdboCddaView::~CdboCddaView(){
	player->saveOptions();
}
//===================================
void CdboCddaView::setupActions(){

	actionCollection = new KActionCollection( this, "CdboCddaView::actionCollection" );
	actionMenu =new KActionMenu( i18n("CdboCddaView Menu"), actionCollection, "CdboCddaView menu");
	
	KAction* a = new KAction( i18n("Select All Tracks"), KShortcut::null(), this, SLOT( slotCheckAll() ), 
			actionCollection, "select_all" );
	actionMenu->insert(a);
	
	a = new KAction( i18n("Unselect All Tracks"), KShortcut::null(), this, SLOT( slotUncheckAll() ), 
			actionCollection, "unselect_all" );
	actionMenu->insert(a);
	
	actionMenu->insert(new KActionSeparator( actionCollection, "separator" ));

	player->setupActions(actionCollection);
	actionMenu->insert(actionCollection->action("player_menu" ));
}
//---------------------------------------------------------------------------------
void CdboCddaView::popupMenuSlot(QListViewItem* , const QPoint& point, int){
	if(trackView->childCount() == 0){
		actionCollection->action("unselect_all" )->setEnabled(false);
		actionCollection->action("select_all" )->setEnabled(false);
	}
	else{
		actionCollection->action("unselect_all" )->setEnabled(true);
		actionCollection->action("select_all" )->setEnabled(true);
	}
	actionMenu->popup(point);
}
//============================================================
void CdboCddaView::load(QString tracks, bool uncheck){
	trackView->clear();

	QStringList list = QStringList::split("\n", tracks);

	QCheckListItem *newItem;
	for(unsigned int i = 0; i < list.count(); i++){
		newItem = new QCheckListItem(trackView, list[i].section(";;;", 0, 0),
					QCheckListItem::CheckBox);
		if(showingSize)
			newItem->setText(1, list[i].section(";;;", 2, 2));
		else
			newItem->setText(1, list[i].section(";;;", 4, 4));
		newItem->setText(2, list[i].section(";;;", 1, 1));
		if(uncheck)
			newItem->setOn(false);
		else
			newItem->setOn(true);
		newItem->setText(3, list[i].section(";;;", 3, 3));//invisible column to store URLs
	}

}
//============================================================
void CdboCddaView::setShowSize(){
	showingSize = true;
	trackView->setColumnText(1, i18n("Size"));
}
//============================================================
void CdboCddaView::reload(){
	emit reloadTracks();
}
//============================================================
void CdboCddaView::stop(){
	player->stop();
}
//---------------------------------------------------------------------------------
void CdboCddaView::slotCheckAll(){
	if(trackView->childCount() == 0)
		return;
	for(QCheckListItem *item = (QCheckListItem *)trackView->firstChild(); item; item = (QCheckListItem *)item->nextSibling())
		item->setOn(true);
}
//---------------------------------------------------------------------------------
void CdboCddaView::slotUncheckAll(){
	if(trackView->childCount() == 0)
		return;
	for(QCheckListItem *item = (QCheckListItem *)trackView->firstChild(); item; item = (QCheckListItem *)item->nextSibling())
		item->setOn(false);
}
//---------------------------------------------------------------------------------
bool CdboCddaView::getPickedTracks(QStringList* pickedTracks){
	pickedTracks->clear();
	if(trackView->childCount() == 0)
		return false;

	QString buff, buff1;
	for(QCheckListItem *item = (QCheckListItem *)trackView->firstChild(); item;
												item = (QCheckListItem *)item->nextSibling()){
		if(item->isOn()){
			buff = item->text(0);
			//buff = buff.replace(QRegExp("Track"), "");
			buff1 = item->text(2);
			if(!renameable)
				pickedTracks->append(buff.stripWhiteSpace());
			else if(buff1 != "<Unknown>" && buff1.stripWhiteSpace() != "")
				pickedTracks->append(buff1 + buff.stripWhiteSpace());
			else
				pickedTracks->append("track" + buff.stripWhiteSpace());
		}
	}
	return true;
}
//---------------------------------------------------------------------------------
void CdboCddaView::slotSelectFile(QListViewItem *item){
	if(!item)
		return;
	QString buff = item->text(3);
	KURL url = buff;
	player->playURL(url);
}
//---------------------------------------------------------------------------------
void CdboCddaView::previewSlot(){
	QList<QListViewItem> selectedList = trackView->selectedItems();
	if(selectedList.count() == 0)
		return;
	QStringList list;
	for(QListViewItem *item = selectedList.first(); item; item = selectedList.next())
		list.append(item->text(3));
	KURL::List urls( list );
	player->playURLs( urls );
}
//---------------------------------------------------------------------------------
void CdboCddaView::setHeaderClickEnabled(bool b){
	for(int i =0; i < trackView->header()->count(); i++)
 		trackView->header()->setClickEnabled( b, i);
}

//---------------------------------------------------------------------------------
void CdboCddaView::showLongTitle(){
	trackView->header()->setLabel( 2, i18n("Base File Name - Title"));
}
//---------------------------------------------------------------------------------
void CdboCddaView::setRenameable(){
	trackView->setItemsRenameable(true);
	trackView->setRenameable(0, false);
	trackView->setRenameable(2, true);
	renameable = true;
}
//---------------------------------------------------------------------------------
void CdboCddaView::invalidate(){
	trackView->clear();
	player->stop();
	trackView->viewport()->update();
}
