/***************************************************************************
                          CdboDataFilesView.h  -  description
                             -------------------
    begin                : Sat Dec 1 2001
    copyright            : (C) 2001 by Alex Podolsky
    email                : alexpod@users.sf.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

// FIXME This class must be broken into independant handler and views in order to support multiple view types as in kdiroperator 


#ifndef CDBODATAFILESVIEW_H
#define CDBODATAFILESVIEW_H

#include <qptrstack.h>
#include <klistview.h>
#include <kaction.h>

#include "CdboDataFolder.h"

class QDropEvent;
class KConfig;


class CdboDataFilesView : public KListView  {
friend class BasicFileItem;
friend class BasicFolderItem;
   Q_OBJECT
public:
	CdboDataFilesView( QWidget *parent=0L, const char *name=0L );
	~CdboDataFilesView();
	
	inline QString getSelectedFilePath(){ return dirPath + "/" + selectedItem()->text(0); }
	inline CdboDataFolder * getParentFolder() { return parentFolder; }

	CdboDataFoldersList* selectedFolders(){ return &selectedFoldersList; }
	CdboFileEntriesList* selectedFiles(){ return &selectedEntriesList; }
	
	void setIgnoreReload(bool doIt);
	void setupActions(KActionCollection* ac);
	void plugMenuAction(KAction* action);
	void prependMenuAction(KAction* action);
	
	
public slots:	
	void loadFiles( CdboDataFolder *item );
	void createSelectedLists();
	void reloadListSlot();
	
	void updateSettings();
	void emitModified() { emit modified(); }
	
	void goUp();
	void forward();
	void back();
	
signals:
	void onFolder(CdboDataFolder *);
	void markSelected();
	void selectFolder(QListViewItem *);
	void modified();
	void menuRequested();
	
	void processStarted();
	void processDone();
	void haveStatus(const QString &);

protected slots:
	void openWithSlot();
	
	virtual QDragObject *dragObject();
	void popupMenuSlot(QListViewItem*, const QPoint&, int);
	void remove();
	
	void slotSelectFile(QListViewItem *);
	void startRenameSlot();
	
	void currentModifiedSlot();
	//void checkShowHidden();FIXME
	
protected:
	virtual bool acceptDrag (QDropEvent* event) const;
	
	void reloadNavActions(CdboDataFolder *prevParent, CdboDataFolder *newParent);
	
private:
	CdboDataFolder *parentFolder;
	QPtrStack<CdboDataFolderHolder> backStack;
	QPtrStack<CdboDataFolderHolder> forwardStack;
	CdboDataFolderPtr parentFolderPtr;
	
	QListViewItem *lastSelected;
	
	KActionCollection* actionCollection;
	KActionMenu* actionMenu;
	
	KConfig* config;
	
	bool ignoreReload;
	bool reloadNav;
	CdboDataFoldersList selectedFoldersList;
	CdboFileEntriesList selectedEntriesList;
	
	QString dirPath;
	//bool showHidden;
	
};

#endif
