/***************************************************************************
                          CdboFileListView.h  -  description
                             -------------------
    begin                : Sat Oct 21 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
	 
	 Some code here copied from Kate of kdebase.
	 
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/ 

#ifndef CDBOFILELISTVIEW_H
#define CDBOFILELISTVIEW_H

#include <qsplitter.h>

#include <kdiroperator.h>
#include <kfiletreeview.h>
#include <kbookmarkmanager.h>
#include <kbookmarkmenu.h>

class QTextStream;

class QListViewItem;
class QToolButton;
class QHBox;

class KFileTreeBranch;
class KURLComboBox;
class KHistoryCombo;

class KBookmarkHandler;
class KPopupMenu;

class KConfig;


class CdboDirOperator;
class CdboFileTreeView;

class CdboFileListView : public QSplitter{
  Q_OBJECT
public:
	CdboFileListView(QWidget *parent = 0, const char *name=0);
	~CdboFileListView();
	 
	QString activeURL();
	
	void addBranch(QString path, QString name, const QPixmap& pix, bool open = false, bool dirOnly = true);
	KAction* getAction(const char *name);
	KActionCollection* getActionCollection();
	void plugMenuAction(KAction* action);
	void enableBookmarkAction();
	void enableAddToCDAction();
	
	void readOptions(KConfig* config = 0);
	void saveOptions(KConfig* config = 0);
	 
	void disableLocationPnl();
	void setTopLimitUrl(const QString& url) { topLimitUrl = url; }
	void clearHistory();
	
	void adoptCopyMenu(const QString& copyStr, const QString& moveStr);

signals:
	void addToCD(KURL::List&);
	
protected slots:
	void dirSelected(QListViewItem*);
	void slotSelectUrl(const KURL&);
	void openURL(const QString& url);
	void slotDropped(KURL::List&,KURL&);
	void emitAddToCD(KURL::List& list){ emit addToCD(list); }
	void copySlot();
	void moveSlot();
	void finishedLoadingSlot();
	void cleanUpMoved();
	void slotShowFilterPnl();
	void slotShowLocationPnl();
	
	void mkdir(const KURL&, const QString&, bool);
	void del(const KFileItemList&);
	void slotFilterChange( const QString & nf );
	void btnFilterClick();
	
	void clearPathSLot();
	void cmbPathActivated(const KURL&);
	void cmbPathReturnPressed(const QString&);
	 
protected:
	void setupActions();
	
	CdboDirOperator *filesView;
	CdboFileTreeView *dirsView;
	KFileTreeBranch *currBranch;
	
	
	QHBox* locationBox;
	KURLComboBox *cmbPath;
	QToolButton *btnClearPath;
	
	QHBox* filterBox;
	KHistoryCombo * filter;
	QToolButton *btnFilter;
	
	KActionMenu* copyActionMenu;
	
	KURL::List listToMove;
	KURL moveToUrl;
	KToggleAction* showFilterAction;
	KToggleAction* showLocationAction;
	
	KBookmarkHandler *bookmarkHandler;
	
	QString lastFilter;
	QString topLimitUrl;

};

///////////////////////////////////////

class CdboDirOperator : public KDirOperator{
	Q_OBJECT
public:
	CdboDirOperator(const KURL& urlName = KURL(), QWidget *parent = 0, const char* name = 0);
	virtual ~CdboDirOperator();
	
	void initActioins();
	void removeAction(const char *name);
	void plugMenuAction(KAction* action);
	void prependMenuAction(KAction* action);
	void plugViewMenuAction(KAction* action, int pos = 0);
	void plugBookmarkAction(KAction* bookmarksAction);

	virtual void readConfig( KConfig *, const QString& group = QString::null );
	virtual void writeConfig( KConfig *, const QString& group = QString::null );
	
signals:
	void dropped(KURL::List&,KURL&);
	void addToCD(KURL::List&);
	
public slots:
	void setIconView();
	void setDetailView();
	void stopLoading();
	
protected slots:
	void slotDropped( KFileView*, KURL::List& );
	void addToCD();
	void resetActions();
	
protected:
	QWidget *filesViewMenu;
	KRadioAction* detailesViewAction;
	KRadioAction* iconViewAction;
	bool firstMenuInsert;
	bool bookmarksUsed;
	int menuTopOffset;
};


///////////////////////////////////////

class CdboFileTreeView : public KFileTreeView{
	Q_OBJECT
public:
	CdboFileTreeView(QWidget *parent = 0, const char *name=0);
	~CdboFileTreeView();
	
	void setupActions(KActionCollection *ac);
	void plugMenuAction(KAction* action);
	void prependMenuAction(KAction* action);
	void plugBookmarkAction(KAction* bookmarksAction);

signals:
	void mkdir(const KURL&, const QString&, bool);
	void del(const KFileItemList&);
	void addToCD(KURL::List&);
	
public slots:
	void popupMenuSlot(QListViewItem*, const QPoint& point, int);
	
protected slots:
	void mkdir();
	void deleteSlot();
	void slotProperties();
	void addToCD();

protected:
	KActionMenu* actionMenu;
	KActionCollection *actionCollection;
	bool firstMenuInsert;
	bool bookmarksUsed;
	int menuTopOffset;
};

///////////////////////////////////////

class KBookmarkHandler : public QObject, public KBookmarkOwner
{
    Q_OBJECT

public:
	KBookmarkHandler( CdboFileListView *parent, KPopupMenu *kpopupmenu=0 );
	~KBookmarkHandler();

	// KBookmarkOwner interface:
	virtual void openBookmarkURL( const QString& url ) { emit openURL( url ); }
	virtual QString currentURL() const;

	KPopupMenu *menu() const { return m_menu; }

signals:
	void openURL( const QString& url );

private slots:
	void slotNewBookmark( const QString& text, const QCString& url,
                          const QString& additionalInfo );
	void slotNewFolder( const QString& text, bool open, const QString& additionalInfo );
	void newSeparator();
	void endFolder();

protected:
	virtual void virtual_hook( int id, void* data );

private:
	CdboFileListView *mParent;
	KPopupMenu *m_menu;
	KBookmarkMenu *m_bookmarkMenu;

	QTextStream *m_importStream;

};

#endif
