/***************************************************************************
                          CdboPlayerPnl.cpp  -  description
                             -------------------
    begin                : Wed Jan 30 2002
    copyright            : (C) 2002 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#include <qfile.h>
#include <qframe.h>
#include <qlcdnumber.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qlabel.h>
#include <qtimer.h>

#include <klocale.h>
#include <kdebug.h>
#include <kiconloader.h>
#include <kmimetype.h>
#include <klibloader.h>
#include <kmessagebox.h>
#include <kmediaplayer/player.h>
#include <kurl.h>
#include <kaction.h>
#include <kconfig.h>

#include "CdboPlayerPnl.h"
#include "CdboPlayerPnl.moc"

CdboPlayerPnl::CdboPlayerPnl( QWidget* parent,  const char* name, WFlags fl )
    : QWidget( parent, name, fl )
{
	if ( !name )
		setName( "CdboPlayerPnl" );

	setCaption( i18n( "Cdbo Player")  );
	CdboPlayerPnlLayout = new QHBoxLayout( this );
	CdboPlayerPnlLayout->setSpacing( 0 );
	CdboPlayerPnlLayout->setMargin( 0 );

	cntPnl = new QFrame( this, "cntPnl" );
	cntPnl->setFrameShape( QFrame::StyledPanel );
	cntPnl->setFrameShadow( QFrame::Plain );
	cntPnlLayout = new QHBoxLayout( cntPnl );
	cntPnlLayout->setSpacing( 0 );
	cntPnlLayout->setMargin( 0 );

	playBtn = new QPushButton( cntPnl, "playBtn" );
	playBtn->setMaximumSize( QSize( 50, 25 ) );
	playBtn->setMinimumSize( QSize( 35, 25 ) );
	playBtn->setText( "..." );
	playBtn->setPixmap( BarIcon("player_play", KIcon::SizeSmall ) );
	QToolTip::add(  playBtn, i18n( "Play/Pause" ) );
	cntPnlLayout->addWidget( playBtn );

	stopBtn = new QPushButton( cntPnl, "stopBtn" );
	stopBtn->setMaximumSize( QSize( 50, 25 ) );
	stopBtn->setMinimumSize( QSize( 35, 25 ) );
	stopBtn->setText( "..." );
	stopBtn->setPixmap( BarIcon("player_stop", KIcon::SizeSmall ) );
	QToolTip::add(  stopBtn, i18n( "Stop" ) );
	cntPnlLayout->addWidget( stopBtn );

	backBtn = new QPushButton( cntPnl, "backBtn" );
	backBtn->setMaximumSize( QSize( 50, 25 ) );
	backBtn->setMinimumSize( QSize( 35, 25 ) );
	backBtn->setText( "..." );
	backBtn->setPixmap( BarIcon("player_rew", KIcon::SizeSmall ) );
	QToolTip::add(  backBtn, i18n( "30 Secs Backward" ) );
	cntPnlLayout->addWidget( backBtn );

	forwardBtn = new QPushButton( cntPnl, "forwardBtn" );
	forwardBtn->setMaximumSize( QSize( 50, 25 ) );
	forwardBtn->setMinimumSize( QSize( 35, 25 ) );
	forwardBtn->setText( "..." );
	forwardBtn->setPixmap( BarIcon("player_fwd", KIcon::SizeSmall ) );
	QToolTip::add(  forwardBtn, i18n( "30 Secs Forward" ) );
	cntPnlLayout->addWidget( forwardBtn );


	prevBtn = new QPushButton( cntPnl, "prevBtn" );
	prevBtn->setMaximumSize( QSize( 50, 25 ) );
	prevBtn->setMinimumSize( QSize( 35, 25 ) );
	prevBtn->setText( "..." );
	prevBtn->setPixmap( BarIcon("player_start", KIcon::SizeSmall ) );
	QToolTip::add(  prevBtn, i18n( "Previous" ) );
	cntPnlLayout->addWidget( prevBtn );

	nextBtn = new QPushButton( cntPnl, "nextBtn" );
	nextBtn->setMaximumSize( QSize( 50, 25 ) );
	nextBtn->setMinimumSize( QSize( 35, 25 ) );
	nextBtn->setText( "..." );
	nextBtn->setPixmap( BarIcon("player_end", KIcon::SizeSmall ) );

	QToolTip::add(  nextBtn, i18n( "Next" ) );
	cntPnlLayout->addWidget( nextBtn );
	QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
	cntPnlLayout->addItem( spacer );

	
	
	curLbl = new QLabel( cntPnl, "curLbl" );
	curLbl->setMaximumSize( QSize( 32767, 25 ) );
	curLbl->setFrameShape( QLabel::Box );
	curLbl->setFrameShadow( QLabel::Plain );
	curLbl->setPalette( QPalette( Qt::white, Qt::black) );


	cntPnlLayout->addWidget( curLbl );

	timeLbl = new QLabel( cntPnl, "timeLbl" );
	timeLbl->setMaximumSize( QSize( 32767, 25 ) );
	timeLbl->setFrameShape( QLabel::Box );
	timeLbl->setFrameShadow( QLabel::Plain );
	timeLbl->setPalette( QPalette( Qt::white, Qt::black ) );


	cntPnlLayout->addWidget( timeLbl );
	QSpacerItem* spacer1 = new QSpacerItem( 1, 1, QSizePolicy::Minimum, QSizePolicy::Minimum );
	cntPnlLayout->addItem( spacer1 );

	CdboPlayerPnlLayout->addWidget( cntPnl );
	curLbl->hide();
	timeLbl->hide();
	
	ticker = new QTimer(this);
	playIndex = 0;
	connect( ticker, SIGNAL( timeout() ), this, SLOT( updateTime() ) );
	kaboodle = 0;
	if(loadKaboodle()){
		connect( playBtn, SIGNAL( clicked() ), this, SLOT( play() ) );
		connect( stopBtn, SIGNAL( clicked() ), this, SLOT( stop() ) );
		connect( forwardBtn, SIGNAL( clicked() ), this, SLOT( forward() ) );
		connect( backBtn, SIGNAL( clicked() ), this, SLOT( back() ) );
		connect( prevBtn, SIGNAL( clicked() ), this, SLOT( prev() ) );
		connect( nextBtn, SIGNAL( clicked() ), this, SLOT( next() ) );
	}
}
//---------------------------------------------------------------------------------------------
CdboPlayerPnl::~CdboPlayerPnl(){
	if(kaboodle){
		stop();
		delete kaboodle;
	}
}
//---------------------------------------------------------------------------------------------
bool CdboPlayerPnl::loadKaboodle(){
	KLibFactory *factory = KLibLoader::self()->factory("libkaboodlepart");
	if (!factory){
		kdDebug() << "no lib libkaboodlepart" << endl; 
		KMessageBox::error(this, i18n("Failed to load required lib: %1!").arg("libkaboodlepart"));
		return false;
	}
	QWidget* w = new QWidget();
	kaboodle = static_cast<KMediaPlayer::Player *>(factory->create(w, "KMediaPlayer::Player", "KMediaPlayer::Player"));
	if(!kaboodle){
		kdDebug() << "no part" << endl;
		KMessageBox::error(this, i18n("Failed to load %1!").arg("KMediaPlayer::Player"));
		return false;
	}
	kdDebug() << "Kaboodle loaded" << endl;
	
	connect( kaboodle, SIGNAL( stateChanged(int) ), this, SLOT( stateChangedSlot(int) ) );
	kaboodle->setLooping(false);
	return true;
}

//===================================
void CdboPlayerPnl::readOptions(){
	KConfig *config = new KConfig("cdbakeovenrc");
	config->setGroup("Default Settings");
	showPlayerAction->setChecked(config->readBoolEntry(QString("Show Player In ") + this->name(), true));
	slotShowPlayer();
	loopAction->setChecked(config->readBoolEntry(QString("Loop enabled in Player In ") + this->name(), true));
	delete config;
}
//===================================
void CdboPlayerPnl::saveOptions(){
	KConfig *config = new KConfig("cdbakeovenrc");
	config->setGroup("Default Settings");
	config->writeEntry(QString("Show Player In ") + this->name(), showPlayerAction->isChecked());
	config->writeEntry(QString("Loop enabled in Player In ") + this->name(), loopAction->isChecked());
	config->sync();
	delete config;
}
//===================================
void CdboPlayerPnl::setupActions(KActionCollection* actionCollection){
	
	KActionMenu* playerMenu =new KActionMenu( i18n("Player"), actionCollection, "player_menu");
	loopAction = new KToggleAction(i18n("Loop"), KShortcut::null(),
			this, SLOT(emptySlot()), actionCollection, "loop_tracks_in_player_action");
	playerMenu->insert(loopAction);
		
	showPlayerAction = new KToggleAction(i18n("Enable Player"), KShortcut::null(),
			this, SLOT(slotShowPlayer()), actionCollection, "show_player_action");
	playerMenu->insert(showPlayerAction);
}
//===================================
void CdboPlayerPnl::slotShowPlayer(){
	if(!showPlayerAction->isChecked()){
		exceptRequests = false;
		hide();
		stop();
	}
	else{
		if(kaboodle)
			exceptRequests = true;
		show();
	}
}
//===================================
void CdboPlayerPnl::playURL(const KURL &url){
	if(!exceptRequests)
		return;
	urlsToPlay.clear();
	urlsToPlay.append(url);
	playIndex = 0;
	openURL(url);
}
//===================================
void CdboPlayerPnl::playURLs(KURL::List &urls){
	if(!exceptRequests)
		return;
	urlsToPlay.clear();
	playIndex = 0;
	urlsToPlay = urls;
	openURL(urlsToPlay[playIndex]);
}
//---------------------------------------------------------------------------------------------
bool CdboPlayerPnl::openURL(const KURL &url){
	if(!kaboodle)
		return false;
		
	stoped = true;
	reset();
	bool loaded =  ((KMediaPlayer::PlayerDCOPObject*)kaboodle)->openURL(url);
	if(!loaded)
		return false;
	timeLbl->setText( " 00:00 "  );
	curLbl->setText(url.fileName());
	play();
	return true;
}
//---------------------------------------------------------------------------------------------
void CdboPlayerPnl::play(){
	if(urlsToPlay.count() == 0){
		emit emptyPlayRequested();
		return;
	}
	stoped = false;
	if(kaboodle->state() == KMediaPlayer::Player::Play){
		kaboodle->pause();
		ticker->stop();
		playBtn->setPixmap( BarIcon("player_play", KIcon::SizeSmall ) );
	}
	else{
		kaboodle->play();
		ticker->start(1000);
		playBtn->setPixmap( BarIcon("player_pause", KIcon::SizeSmall ) );
		curLbl->show();
		timeLbl->show();
	}
}
//---------------------------------------------------------------------------------------------
KURL CdboPlayerPnl::currentUrl(){
	return urlsToPlay[playIndex];
}
//---------------------------------------------------------------------------------------------
void CdboPlayerPnl::stop(){
	stoped = true;
	urlsToPlay.clear();
	reset();
}
//---------------------------------------------------------------------------------------------
void CdboPlayerPnl::reset(){
	playBtn->setPixmap( BarIcon("player_play", KIcon::SizeSmall ) );
	
	ticker->stop();
	curLbl->hide();
	timeLbl->hide();
	kaboodle->stop();
}
//---------------------------------------------------------------------------------------------
void CdboPlayerPnl::prev() {
	reset();
	if(urlsToPlay.count() == 0)
		return;
	playIndex--;
	if(playIndex < 0){
		playIndex = urlsToPlay.count() - 1;
	}
	openURL(urlsToPlay[playIndex]);
}
//---------------------------------------------------------------------------------------------
void CdboPlayerPnl::next() {
	reset();
	if(urlsToPlay.count() == 0)
		return;
	playIndex++;
	if(playIndex == (int)urlsToPlay.count()){
		playIndex = 0;
	}
	openURL(urlsToPlay[playIndex]);
}
//---------------------------------------------------------------------------------------------
void CdboPlayerPnl::forward(){
	if(urlsToPlay.count() == 0 || !kaboodle->isSeekable())
		return;
	kaboodle->seek(kaboodle->position() + 30000);
}
//---------------------------------------------------------------------------------------------
void CdboPlayerPnl::back(){
	if(urlsToPlay.count() == 0 || !kaboodle->isSeekable())
		return;
	if(kaboodle->position() > 30000)
		kaboodle->seek(kaboodle->position() - 30000);
	else
		kaboodle->seek(0);
}
//---------------------------------------------------------------------------------------------
void CdboPlayerPnl::stateChangedSlot(int state){
	if(state == KMediaPlayer::Player::Stop && !stoped){
		if(loopAction->isChecked() || playIndex+1 < (int)urlsToPlay.count()){
			next();
		}
		else{
			reset();
		}
	}
}
//---------------------------------------------------------------------------------------------
void CdboPlayerPnl::updateTime(){
	if(kaboodle->state() == KMediaPlayer::Player::Stop || kaboodle->state() == KMediaPlayer::Player::Empty || 
						(kaboodle->isSeekable() && kaboodle->position() == kaboodle->length()) ){
		if(loopAction->isChecked() || playIndex+1 < (int)urlsToPlay.count()){
			stoped = true;
			next();
		}
		else{
			reset();
		}
		return;
	}
	timeLbl->setText(timeString(kaboodle->position()));
}
//---------------------------------------------------------------------------------------------
QString CdboPlayerPnl::timeString(unsigned long time){
	int posSecs = (int)(time / 1000);
	int posSeconds = posSecs % 60;
	int posMinutes = (posSecs - posSeconds) / 60;

	QString result;
	result.sprintf("%.2d:%.2d", posMinutes, posSeconds);
	return result;
}


