/***************************************************************************
                          CdboTracksEditor.h  -  description
                             -------------------
    begin                : Tue Jan 7 2003
    copyright            : (C) 2001 by Alexy Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#ifndef CDBOTRACKSEDITOR_H
#define CDBOTRACKSEDITOR_H

#include "CdboTracksEditorBase.h"

class QListViewItem;
class QRegExpValidator;

class CdboTracksEditor : public CdboTracksEditorBase
{
	Q_OBJECT

public:
	CdboTracksEditor( QWidget* parent = 0, const char* name = 0, bool modal = FALSE, WFlags fl = 0 );
	~CdboTracksEditor();
	
	void loadFileList(QString fileList, const QString &tocName, const QString &delim = ",",
			const QString &title = QString::null, const QString &artist = QString::null);

	bool loadToc(const QString &tocName, const QString &title = QString::null, const QString &artist = QString::null);
	bool isValidToc(const QString &tocName, QString &title, QString &artist);
	bool copyToc(const QString &oldTocName, const QString &newTocName, bool updatePath = false, 
			const QString &title = QString::null, const QString &artist = QString::null);

	bool createDefaultToc(QString fileList, const QString &tocName, const QString &delim = ",", 
			const QString &title = QString::null, const QString &artist = QString::null);

protected slots:
	virtual void accept();
	virtual void selectionChangedSlot(QListViewItem* item);
	virtual void divideSlot();
	virtual void removeSlot();
    virtual void helpSlot();
	virtual void startTimeChangedSlot(const QTime& newTime);

private:
	void init();
	QListViewItem *createEntry(const QString &path, bool withDefaultTrack = true);
	QListViewItem *getEntry(const QString &path);
	void createTrack(QListViewItem *parentItem);
	bool createToc();
	void fixTrackCount();
	
	void createTocHeader(QTextStream &stream, const QString &title = "", const QString &artist = "", const QString &catalogID = "");
	void createTocEntry(QTextStream &stream, const QString &track, const QString &file, const QString &starts = "0", 
		const QString &length = "", const QString &silence = "0:2:0", const QString &pregap = "0:2:0", 
		const QString &title = "", const QString &artist = "", bool copy_protected = false, bool preemphasis = false, const QString &writer = "",
		const QString &composer = "", const QString &arranger = "", const QString &isrc = "", const QString &message = "");
	
	bool loadTocHeader(QStringList &header, QString &title, QString &artist, QString &catalog);
	void loadTocTracks(QStringList &tracks);
	
	void updateLastSelected();
	void initMetaInfo(QListViewItem* item);
	void initTrackInfo(QListViewItem* item);
	bool updateTime(QTimeEdit * edit, QString length);
	void updateTotalTracks();
	void updateAlbum(const QString &title, const QString &artist);
	
	int trackCount;
	int positionTracker;
	
	QRegExpValidator *catValidator;
	
	QString tocFile;
	QString CDTitle;
	QString CDPerformer;
	QString defaultPregap;
	QString defaultSilence;
	
	QListViewItem *lastSelected;
};



#endif // CDBOTRACKSEDITOR_H
